/*
 * VXPLib COMPONENT
 *
 * Module: AboutBox.cpp - Implementation of class CAboutBox;
 *
 * Synopsis: About Box contains legal and version notices
 * about the component.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"

#include "vxplib.h"
#include "AboutBox.h"
#include "String.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAboutBox::CAboutBox()
{
	m_Fade = 50;

	HMODULE hUser32 = GetModuleHandle(_T("USER32.DLL"));
	m_fnSetLayeredWindowAttributes = (FN_SET_LAYERED_WINDOW_ATTRIBUTES)::GetProcAddress(hUser32, "SetLayeredWindowAttributes");

	::CoCreateInstance(CLSID_VXPQuickHTML, NULL, CLSCTX_ALL, IID_IVXPQuickHTML, (void**)&m_pHTML);

	CString src;
	src.LoadHTML(_Module.GetResourceInstance(), IDR_ABOUTBOX);
	BSTR str = src.SysAllocString();
	m_pHTML->SetSourceText(str);
	m_pHTML->put_Transparent(TRUE);
	SysFreeString(str);

	short width, height;
	m_pHTML->get_Width(&width);
	m_pHTML->get_Height(&height);
	m_size.SetSize(width, height);
}

CAboutBox::~CAboutBox()
{
	m_pHTML->Release();
}

LRESULT CAboutBox::OnPaint(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled)
{
	bHandled = FALSE;

	HDC dc = ::GetDC(m_hWnd);
	VARIANT_BOOL success;
	m_pHTML->Draw((long)dc,0,0, &success);
	ReleaseDC(dc);

	HWND hWnd = GetDlgItem(IDOK);
	::InvalidateRect(hWnd, NULL, TRUE);

	return DefWindowProc();
}

LRESULT CAboutBox::OnOK(WORD wNotifyCode, WORD wID, HWND hWndCtrl, BOOL & bHandled)
{
	EndDialog(m_hWnd, 0);
	return 0;
}

LRESULT CAboutBox::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled)
{
	if(m_fnSetLayeredWindowAttributes)
	{
		ModifyStyleEx(0, WS_EX_LAYERED);
		m_fnSetLayeredWindowAttributes(m_hWnd, 0, m_Fade, LWA_ALPHA);
		g_Timer->SetTimer(m_hWnd);
	}

	// Rectifying the window size
	RECT r, client;
	GetWindowRect(&r);
	GetClientRect(&client);

	int screen_x = GetSystemMetrics(SM_CXFULLSCREEN);
	int screen_y = GetSystemMetrics(SM_CYFULLSCREEN);
	int W = m_size.cx + (r.right - r.left) - client.right;
	int H = m_size.cy + (r.bottom - r.top) - client.bottom;
	SetWindowPos(NULL, (screen_x - W) / 2, (screen_y - H) / 2, W, H, SWP_NOZORDER);
	
	short x, y, w, h;
	VARIANT_BOOL result;
	UINT controls[] = {IDC_COMPANY, IDC_ADDRESS, IDC_CONTACTS, IDOK};
	for(int i=0;i<sizeof(controls)/sizeof(UINT);i++)
	{
		m_pHTML->GetControlPos(i + 1, &x, &y, &w, &h, &result);
		if(result)
		{
			HWND hWnd = GetDlgItem(controls[i]);
			if(hWnd)
				::SetWindowPos(hWnd, NULL, x, y, w, h, SWP_NOZORDER);
		}
	}

	return 0;
}

LRESULT CAboutBox::OnDestroy(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	if(m_fnSetLayeredWindowAttributes)
		g_Timer->KillTimer(m_hWnd);
	return 0;
}

LRESULT CAboutBox::OnCancel(WORD wNotifyCode, WORD wID, HWND hWndCtrl, BOOL & bHandled)
{
	EndDialog(m_hWnd, 0);
	return 0;
}

LRESULT CAboutBox::OnVXPTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled)
{
	if(m_fnSetLayeredWindowAttributes && m_Fade < 255)
	{
		m_Fade += 2;
		if(m_Fade > 255)
			ModifyStyleEx(WS_EX_LAYERED, 0);
		else
			m_fnSetLayeredWindowAttributes(m_hWnd, 0, m_Fade, LWA_ALPHA);
	}

	return 0;
}


