/*
 * VXPLib COMPONENT
 *
 * Module: GlobalObject.h - Declaration of some global classes
 * of the component
 *
 * Synopsis: Find here declarations of a few classes whose secondary role
 * is in providing extra functionality and being information holders
 * for other clases and logical objects of the component.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#if !defined(AFX_GLOBALFONT_H__24FF4A03_5562_4EC1_967E_4182BBF42FE8__INCLUDED_)
#define AFX_GLOBALFONT_H__24FF4A03_5562_4EC1_967E_4182BBF42FE8__INCLUDED_

#include "resource.h"       // main symbols

#include "HCollection.h"
#include "String.h"
#include "Size.h"
#include "PathList.h"
#include "TagObject.h"

class CTOControl;

// This class is to encapsulate all properties of
// a gradient filling:
class CGradientInfo
{
public:
	CGradientInfo();
	void Init(CTag * pTag);
	void Reset();

// Properties:

	COLORREF m_GFC;	// Gradient Filling Color; Attribute GFC=#000000

	bool m_bGFCValid; // Indicates whether m_GFC is valid

	// Attribute GRAD={HOR, VERT};
	bool m_bGFCHorizontal; // Contains true, of the gradient
						   // is to be horizontal
};

struct LOGFONTEX:public LOGFONT
{
	LOGFONTEX()
	{
		// COLOR_WINDOWTEXT is to be changed later
		// depending on implementation
		m_TextColor = GetSysColor(COLOR_WINDOWTEXT);
		m_bBkColorValid = false;
	}
	LOGFONT & operator = (LOGFONT & lf)
	{
		memcpy(this, &lf, sizeof(LOGFONT));
		return *this;
	}
	COLORREF m_TextColor;
	COLORREF m_TextBkColor;
	bool m_bBkColorValid;
};

class CFontObject:public HCollection
{
public:
	void GetFont(LOGFONTEX * lf);
	HGDIOBJ SelectFont(HDC dc, bool IsBold, bool IsItalic, bool IsUnderlined, bool IsStriked);
	CFontObject(LOGFONTEX * lf);
	~CFontObject();
private:
	HFONT m_Font;
	LOGFONTEX m_LogFont;
};

interface IVXPFont;

class CGlobalObject
{
public:
	long GetTxtCodePage();

	bool IsItalic();
	// Constructor:
	CGlobalObject();
	// Destructor:
	virtual ~CGlobalObject();

	virtual CSize GetDefIconSize();

	// Document support methods: *****
	virtual CSize GetDocumentSize(CString & FilePath, CString & Param, short Width, short Height){return CSize();}
	
	virtual void DrawDocument(CString & FilePath, CString & Param, HDC dc, short x, short y, short width, short height){};
	// *******************************

	// Controls support:
	virtual CSize GetControlSize(long ID, CString & Name, short Width, short Height, CString & Param){return CSize();}

	// Attribute ID support for tag TXT:
	virtual CString OnGetText(CString & FilePath, CString & Res, long Size, long ID){return CString();};

	virtual void AddError(long ExceptionFlags){};

	HINSTANCE GetDefaultInstance();
	
	// If the second parameter is not null, the file won't be closed,
	// and the handle will be returned through the second parameter.
	CString GetValidFilePath(LPCTSTR FilePath, HANDLE * hFile = NULL);

	short AddFilePath(BSTR FilePath);
	CSize GetTextSize(HDC dc, CString & txt);
	void TextOut(HDC dc, int x, int y, CString & txt);
	void DecrementBold();
	void IncrementBold();

	void DecrementStrike();
	void IncrementStrike();

	void DecrementItalic();
	void IncrementItalic();

	void DecrementUnderline();
	void IncrementUnderline();

	LOGFONTEX GetDefaultLogFont();
	void ResetState();
	void SetPreviousFont();
	void AddFont(LOGFONTEX * lf);
	void GetCurrentFont(LOGFONTEX * lf);
	HGDIOBJ SelectCurrentFont(HDC dc);

	bool m_bNewLine;

	// This is to optimize requests coming through function GetControlAttr:
	CTOControl * m_pLastRequestedControl;
	long m_LastRequestedIndex;

protected:

	void SetDefaultRes(BSTR FilePath, VARIANT_BOOL * succeeded);
	
	IVXPFont * m_pFont; // This object is managed by CVXPQuickHTML only.

	long m_BodyCodePage;	// Default Code Page for the body
	long m_TxtCodePage;		// Default Code Page for tag <TXT>

// Gradient filling support:
	CGradientInfo m_grad;

private:
	HFONT m_DefaultFont;
	CFontObject * m_pFontList;
	LOGFONTEX m_DefaultLogFont;

	// This resource instance is fully managed within CVXPQuickHTML
	// It is yet declared in this class to avoid virtual functions
	// usage to optimize the speed of reading the variable.
	// It is also set to NULL in this constructor, and destroyed
	// in this destructor upon the interface destruction.
	HINSTANCE m_hDefaultRes;

	// Font style counters
	int m_uBoldCounter;
	int m_uItalicCounter;
	int m_uUnderlineCounter;
	int m_uStrikeCounter;
	CPathList * m_pPathList;
};

#endif // !defined(AFX_GLOBALFONT_H__24FF4A03_5562_4EC1_967E_4182BBF42FE8__INCLUDED_)
