/*
 * VXPLib COMPONENT
 *
 * Module: HCollection.cpp - Implementation of class HCollection
 *
 * Synopsis: Class HCollection implements a colection of abstract
 * elements. Any type derived from this class will gain properties of
 * a collection element.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "HCollection.h"

//**************************************************************************

HCollection::HCollection():Prev(0),Next(0)
{
}

//**************************************************************************

HCollection::HCollection(HCollection * Queue):Prev(0),Next(0)
{
	Add(Queue);
}

//**************************************************************************

// This virtual destructor provides each object
// in the collection is destroyed properly

HCollection::~HCollection()
{
	HCollection * A;
	while((A = GetChain()) != this)
		A->Remove();
}

//**************************************************************************

HCollection::connection::connection(HCollection * prev, HCollection * next):Prev(prev), Next(next)
{
}

HCollection::connection HCollection::GetConnection()
{
	return connection(Prev, Next);
}

bool HCollection::SetConnection(HCollection::connection a)
{
// The following two mirrored modules are to reflect the following logics:
// If there were previous and next objects, then when we restore the connection
// they whether both point on THIS object (when we haven't used Cut()), or they must
// point on each other. Otherwise, the sequence of objects in the collection has
// changed between calling HCollection::GetConnection and HCollection::SetConnection
// In this case we cannot assure a correct reconnection of THIS object, and return false.
// For instance, it may happen if we first call GetConnection, then Cut(), then Sort()
// and then SetConnection. If Sort() changes Prev and Next objects with respect to each other,
// then we cannot reconnect THIS object correctly, and return false.

// Verification for the previous object *******

	if(a.Prev)
		if(a.Prev->Next && a.Prev->Next != this)
		{
			if(a.Next)
			{
				if(a.Next->Prev != a.Prev)
					return false;
			}
			else
				return false;
		}

// Verification for the next object ***********

	if(a.Next)
		if(a.Next->Prev && a.Next->Prev != this)
		{
			if(a.Prev)
			{
				if(a.Prev->Next != a.Next)
					return false;
			}
			else
				return false;
		}

//**********************************************

	Prev = a.Prev;
	if(Prev)
		Prev->Next = this;
	Next = a.Next;
	if(Next)
		Next->Prev = this;

	return true;
}

//**************************************************************************

HCollection * HCollection::GetFirst()
{
	HCollection * A = this;
	while(A->Prev)
		A = A->Prev;
	return A;
}

//**************************************************************************

HCollection * HCollection::GetLast()
{
	HCollection * A = this;
	while(A->Next)
		A = A->Next;
	return A;
}

//**************************************************************************

HCollection * HCollection::Add(HCollection * A)
{
	HCollection * B = A->GetFirst();
	HCollection * C = GetLast();
	C->Next = B;
	B->Prev = C;
	return A;
}

//**************************************************************************

HCollection * HCollection::Insert(HCollection * A)
{
	HCollection * B = A->GetFirst();
	if (Next)
	{
		HCollection * C = A->GetLast();
		Next->Prev = C;
		C->Next = Next;
	}
	B->Prev = this;
	Next = B;
	return A;
}

//**************************************************************************

void HCollection::CheckSorted()
{
	HCollection * A = GetChain();
	if (A != this)
	{
		Cut();
		A->Include(this);
	}
}

//**************************************************************************

void HCollection::ForEach(CallFunction func, void * parameter)
{
	HCollection * C = GetFirst();
	while(C)
	{
		func(this, C, parameter); // Caller, receiver, parameter
		C = C->Next;
	}
}

//**************************************************************************

HCollection * HCollection::GetChain()
{
	if (Prev)
		return Prev;
	if (Next)
		return Next;
	return this;
}

//**************************************************************************

HCollection * HCollection::Include(HCollection * A)
{
	HCollection * B = A->GetFirst();
	HCollection * C = B->Next;
	HCollection * First = GetFirst();
	while(B)
	{
		B->Cut();
		HCollection * F = First;
		int Flag = 0;
		while(F->Compare(B))
		{
			if (F->Next)
				F = F->Next;
			else
			{
				F->Next = B;
				B->Prev = F;
				Flag = 1;
				break;
			}
		}
		if (!Flag)
		{
			if (F->Prev)
			{
				B->Prev = F->Prev;
				B->Next = F;
				F->Prev->Next = B;
				F->Prev = B;
			}
			else
			{
				B->Next = F;
				F->Prev = B;
				First = First->Prev;
			}
		}
		if (C)
		{
			B = C;
			C = B->Next;
		}
		else
			break;
	}
	return A;
}

//**************************************************************************

HCollection * HCollection::Cut()
{
	if (Next && Prev)
	{
		Prev->Next = Next;
		Next->Prev = Prev;
	}
	else
		if (Prev)
			Prev->Next = 0;
		else
			if (Next)
				Next->Prev = 0;
	Prev = Next = 0;
	return this;
}

//**************************************************************************

void HCollection::CutAll()
{
	HCollection * A;
	while((A = GetChain()) != this)
		A->Cut();
}

//**************************************************************************

void HCollection::Remove()
{
	delete Cut();
}

//**************************************************************************

void HCollection::FirstLast()
{
	HCollection * A = GetFirst();
	HCollection * B = GetLast();
	if (A != B)
	{
		A->Next->Prev = 0;
		A->Next = 0;
		A->Prev = B;
		B->Next = A;
	}
}

//**************************************************************************

void HCollection::LastFirst()
{
	HCollection * A = GetFirst();
	HCollection * B = GetLast();
	if (A != B)
	{
		B->Prev->Next = 0;
		B->Prev = 0;
		B->Next = A;
		A->Prev = B;
	}
}

//**************************************************************************

void HCollection::MoveFirst()
{
	HCollection * A = GetFirst();
	if (A != this)
		Cut()->Add(A);
}

//**************************************************************************

void HCollection::MoveLast()
{
	HCollection * A = GetLast();
	if (A != this)
		A->Add(Cut());
}

//**************************************************************************

void HCollection::Sort()
{
	HCollection * A = GetFirst();
	HCollection * B = A->GetLim(),*C,*D;
	if (A == B)
		C = A->Next;
	else
		C = A;
	A = B->Cut();
	while(C)
	{
		B = C->GetLim();
		if (B == C)
			C = B->Next;
		D = B->Cut();
		A->Next = D;
		D->Prev = A;
		A = D;
	}
}

//**************************************************************************

void HCollection::Reverse()
{
	HCollection * A = GetFirst();
	while(A)
	{
		HCollection * B = A->Next;
		A->Next = A->Prev;
		A->Prev = B;
		A = A->Prev;
	}
}

//**************************************************************************

HCollection * HCollection::GetObject(int Index)
{
	HCollection * A = GetFirst();
	int K = Index;
	while(K)
	{
		if(A)
			A = A->Next;
		else
			break;
		K --;
	}
	return A;
}

//**************************************************************************

int HCollection::GetIndex()
{
	int K = 0;
	HCollection * A = this;
	while(A->Prev)
	{
		A = A->Prev;
		K ++;
	}
	return K;
}

//**************************************************************************

int HCollection::CountAll()
{
	HCollection * A = GetFirst();
	int K = 0;
	while(A)
	{
		K ++;
		A = A->Next;
	}
	return K;
}

//**************************************************************************

int HCollection::CountBefore()
{
	HCollection * A = this;
	int K = 0;
	while(A->Prev)
	{
		A = A->Prev;
		K ++;
	}
	return K;
}

//**************************************************************************

int HCollection::CountAfter()
{
	HCollection * A = this;
	int K = 0;
	while(A->Next)
	{
		A = A->Next;
		K ++;
	}
	return K;
}

//**************************************************************************

HCollection * HCollection::GetLim()
{
	HCollection * Min = this;
	HCollection * B = Next;
	while(B)
	{
		if (B->Compare(Min))
			Min = B;
		B = B->Next;
	}
	return Min;
}

//**************************************************************************

HCollection * HCollection::operator + (HCollection * A)
{
	Add(A);
	return A->GetLast();
}

//**************************************************************************

HCollection * HCollection::operator += (HCollection * A)
{
	return Add(A);
}

//**************************************************************************

bool HCollection::Compare(HCollection *)
{
	return false;
}

//**************************************************************************
//******************* END **************************************************
//**************************************************************************

