/*
 * VXPLib COMPONENT
 *
 * Module: Picture.h - Image library
 *
 * Synopsis: This file contains declarations of all classes
 * that implement image support (loading and rendering) of
 * some of the most known graphical formats: ICO, CUR, ANI, BMP, GIF,
 * JPG, EMF and WMF.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#if !defined(AFX_PICTURE_H__AD0008F8_EEA4_4816_90A3_8E78D75AE3D7__INCLUDED_)
#define AFX_PICTURE_H__AD0008F8_EEA4_4816_90A3_8E78D75AE3D7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Size.h"
#include "String.h"
#include "GlobalObject.h"
#include "Tag.h"

class CPicture
{
public:
	CPicture(){};
	virtual ~CPicture(){};

	CSize GetSize();
	void SetSize(int cx, int cy);

	// The main procedure to draw part of the picture
	// (0, 0, cx, cy) at the point (x, y).
	virtual void Draw(HDC dc, int x, int y, int cx = 0, int cy = 0){};

	// This procedure populates area{x,y,cx,cy} with the picture
	virtual void Populate(HDC dc, int x, int y, int cx, int cy){};

	// Loads the picture from the file.
	// Supported file formats so far: ICO, ANI, CUR, BMP, GIF, EMF, WMF, JPG(JPEG)

	// Loads the picture from the current resource file

	// If SourceName is NULL, it will try first BG, and then BACKGROUND
	static CPicture * GetObjectFromTag(CTag * pTag, LPCTSTR SourceName, CGlobalObject * globalObject, int cxDesired = 0, int cyDesired = 0);
	static CPicture * GetObjectFromFile(LPCTSTR file, CGlobalObject * globalObject, int cxDesired = 0, int cyDesired = 0);
	static CPicture * GetObjectFromResource(CGlobalObject * globalObject, HINSTANCE hInst, CString Res, int cxDesired, int cyDesired);

protected:
	void ResetSize();
	
	// Populates area of size {cx,cy} with the bitmap
	void PopulateWithBitmap(HDC dc, HBITMAP bmp, int x, int y, int cx, int cy);
	
	// Sets information about bitmap size into m_size
	// If one of the parameters cxDesired or cyDesired is zero,
	// it is taken from the bitmap header and set into m_size
	void SetBitmapDimensions(HBITMAP bmp, int cxDesired, int cyDesired);
	
	// This function is to facilitate reading the file into memory
	// The returned pointer is to be released when no longer needed.
	// If the function fails, the returned value is NULL.

	// Size of the picture
	CSize m_size;

private:
	static int GetResInfo(CString res, CString & ResType, CString & ResName);
};

class CIcon:public CPicture
{
public:
	CIcon(bool IsCursor = false);
	~CIcon();

	virtual bool LoadFromFile(LPCTSTR FileName, int cxDesired, int cyDesired);
	bool LoadFromRes(HINSTANCE hInst, CString ResID, int cxDesired, int cyDesired);

	// The main procedure to draw part of the picture
	// (0, 0, cx, cy) at the point (x, y).
	virtual void Draw(HDC dc, int x, int y, int cx = 0, int cy = 0);

	// This procedure populates area{x,y,cx,cy} with the picture
	virtual void Populate(HDC dc, int x, int y, int cx, int cy);

private:
	
	void UnloadIcon();
	
	HICON m_hIcon;
	ICONINFO m_IconInfo;
	bool m_bCursor;
};

class CBmp:public CPicture
{
public:
	CBmp();
	~CBmp();

	virtual bool LoadFromFile(LPCTSTR FileName, int cxDesired, int cyDesired);
	virtual bool LoadFromRes(HINSTANCE hInst, CString ResID, int cxDesired, int cyDesired);

	// The main procedure to draw part of the picture
	// (0, 0, cx, cy) at the point (x, y).
	virtual void Draw(HDC dc, int x, int y, int cx = 0, int cy = 0);

	// This procedure populates area{x,y,cx,cy} with the picture
	virtual void Populate(HDC dc, int x, int y, int cx, int cy);

private:
	
	void UnloadBitmap();

	HBITMAP m_hBitmap;
};

class CComPicture:public CPicture
{
public:
	CComPicture();
	~CComPicture();

	bool LoadFromFile(LPCTSTR FileName, int cxDesired = 0, int cyDesired = 0);
	bool LoadFromRes(HINSTANCE hInst, CString TypeName, CString ResName, int cxDesired, int cyDesired);

	// The main procedure to draw part of the picture
	// (0, 0, cx, cy) at the point (x, y).
	virtual void Draw(HDC dc, int x, int y, int cx = 0, int cy = 0);

	// This procedure populates area{x,y,cx,cy} with the picture
	virtual void Populate(HDC dc, int x, int y, int cx, int cy);

private:
	
	void UnloadPicture();
	bool LoadFromBuffer(BYTE * buffer, DWORD nSize, int cxDesired, int cyDesired);

	IPicture * m_pPicture;
};

#endif // !defined(AFX_PICTURE_H__AD0008F8_EEA4_4816_90A3_8E78D75AE3D7__INCLUDED_)
