/*
 * VXPLib COMPONENT
 *
 * Module: StdAfx.cpp - Main implementation file of the project
 *
 * Synopsis: Main purpose of this file is assembling the project
 * while using precompiled headers provided by the compiler.
 * It also contains implementations of a few globally used functions
 * and C++ structures and classes used throughout the project.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "VXPLib.h"

#ifdef _ATL_STATIC_REGISTRY

// If your compiler complains about the following
// two includes being obsolete just comment them.

#include <statreg.h>

#if _MSC_VER < 1300	// Ignore this file in VC++.NET
#include <statreg.cpp>
#endif

#endif

#if _MSC_VER < 1300	// Ignore this file in VC++.NET
#include <atlimpl.cpp>
#endif

#include "resource.h"

// It returns true, if the file specified by the full path exists
bool FileExists(LPCTSTR FilePath, HANDLE * hFile)
{
#ifdef _MSLU
	HANDLE file;
	if(g_bNTCompitable)
	{
		CString Unicode_Path(_T("\\\\?\\"));
		Unicode_Path += FilePath;
		file = ::CreateFileW(Unicode_Path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	}
	else
		file = ::CreateFile(FilePath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
#else
	// Here we try to use the Unicode-only function to access files
	CString Unicode_Path(_T("\\\\?\\"));
	Unicode_Path += FilePath;
	HANDLE file = ::CreateFileW(Unicode_Path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
#endif

	if(file == INVALID_HANDLE_VALUE)
		return false;
	if(hFile)
		*hFile = file;
	else
		CloseHandle(file);
	return true;
}

// This function will allocate a buffer large enough to read
// MaxBytes bytes. This buffer must be released when no longer needed.
// It will also set the actual size of the read into 'size'
BYTE * ReadFromFile(LPCTSTR FileName, int & size, long MaxBytes)
{
	size = 0;
	if(!FileName)
		return NULL;

	HANDLE file;
	
	if(!FileExists(FileName, &file))
		return NULL;

	size = GetFileSize(file, NULL);
	if(!size)
	{
		CloseHandle(file);
		return NULL;
	}
	
	if(MaxBytes && size > MaxBytes)
		size = MaxBytes;

	BYTE * buffer = new BYTE[size];
	if(!buffer)
	{
		CloseHandle(file);
		return NULL;
	}
	DWORD ReadBytes;
	if(!ReadFile(file, buffer, size, &ReadBytes, NULL) || ReadBytes != (DWORD)size)
	{
		delete []buffer;
		CloseHandle(file);
		return NULL;
	}
	CloseHandle(file);
	return buffer;
}

bool GetResource(HINSTANCE hInst, LPCTSTR lpName, LPCTSTR lpType, void * pResource, int & nBufSize)
{ 
	HRSRC hResInfo;
	HANDLE hRes;
	LPSTR lpRes	= NULL; 
	int	nLen = 0;
	bool bResult = FALSE;
	// Find the resource
	hResInfo = FindResource(hInst, lpName, lpType);
	if (!hResInfo) 
		return false;
	// Load the resource
	hRes = LoadResource(hInst, hResInfo);
	if (!hRes) 
		return false;

	// Locking the resource:
	lpRes = (char*)LockResource(hRes);

	if(lpRes != NULL)
	{ 
		if(pResource == NULL)
		{
			nBufSize = SizeofResource(hInst, hResInfo);
			bResult = true;
		}
		else
		{
			int ResourceSize = (int)SizeofResource(hInst, hResInfo);
			if(nBufSize > ResourceSize)
				memcpy(pResource, lpRes, ResourceSize);
			else
				memcpy(pResource, lpRes, nBufSize);
			bResult = true;
		} 
	}

	// Unlocking the resource:
	UnlockResource(hRes);

	// Freeing the resource:
	FreeResource(hRes);

	return bResult;
}

BOOL FillGradient(HDC hDC, COLORREF StartColor, COLORREF EndColor, int x1, int y1, int x2, int y2, bool bHorizontal, bool bDirect)
{
	if(g_pGradientFillFunc)
	{
		TRIVERTEX vert[2];
		GRADIENT_RECT gRect;

		vert[0].x      = x1;
		vert[0].y      = y1;
		vert[0].Red    = GetRValue(StartColor)<<8;
		vert[0].Green  = GetGValue(StartColor)<<8;
		vert[0].Blue   = GetBValue(StartColor)<<8;
		vert[0].Alpha  = 0x0000;

		vert[1].x      = x2;
		vert[1].y      = y2;
		vert[1].Red    = GetRValue(EndColor)<<8;
		vert[1].Green  = GetGValue(EndColor)<<8;
		vert[1].Blue   = GetBValue(EndColor)<<8;
		vert[1].Alpha  = 0x0000;

		gRect.UpperLeft  = 0;
		gRect.LowerRight = 1;

		try
		{
			return g_pGradientFillFunc(hDC, vert, 2, &gRect, 1, bHorizontal?GRADIENT_FILL_RECT_H:GRADIENT_FILL_RECT_V);
		}
		catch(...)
		{
		}
	}

// Otherwise, it is Windows 95/NT4, and we need to draw gradient ourselves:

	RECT r2;
	COLORREF color;
	float fStep = ((float)(x2 - x1))/255.0f;

	if(!bHorizontal)
		fStep = ((float)(y2 - y1))/255.0f;

	if(bDirect)
		for(int i = 0; i < 255; i++)
		{
			if(bHorizontal)
				SetRect(&r2, x1 + (int)(i * fStep), y1, x1 + (int)((i + 1)* fStep), y2);
			else
				SetRect(&r2, x1, y1 + (int)(i * fStep), x2, y1 + (int)((i + 1)* fStep));

			color = RGB((GetRValue(EndColor)-GetRValue(StartColor))*((float)i)/255.0f+GetRValue(StartColor),
					(GetGValue(EndColor)-GetGValue(StartColor))*((float)i)/255.0f+GetGValue(StartColor),
					(GetBValue(EndColor)-GetBValue(StartColor))*((float)i)/255.0f+GetBValue(StartColor));

			HBRUSH hBrush = ::CreateSolidBrush(color);
			::FillRect(hDC, &r2, hBrush);
			::DeleteObject(hBrush);
		}
	else
	{
		HDC hMemDC = ::CreateCompatibleDC(hDC);
		HBITMAP hBitmap = ::CreateCompatibleBitmap(hDC, x2 - x1, y2 - y1);
		HGDIOBJ hOldObject = ::SelectObject(hMemDC, hBitmap);
		for (int i = 0; i < 255; i++)
		{
			if(bHorizontal)
				SetRect(&r2, (int)(i * fStep), 0, (int)((i + 1)* fStep), y2);
			else
				SetRect(&r2, 0, (int)(i * fStep), x2, (int)((i + 1)* fStep));

			color = RGB((GetRValue(EndColor) - GetRValue(StartColor)) * ((float)i)/255.0f + GetRValue(StartColor),
						(GetGValue(EndColor) - GetGValue(StartColor)) * ((float)i)/255.0f + GetGValue(StartColor),
						(GetBValue(EndColor) - GetBValue(StartColor)) * ((float)i)/255.0f + GetBValue(StartColor));

			HBRUSH hBrush = ::CreateSolidBrush(color);
			::FillRect(hMemDC, &r2, hBrush);
			::DeleteObject(hBrush);
		}

		::BitBlt(hDC, x1, y1, x2 - x1, y2 - y1, hMemDC, 0, 0, SRCCOPY);
		::SelectObject(hMemDC, hOldObject);
		::DeleteObject(hBitmap);
		::DeleteDC(hMemDC);
	}

	return TRUE;
}

bool g_bNTCompitable = false;
bool g_bWin95orNT4 = false;
VXPOSVersion g_osVersion = osUnknown;

void InitializeVersionInfo()
{
	OSVERSIONINFO info;
	info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if(GetVersionEx(&info))
	{
		if(info.dwPlatformId == VER_PLATFORM_WIN32_NT) // NT4/2000/XP
		{
			g_bNTCompitable = true;
			if(info.dwMajorVersion <= 3) // NT 3.51
				g_osVersion = osWindowsNT3;
			else // NT4/2000/XP
				if(info.dwMajorVersion == 4) // NT 4.0
					g_osVersion = osWindowsNT4;
				else
					if(info.dwMajorVersion == 5) // 2000/XP/2003
					{
						if(!info.dwMinorVersion) // 2000
							g_osVersion = osWindows2000;
						else
							if(info.dwMinorVersion == 1)//XP
								g_osVersion = osWindowsXP;
							else
								if(info.dwMinorVersion == 2)//2003 Server
									g_osVersion = osWindows2003Server;
								else
									if(info.dwMinorVersion > 2)
										g_osVersion = osWindowsLonghorn; // Longhorn
					}
					else
						if(info.dwMajorVersion > 5)
							g_osVersion = osWindowsLonghorn;
		}
		else // 95/98/Me
		{
			if(!info.dwMinorVersion) // 95
				g_osVersion = osWindows95;
			else // 98/Me
				if(info.dwMinorVersion == 10) // 98
					g_osVersion = osWindows98;
				else // Me
					g_osVersion = osWindowsMe;
		}

		if((info.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS && !info.dwMinorVersion) // Windows 95
				|| (g_bNTCompitable && info.dwMajorVersion <= 4))// Windows NT4
			g_bWin95orNT4 = true;
	}
}

HRESULT GetGlobalExitCode(long ExceptionFlags, const CLSID & clsID, const IID ID)
{
	if(!ExceptionFlags)
		return S_OK;

	CString error;
	if(ExceptionFlags & EX_GENERALFAILURE)
	{
		error.LoadString(_Module.GetResourceInstance(), IDS_GENERALFAILURE);
		return AtlReportError(clsID, (LPCOLESTR)(LPCTSTR)error, ID, E_GENERALFAILURE);
	}
	if(ExceptionFlags & EX_OUTPUTEXCESS)
	{
		error.LoadString(_Module.GetResourceInstance(), IDS_OUTPUTEXCESS);
		return AtlReportError(clsID, (LPCOLESTR)(LPCTSTR)error, ID, E_OUTPUTEXCESS);
	}
	if(ExceptionFlags & EX_UNSUPPORTEDCODEPAGE)
	{
		error.LoadString(_Module.GetResourceInstance(), IDS_UNSUPPORTEDCODEPAGE);
		return AtlReportError(clsID, (LPCOLESTR)(LPCTSTR)error, ID, E_UNSUPPORTEDCODEPAGE);
	}
	if(ExceptionFlags & EX_INVALIDHANDLE)
	{
		error.LoadString(_Module.GetResourceInstance(), IDS_INVALIDHANDLE);
		return AtlReportError(clsID, (LPCOLESTR)(LPCTSTR)error, ID, E_INVALIDHANDLE);
	}
	if(ExceptionFlags & EX_MENUFAILURE)
	{
		error.LoadString(_Module.GetResourceInstance(), IDS_MENUFAILURE);
		return AtlReportError(clsID, (LPCOLESTR)(LPCTSTR)error, ID, E_MENUFAILURE);
	}
	if(ExceptionFlags & EX_RECURSIVECALL)
	{
		error.LoadString(_Module.GetResourceInstance(), IDS_RECURSION);
		return AtlReportError(clsID, (LPCOLESTR)(LPCTSTR)error, ID, E_RECURSIVECALL);
	}
	if(ExceptionFlags & EX_TOOLRECURSION)
	{
		error.LoadString(_Module.GetResourceInstance(), IDS_TOOLRECURSION);
		return AtlReportError(clsID, (LPCOLESTR)(LPCTSTR)error, ID, E_TOOLRECURSION);
	}
	if(ExceptionFlags & EX_INDEXOUTOFRANGE)
	{
		error.LoadString(_Module.GetResourceInstance(), IDS_INDEXOUTOFRANGE);
		return AtlReportError(clsID, (LPCOLESTR)(LPCTSTR)error, ID, E_INDEXOUTOFRANGE);
	}

	return S_FALSE;
}

