/*
 * VXPLib COMPONENT
 *
 * Module: StdAfx.h - Main include file of the project
 *
 * Synopsis: Main purpose of this file is assembling the project
 * while using precompiled headers provided by the compiler.
 * It also contains declarations of a few globally used functions
 * and C++ structures and classes used throughout the project.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#if !defined(AFX_STDAFX_H__BF86A97F_0031_41A1_BB1F_0DD2811E80E8__INCLUDED_)
#define AFX_STDAFX_H__BF86A97F_0031_41A1_BB1F_0DD2811E80E8__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define WINVER	0x0400

#define STRICT

#define _ATL_APARTMENT_THREADED

#define WS_EX_LAYERED   0x00080000
#define LWA_ALPHA       0x00000002
#define CS_DROPSHADOW   0x00020000

#ifndef SPI_GETMENUFADE
#define SPI_GETMENUFADE			0x1012
#define SPI_GETUIEFFECTS		0x103E
#define SPI_GETMENUANIMATION	0x1002
#endif

// This constant is defined in NT5 only:
#ifndef COLOR_HOTLIGHT
#define COLOR_HOTLIGHT	26
#endif

#define TOOLTIPCLASS	_T("IVXPTooltipClass_WithShadow")
#define TOOLTIPCLASS2	_T("IVXPTooltipClass_WithoutShadow")

// Exception flags:
#define EX_GENERALFAILURE		1	// Generic or unknown run-time error
#define EX_OUTPUTEXCESS			2	// Too big output size (more than 10.000.000 pixels)
#define EX_UNSUPPORTEDCODEPAGE	4	// The code page is not supported
#define EX_INVALIDHANDLE		8	// Invalid handle passed
#define EX_MENUFAILURE			16	// Failed to retrieve a menu handle, or find a menu item.
#define EX_RECURSIVECALL		32	// Method RecalculateLayout has been called recursively.
#define EX_TOOLRECURSION		64	// Recursive invocation of a IVXPTooltip method.
#define EX_INDEXOUTOFRANGE		128	// Index of the requested tag CONTROL is out of range.

// Error codes for exceptions:
#define S_GENERALFAILURE		40	// Generic or unknown run-time error
#define S_OUTPUTEXCESS			41	// Too big output size (more than 10.000.000 pixels)
#define S_UNSUPPORTEDCODEPAGE	42	// The code page is not supported
#define S_INVALIDHANDLE			43	// Invalid handle passed
#define S_MENUFAILURE			44	// Failed to retrieve a menu handle, or find a menu item.
#define S_RECURSIVECALL			45	// Method RecalculateLayout has been called recursively.
#define S_TOOLRECURSION			46	// Recursive invocation of a IVXPTooltip method.
#define S_INDEXOUTOFRANGE		47	// Index of the requested tag CONTROL is out of range.

#define DECLARE_EXCEPTION(A) const int E_##A = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 0x200 + S_##A);

#include <atlbase.h>
//You may derive a class from CComModule and use it if you want to override
//something, but do not change the name of _Module
extern CComModule _Module;
#include <atlcom.h>
#include <comdef.h>
#include <ocidl.h>	// Added by ClassView
#include "String.h"

DECLARE_EXCEPTION(GENERALFAILURE)
DECLARE_EXCEPTION(OUTPUTEXCESS)
DECLARE_EXCEPTION(UNSUPPORTEDCODEPAGE)
DECLARE_EXCEPTION(INVALIDHANDLE)
DECLARE_EXCEPTION(MENUFAILURE)
DECLARE_EXCEPTION(RECURSIVECALL)
DECLARE_EXCEPTION(TOOLRECURSION)
DECLARE_EXCEPTION(INDEXOUTOFRANGE)

// All timers will work with this delay only:
#define GLOBAL_TIMER_DELAY	10

// For when we do not know what maximum file path to expect:
#define MAX_PATH_LENGTH	1024

// This function will try to open the file for read-only,
// and return the file handle, if requested.
// The file will be opened in the Unicode mode.
bool FileExists(LPCTSTR FilePath, HANDLE * hFile);

// This function will read the file and return a pointer to the memory buffer.
// MaxBuffer specifies a limit for the file size. If the file is bigger than
// MaxBuffer, only the first MaxBytes bytes will be read.
// When MaxBytes equals zero, no limit applies.
BYTE * ReadFromFile(LPCTSTR FileName, int & size, long MaxBytes = 0);

// This function tries to allocate the resource.
// If the resource is found, then it returns true.
// If pResource is NULL, then it will set nBufSize equal
// the resource size.
// If pResource is not null, it will read the resource into the buffer
// and it will read as many bytes as set by nBufSize
bool GetResource(HINSTANCE hInst, LPCTSTR lpName, LPCTSTR lpType, void * pResource, int & nBufSize);

// This function fills area{x1,y1,x2,y2} with gradient color
// from StartColor to EndColor. bHorizontal defines direction
// of the gradient.
// bDirect = true - direct output into DC; false - through BMP buffer
BOOL FillGradient(HDC hDC, COLORREF StartColor, COLORREF EndColor, int x1, int y1, int x2, int y2, bool bHorizontal = true, bool bDirect = true);

extern bool g_bNTCompitable;	// Contains true, if the operational system is one of
								// the following: Windows NT4/2000/XP/LongHorn,
								// and or false for Windows 95/98/Me

extern bool g_bWin95orNT4;		// Contains true, if the OS is whether Windows 95 or NT4

void InitializeVersionInfo();	// This function is to initialize g_bNTCompitable

#if WINVER < 0x0500
#define	COLOR_GRADIENTACTIVECAPTION		27
#define	COLOR_GRADIENTINACTIVECAPTION	28
#define WM_UNINITMENUPOPUP				0x0125
#define ODS_HOTLIGHT					0x0040
#define OBJID_MENU						0xFFFFFFFD
#define MIM_BACKGROUND					0x00000002

typedef struct tagMENUINFO
{
	DWORD	cbSize;
	DWORD	fMask;
	DWORD	dwStyle;
	UINT	cyMax;
	HBRUSH	hbrBack;
	DWORD	dwContextHelpID;
	long	dwMenuData;
} MENUINFO, FAR *LPMENUINFO;
typedef MENUINFO CONST FAR *LPCMENUINFO;

typedef struct tagMENUBARINFO
{
	DWORD	cbSize;
	RECT	rcBar;
	HMENU	hMenu;
	HWND	hwndMenu;
	BOOL	fBarFocused:1;
	BOOL	fFocused:1;
} MENUBARINFO, *PMENUBARINFO;

typedef BOOL (CALLBACK * SetMenuInfo)(HMENU, LPCMENUINFO);
typedef BOOL (CALLBACK * GetMenuBarInfo)(HWND, LONG, LONG, PMENUBARINFO);
typedef BOOL (CALLBACK * GRADIENT_FILL)(HDC, PTRIVERTEX, ULONG, PVOID, ULONG, ULONG);
typedef DWORD (CALLBACK * GetLongPathNameWide)(LPCWSTR lpszShortPath, LPWSTR lpszLongPath, DWORD cchBuffer);

extern HMODULE hGraphics;
extern GRADIENT_FILL g_pGradientFillFunc;
extern SetMenuInfo g_pSetMenuInfo;
extern GetMenuBarInfo g_pGetMenuBarInfo;
extern GetLongPathNameWide g_pGetLongPathNameW;

#endif

#define __msg(A) ::MessageBox(NULL, _T(A), _T("Msg"), MB_OK|MB_ICONINFORMATION);
#define _msg(A) ::MessageBox(NULL, A, _T("Msg"), MB_OK|MB_ICONINFORMATION);

#ifdef _DEBUG
#define __TRACE(A) ::OutputDebugString(_T(A));
#define _TRACE(A) ::OutputDebugString(A);
#else
#define __TRACE(A)
#define _TRACE(A)
#endif

HRESULT GetGlobalExitCode(long ExceptionFlags, const CLSID & clsID, const IID ID);

template<const CLSID * clsID, const IID * ID>
class CVXPException
{
public:
	CVXPException():m_Exception(0){}
protected:
	void ResetExceptions()
	{
		m_Exception = 0;
	}
	void AddException(long ExceptionFlags)
	{
		m_Exception |= ExceptionFlags;
	}

	HRESULT GetExitCode()
	{
		return GetGlobalExitCode(m_Exception, *clsID, *ID);
	}

	bool HasExceptions()
	{
		return (m_Exception != 0);
	}

private:

	long m_Exception;
};

#define VXPLIB_BEGIN	ResetExceptions();try{
#define VXPLIB_END		}catch(...){AddException(EX_GENERALFAILURE);}return GetExitCode();

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_STDAFX_H__BF86A97F_0031_41A1_BB1F_0DD2811E80E8__INCLUDED)
