/*
 * VXPLib COMPONENT
 *
 * Module: String.cpp - Implementation of class CString
 *
 * Synopsis: Life in C++ without CString is a bit of a hurdle.
 * Without using MFC I had to implement my own CString, plus lots
 * of extra functionality as per working with code pages, HTML
 * resources, better memory management,...etc, etc, etc.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "String.h"
#include "tchar.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CString::CString(BSTR txt)
{
	m_uSize = 0;
	m_uBufferSize = 0;
	m_pText = NULL;
	Strcpy(txt);
}

CString::CString(CString & object)
{
	m_uSize = 0;
	m_uBufferSize = 0;
	m_pText = NULL;
	Strncpy(object.m_pText, object.m_uSize);
}

CString::CString()
{
	m_uSize = 0;
	m_uBufferSize = 0;
	m_pText = NULL;
}

CString::CString(LPCTSTR txt)
{
	m_uSize = 0;
	m_uBufferSize = 0;
	m_pText = NULL;
	Strcpy(txt);
}

CString::CString(UINT nID)
{
	m_uSize = 0;
	m_uBufferSize = 0;
	m_pText = NULL;
	LoadString(nID);
}

CString::~CString()
{
	Release();
}

bool CString::LoadString(UINT nID)
{
	return LoadString(GetModuleHandle(NULL), nID);
}

bool CString::LoadHTML(UINT nID)
{
	return LoadHTML(GetModuleHandle(NULL), nID);
}

bool CString::LoadString(HINSTANCE inst, UINT nID, WORD Length)
{
	Release();
	if(::FindResource(inst, MAKEINTRESOURCE((nID>>4)+1), RT_STRING) == NULL)
		return false;
	int size = MAX_STRING_SIZE;
	if(Length > 0 && Length < MAX_STRING_SIZE)
		size = Length;
	LPTSTR buffer = new TCHAR[size + 1];
	if(!buffer)
		return false;
	Strncpy(buffer, ::LoadString(inst, nID, buffer, size + 1));
	delete []buffer;
	return true;
}

bool CString::LoadHTML(HINSTANCE inst, UINT nID)
{
	Release();
	int nSize = 0;
	if(GetResource(inst, MAKEINTRESOURCE(nID), RT_HTML, NULL, nSize))
	{
		if(nSize < 1)
			return false;
		BYTE * buffer = new BYTE[nSize + 1];
		GetResource(inst, MAKEINTRESOURCE(nID), RT_HTML, buffer, nSize);
		TakeFromANSI((char*)buffer, nSize, CP_ACP);
		delete []buffer;
		return true;
	}
	return false;
}

bool CString::IsEmpty()
{
	return !m_uSize;
}

LPCTSTR CString::Strcpy(LPCTSTR text)
{
	return Strncpy(text, text?_tcslen(text):NULL);
}

LPCTSTR CString::Strncpy(LPCTSTR text, int nSymbols)
{
	if(!text)
	{
		Release();
		return NULL;
	}
	if(nSymbols > m_uBufferSize)
	{
		Release();
		m_pText = new TCHAR[nSymbols + 1];
		m_uBufferSize = nSymbols;
	}
	if(!m_pText)
		return NULL;
	
	_copynstr(m_pText, text, nSymbols);
	m_pText[nSymbols] = 0;
	m_uSize = nSymbols;
	return m_pText;
}

LPCTSTR CString::Strcpy(BSTR text)
{
	_bstr_t bstr(text);
	return Strncpy(bstr, bstr.length());
}

void CString::Release()
{
	m_uSize = 0;
	m_uBufferSize = 0;
	if(m_pText)
	{
		delete []m_pText;
		m_pText = NULL;
	}
}

int CString::GetLength()
{
	return m_uSize;
}

int CString::Compare(LPCTSTR lpsz)
{
	if(!m_uSize)
		return -1;
	if(!lpsz)
		return 1;
	return _tcscmp(m_pText, lpsz);
}

int CString::CompareNoCase(LPCTSTR lpsz)
{
	if(!m_uSize)
		return -1;
	if(!lpsz)
		return 1;
	return _tcsicmp(m_pText, lpsz);
}

int CString::FindOneOf(LPCTSTR lpszCharSet)
{
	if(!lpszCharSet || !m_uSize)
		return -1;
	LPTSTR lpsz = _tcspbrk(m_pText, lpszCharSet);
	return (lpsz == NULL) ? -1 : (int)(lpsz - m_pText);
}

int CString::Find(TCHAR ch)
{
	return Find(ch, 0);
}

int CString::Find(TCHAR ch, int nStart)
{
	if(m_uSize <= nStart)
		return -1;
	LPTSTR lpsz = _tcschr(m_pText + nStart, (_TUCHAR)ch);
	return (lpsz == NULL) ? -1 : (int)(lpsz - m_pText);
}

int CString::Find(LPCTSTR lpszSub)
{
	return Find(lpszSub, 0);
}

int CString::Find(LPCTSTR lpszSub, int nStart)
{
	if(m_uSize <= nStart || !lpszSub)
		return -1;
	LPTSTR lpsz = _tcsstr(m_pText + nStart, lpszSub);
	return (lpsz == NULL) ? -1 : (int)(lpsz - m_pText);
}

int CString::ReverseFind(TCHAR ch)
{
	if(!m_uSize)
		return -1;
	LPTSTR lpsz = _tcsrchr(m_pText, (_TUCHAR)ch);
	return (lpsz == NULL) ? -1 : (int)(lpsz - m_pText);
}

void CString::Empty()
{
	Release();
}

TCHAR CString::GetAt(int nIndex)
{
#ifdef _DEBUG
	if(nIndex > 0 && nIndex < m_uSize)
		return m_pText[nIndex];
	return -1;
#else
	return m_pText[nIndex];
#endif
}

void CString::SetAt(int nIndex, TCHAR ch)
{
#ifdef _DEBUG
	if(nIndex > 0 && nIndex < m_uSize)
#endif
		m_pText[nIndex] = ch;
}

CString & CString::operator = (LPCTSTR txt)
{
	Strcpy(txt);
	return *this;
}

CString & CString::operator = (BSTR txt)
{
	Strcpy(txt);
	return *this;
}

CString & CString::operator = (CString & object)
{
	Strncpy(object.m_pText, object.m_uSize);
	return *this;
}

CString & CString::operator += (LPCTSTR txt)
{
	if(!txt)
		return *this;

	AddText(txt, _tcslen(txt));
	return *this;
}

CString & CString::operator += (TCHAR symbol)
{
	AddText(&symbol, 1);
	return *this;
}

TCHAR CString::operator[](int nIndex)
{
#ifdef _DEBUG
	if(nIndex >= 0 && nIndex < m_uSize)
		return m_pText[nIndex];
	return -1;
#else
	return m_pText[nIndex];
#endif
}

bool CString::operator == (LPCTSTR txt)
{
	return !Compare(txt);
}

bool CString::operator != (LPCTSTR txt)
{
	return Compare(txt)?true:false;
}

CString::operator LPCTSTR()
{
	return m_pText;
}

void CString::AddText(LPCTSTR txt, int Length)
{
	if(txt)
	{
		if(!Length)
			return;

		if(Length + m_uSize > m_uBufferSize)
		{
			LPTSTR tmp = new TCHAR[sizeof(TCHAR)*(Length + m_uSize + 1)];
			_copynstr(tmp, m_pText, m_uSize);
			_copynstr(tmp + m_uSize, txt, Length);
			tmp[m_uSize + Length] = 0;

			if(m_pText)
				delete []m_pText;
			m_pText = tmp;
			m_uSize += Length;
			m_uBufferSize = m_uSize;
		}
		else
		{
			_copynstr(m_pText + m_uSize, txt, Length);
			m_uSize += Length;
			m_pText[m_uSize] = 0;
		}
	}
}

void CString::Shrink()
{
	if(m_uBufferSize > m_uSize)
	{
		if(m_uSize)
		{
			m_pText = (LPTSTR)realloc(m_pText, (m_uSize + 1) * sizeof(TCHAR));
			m_pText[m_uSize] = 0;
			m_uBufferSize = m_uSize;
		}
		else
			Release();
	}
}

void CString::ShrinkEffective()
{
	if (m_uSize * sizeof(TCHAR) > 16 && m_uBufferSize >= m_uSize * 2)
	{
		m_pText = (LPTSTR)realloc(m_pText, (m_uSize + 1) * sizeof(TCHAR));
		m_pText[m_uSize] = 0;
		m_uBufferSize = m_uSize;
	}
}

CString CString::Left(int nCount)
{
	if (nCount < 0)
		nCount = 0;
	if(nCount >= m_uSize)
		return *this;
	
	CString dest;
	dest.Strncpy(m_pText, nCount);
	return dest;
}

CString CString::Right(int nCount)
{
	if (nCount < 0)
		nCount = 0;
	if(nCount >= m_uSize)
		return *this;
	
	CString dest;
	dest.Strncpy(m_pText + m_uSize - nCount, nCount);
	return dest;
}

CString CString::Mid(int nFirst, int nCount)
{
	if(!nFirst)
		return Left(nCount);

	CString dest;
	if (nCount < 0)
		nCount = 0;
	if(nFirst >= m_uSize)
		return dest;
	if(nFirst + nCount > m_uSize)
		nCount = m_uSize - nFirst;
	
	dest.Strncpy(m_pText + nFirst, nCount);
	return dest;
}

CString CString::SpanIncluding(LPCTSTR lpszCharSet)
{
	if(!m_uSize)
		return CString();

	return Left(_tcsspn(m_pText, lpszCharSet));
}

CString CString::SpanExcluding(LPCTSTR lpszCharSet)
{
	if(!m_uSize)
		return CString();

	return Left(_tcscspn(m_pText, lpszCharSet));
}

void CString::MakeUpper()
{
	if(!m_uSize)
		return;

	_make_upper(m_pText);
}

void CString::MakeLower()
{
	if(!m_uSize)
		return;

	_make_lower(m_pText);
}

void CString::MakeReverse()
{
	if(!m_uSize)
		return;

	_tcsrev(m_pText);
}

int CString::Replace(TCHAR chOld, TCHAR chNew)
{
	int nCount = 0;
	if (chOld != chNew)
	{
		LPTSTR psz = m_pText;
		LPTSTR pszEnd = psz + m_uSize;
		while (psz < pszEnd)
		{
			if (*psz == chOld)
			{
				*psz = chNew;
				nCount++;
			}
			psz = _tcsinc(psz);
		}
	}
	return nCount;
}

int CString::Remove(TCHAR chRemove)
{
	if(!m_uSize)
		return 0;

	LPTSTR pstrSource = m_pText;
	LPTSTR pstrDest = m_pText;
	LPTSTR pstrEnd = m_pText + m_uSize;
	while (pstrSource < pstrEnd)
	{
		if (*pstrSource != chRemove)
		{
			*pstrDest = *pstrSource;
			pstrDest = _tcsinc(pstrDest);
		}
		pstrSource = _tcsinc(pstrSource);
	}
	if(m_pText)
		*pstrDest = '\0';
	int nCount = pstrSource - pstrDest;
	m_uSize -= nCount;
	return nCount;
}

int CString::Delete(int nIndex, int nCount /* = 1 */)
{
	if (nIndex < 0)
		nIndex = 0;
	int nNewLength = m_uSize;
	if (nCount > 0 && nIndex < nNewLength)
	{
		int nBytesToCopy = nNewLength - (nIndex + nCount) + 1;
		memcpy(m_pText + nIndex, m_pText + nIndex + nCount, nBytesToCopy * sizeof(TCHAR));
		m_uSize = nNewLength - nCount;
	}
	return nNewLength;
}

void CString::TrimLeft(LPCTSTR lpszTargets)
{
	if(!m_uSize)
		return;

	// if we're not trimming anything, we're not doing any work
	if (SafeStrlen(lpszTargets) == 0)
		return;

	LPCTSTR lpsz = m_pText;
	while (*lpsz != '\0')
	{
		if (_tcschr(lpszTargets, *lpsz) == NULL)
			break;
		lpsz = _tcsinc(lpsz);
	}
	if (lpsz != m_pText)
	{
		// fix up data and length
		int nDataLength = m_uSize - (lpsz - m_pText);
		memmove(m_pText, lpsz, (nDataLength + 1)*sizeof(TCHAR));
		m_uSize = nDataLength;
	}
}

void CString::TrimLeft(TCHAR chTarget)
{
	if(!m_uSize)
		return;
	// find first non-matching character
	LPCTSTR lpsz = m_pText;
	while (chTarget == *lpsz)
		lpsz = _tcsinc(lpsz);
	if (lpsz != m_pText)
	{
		// fix up data and length
		int nDataLength = m_uSize - (lpsz - m_pText);
		memmove(m_pText, lpsz, (nDataLength+1)*sizeof(TCHAR));
		m_uSize = nDataLength;
	}
}

void CString::TrimLeft()
{
	if(!m_uSize)
		return;
	// find first non-space character
	LPCTSTR lpsz = m_pText;
	while (_istspace(*lpsz))
		lpsz = _tcsinc(lpsz);
	if (lpsz != m_pText)
	{
		// fix up data and length
		int nDataLength = m_uSize - (lpsz - m_pText);
		memmove(m_pText, lpsz, (nDataLength+1)*sizeof(TCHAR));
		m_uSize = nDataLength;
	}
}

void CString::TrimRight(LPCTSTR lpszTargetList)
{
	if(!m_uSize)
		return;

	LPTSTR lpsz = m_pText;
	LPTSTR lpszLast = NULL;
	while (*lpsz != '\0')
	{
		if (_tcschr(lpszTargetList, *lpsz) != NULL)
		{
			if (lpszLast == NULL)
				lpszLast = lpsz;
		}
		else
			lpszLast = NULL;
		lpsz = _tcsinc(lpsz);
	}
	if (lpszLast != NULL)
	{
		// truncate at left-most matching character
		*lpszLast = '\0';
		m_uSize = lpszLast - m_pText;
	}
}

void CString::TrimRight(TCHAR chTarget)
{
	if(!m_uSize)
		return;

	LPTSTR lpsz = m_pText;
	LPTSTR lpszLast = NULL;
	while (*lpsz != '\0')
	{
		if (*lpsz == chTarget)
		{
			if (lpszLast == NULL)
				lpszLast = lpsz;
		}
		else
			lpszLast = NULL;
		lpsz = _tcsinc(lpsz);
	}

	if (lpszLast != NULL)
	{
		// truncate at left-most matching character
		*lpszLast = '\0';
		m_uSize = lpszLast - m_pText;
	}
}

void CString::TrimRight()
{
	if(!m_uSize)
		return;

	LPTSTR lpsz = m_pText;
	LPTSTR lpszLast = NULL;
	while (*lpsz != '\0')
	{
		if (_istspace(*lpsz))
		{
			if (lpszLast == NULL)
				lpszLast = lpsz;
		}
		else
			lpszLast = NULL;
		lpsz = _tcsinc(lpsz);
	}

	if (lpszLast != NULL)
	{
		// truncate at trailing space start
		*lpszLast = '\0';
		m_uSize = lpszLast - m_pText;
	}
}

CString CString::GetExcluding(LPCTSTR exclusion, int & nStart, int & nLength)
{
	CString result;
	if(!m_uSize || nStart > m_uSize || nStart < 0)
	{
		nLength = nStart = 0;
		return result;
	}
	CString excl(exclusion);
	int i = nStart;
	nLength = 0;
// Now we must find the beginning and the end of such text:
	while(i < m_uSize)
	{
		if(excl.Find(m_pText[i]) == -1)	// Normal text found
		{
			if(!nLength)
				nStart = i;
			nLength ++;						// Accumulating the length
		}
		else
			if(nLength)	// the normal text is over
				break;
		i ++;
	}
	if(nLength)
		result = Mid(nStart, nLength);
	return result;
}

BSTR CString::SysAllocString()
{
	if(m_uSize)
		return ::SysAllocString(m_pText);
	return NULL;
}

LPTSTR CString::PreAllocate(int size)
{
	if(m_uBufferSize >= size)
		return m_pText;
	m_uBufferSize = size;
	if(m_pText)
		m_pText = (LPTSTR)realloc(m_pText, size * sizeof(TCHAR));
	else
		m_pText = new TCHAR[size + 1];
	return m_pText;
}

int CString::Count(TCHAR symbol)
{
	if(!m_uSize)
		return 0;
	int counter = 0;
	for(int i=0;i<m_uSize;i++)
		if(m_pText[i] == symbol)
			counter ++;
	return counter;
}

void CString::PostAllocate(int size)
{
	if(!m_uBufferSize)
		return;
	if(size != -1 && size < m_uBufferSize)
	{
		m_pText[size] = 0;
		m_uSize = size;
	}
	else
	{
		m_uSize = 0;
		int i = 0;
		while(i <= m_uBufferSize)
		{
			if(!m_pText[i])
			{
				m_uSize = i;
				return;
			}
			i ++;
		}
	}
}

bool CString::FormatMessage(UINT ResID, ...)
{
	CString tmp(ResID);
	va_list argList;
	va_start(argList, ResID);
	bool result = FormatMessage(tmp, argList);
	va_end(argList);
	return result;
}

bool CString::FormatMessage(LPCTSTR lpszFormat, ...)
{
	// format message into temporary buffer lpszTemp
	va_list argList;
	va_start(argList, lpszFormat);
	LPTSTR lpszTemp;

	if(!::FormatMessage(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ALLOCATE_BUFFER, lpszFormat, 0, 0, (LPTSTR)&lpszTemp, 0, &argList) || lpszTemp == NULL)
		return false;

	// assign lpszTemp into the resulting string and free the temporary
	*this = lpszTemp;
	LocalFree(lpszTemp);
	va_end(argList);
	
	return true;
}

void CString::AdjustCodePage(long CodePage)
{
	if(!m_uSize || !CodePage || !IsValidCodePage(CodePage))
		return;

	int L = WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK, m_pText, m_uSize, NULL, 0, NULL, NULL);
	char * buffer = new char[L + 1];
	int N = WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK, m_pText, m_uSize, buffer, L, NULL, NULL);
	int K = MultiByteToWideChar(CodePage, MB_PRECOMPOSED, buffer, N, NULL, 0);
	MultiByteToWideChar(CodePage, MB_PRECOMPOSED, buffer, N, PreAllocate(K + 1), K);
	PostAllocate(K);
}

void CString::TakeFromBSTR(BSTR text, long CodePage)
{
	if(CodePage)
	{
		char * pStr = _com_util::ConvertBSTRToString(text);
		int L = strlen(pStr);
		int Size = MultiByteToWideChar(CodePage, MB_PRECOMPOSED, pStr, L, NULL, 0);
		MultiByteToWideChar(CodePage, MB_PRECOMPOSED, pStr, L, PreAllocate(Size + 1), Size);
		PostAllocate(Size);
	}
	else
		Strcpy(text);
}

int CString::TakeFromANSI(LPCSTR buffer, int Length, long CodePage)
{
	int total = 0;
	if(!CodePage || (CodePage && IsValidCodePage(CodePage)))
	{
		int style = (CodePage == CP_UTF8)?0:MB_PRECOMPOSED;
		int Size = MultiByteToWideChar(CodePage, style, buffer, Length, NULL, 0);
		PreAllocate(Size + 1);
		total = MultiByteToWideChar(CodePage, style, buffer, Length, m_pText, Size);
		m_pText[total] = 0;
		m_uSize = _tcslen(m_pText);
	}
	return total;
}

// If the text doesn't have anything except spaces in it
// the text is emptied.
// It also removes all repetitious spaces, leaving single ones
// boundaries.
// The first flag is true when the first spaces are to be cut completely.
void CString::SmartTrim(bool bCutSpaces /* =false */)
{
	if(!m_uSize)
		return;

	bool bStart = true;
	int nSpaces = 0;
	bool bHasSymbols = false;
	for(int I = 0;I < m_uSize;I ++)
	{
		TCHAR symbol = m_pText[I];

		if(symbol == ' ' || symbol == '\n' || symbol == '\r' || symbol == '\t')
		{
			m_pText[I] = ' ';
			nSpaces ++;
		}
		else
		{
			bHasSymbols = true;
			if(bStart)
			{
				// Just reached the beginning of the phrase:
				bStart = false;
				int shift = 1;
				if(bCutSpaces)
					shift = 0;
				if(nSpaces > shift)
				{
					ShiftLeft(nSpaces, nSpaces - shift);
					I -= nSpaces - shift;
				}
				else
					I ++;
			}
			else
			{
				// Somewhere in between:
				if(nSpaces > 1)
				{
					ShiftLeft(I, nSpaces - 1);
					I -= nSpaces - 1;
					
				}
			}
			nSpaces = 0;
		}
	}
	if(!bHasSymbols)
		Empty();
	else
	{
		if(nSpaces > 1)
		{
			m_uSize -= nSpaces - 1;
			m_pText[m_uSize] = 0;
		}
		ShrinkEffective();
	}
}

// This function shifts contents of the string to the left
// by nSymbols symbols beginning from position FirstPos
// without reallocating memory.
void CString::ShiftLeft(int FirstPos, int nSymbols)
{
	_copynstr(m_pText + FirstPos  - nSymbols, m_pText + FirstPos, m_uSize - FirstPos + 1);
	m_uSize -= nSymbols;
}


// Function name	: CString::TextToHTMLEx
// Description	    : Returns the text formatted using HTML:
//		1. All symbols \t are replaced with &#09;
//		2. All symbols \n\r are replaced with <BR>
//		3. All lines are broken using <BR> when needed
//		4. A line can be broken only on symbol ' ' or '\t'
//		5. '<' is replaced with &lt;
//		6. '>' is replaced with &gt;
//		7. '&' is replaced with &amp;
//		8. Spaces and tabs a truncated on ends of each line
// Return type		: CString
// Argument         : int MaxWidth - Maximum number of symbols in one line.
//						If MaxWidth is 0 no length limitation applies.
// Argument         : long & nRows - Maximum number of rows, or 0, if no limit
//						applies. On return this parameter is set to contain the
//						resulting number of rows in the returned text.
//						When the text doesn't fit into the specified number of
//						rows the last word is appended with "..." and bTruncated
//						will contain True (see below).
// Argument         : bool & bTruncated - On return this parameter is set to be
//						True, if on input nRows wasn't 0, and the text didn't
//						fit into the number of rows specified. Otherwise,
//						bTruncated will be set to False.
// Argument         : VXPTextToHTML tthOptions - Additional options to be used
//						during conversion (read VXPTextToHTML in IDL)
CString CString::TextToHTMLEx(int nMaxWidth, long & nRows, bool & bTruncated, VXPTextToHTML tthOptions)
{
	// Working exceptionally with temporary
	// copy of the text string:
	CString tmp = (*this);

	// Removing spaces and tabs on both ends:
	tmp.TrimLeft();
	tmp.TrimRight();

	// Actual length of the line:
	int Length = tmp.GetLength();

	// Pointer to be used:
	LPTSTR pText = (LPTSTR)(LPCTSTR)tmp;

	// Resulting  text:
	CString strResult;

	// Preallocate space for quicker processing:
	strResult.PreAllocate(Length * 2);

	long Index = 0;		// Current index accessed in the source string
	long Bookmark = 0;	// Index of the last symbol that was copied from the source
	long nRowCounter = 0;	// Current number of rows, if nRows is not 0
							// or 0, if nRows is 0.

	if(Length)
		nRowCounter = 1;

	bool bJustTruncated = false;
	
	bTruncated = false;

	CString strBuffer;
	strBuffer.PreAllocate((nMaxWidth ? nMaxWidth : Length) * 5);

	while(Index <= Length && (!nRows || nRowCounter <= nRows)) // Main loop
	{

		//////////////////////////////////////////////////////////
		// Step1: Find where the next valid text begins;
		//////////////////////////////////////////////////////////
		// The search starts at the last position of the bookmark;
		// All spaces and tabs are to be removed/replaced;
		// \r amd \n(0x0A) are to be replaced with <BR> as required;
		// By the end of this step the bookmark is to point at the
		// first valid symbol to be copied.
		// Index must point at the next symbols after the bookmark.

		bool bContinue = true; // The loop is to be continued
		while(Bookmark < Length && bContinue)
		{
			switch(pText[Bookmark])
			{
			case ' ':
			case '\r':
				{
					// Just skip this one here;
					break;
				}
			case '\t':	// Tabulation
				{
					if(!(tthOptions & tthTabsAsSpaces))
						strResult += _T("&#09;");
					break;
				}
			case '\n':	// Next line
				{
					if(!(tthOptions & tthIgnoreNewLines))
					{
						if(nRows && nRowCounter == nRows)
						{
							bTruncated = true;
							break;
						}
						strResult += _T("<BR>");
						nRowCounter ++; // Another row has been added;
						bJustTruncated = true;
					}
					break;
				}
			default:	// Any valid character other than a gap or line breaker
				{
					// Here we found a valid symbol,
					// and, therefore must end the loop:
					bContinue = false;	// End the loop
					break;					
				}
			}
			
			if(bTruncated)
				break;

			if(bContinue)	// Only for special symbols we move bookmark
				Bookmark ++;	// Next character
		}

		if(bTruncated)
			break;

		// If the bookmark has reached the end of the text here
		// there is nothing left to copy, and the whole function
		// should terminate:
		if(Bookmark == Length)
			break;
		// Otherwise, a correct symbol is found at the bookmark:
		Index = Bookmark + 1; // Now analyze characters after the bookmark

		///////////////////////////////////////////////////////////
		// Step2: Getting the line to be copied;
		///////////////////////////////////////////////////////////
		// Continue until the maximum number of characters in the
		// current line is reached or a line breaker is encountered;
		// Keep the counter of symbols starting from the bookmark;
		// Repititious white spaces are skipped if required;
		// At the end the bookmark continues pointing at the beginning
		// of the text to be copied, and the current index is pointing
		// at the next symbol after the one to be copied;

		long nSymbols = 1;		// Current number of symbols
		long LastGapIndex = 0;	// 0 is an invalid index
		bool bValidCharacter = true; // Last symbol was a valid character
		long nGaps = 0; // Current number of gaps
		// The loop goes while:
		// 1. There are symbols left in the source
		// 2. Maximum number of symbols is not reached
		// 3. Line breaker is not reached
		while(Index < Length && (nSymbols <= nMaxWidth || !nMaxWidth || !LastGapIndex))
		{
			if(pText[Index] == '\r')	// If Return symbol is found
				pText[Index] = ' ';		// Just replace it with white space for simplicity

			if(pText[Index] == '\n')	// New line
			{
				if(tthOptions & tthIgnoreNewLines)
					pText[Index] = ' ';		// Just replace it with white space for simplicity
				else
					break; // Go to the next step
			}

			if(pText[Index] == ' ' || pText[Index] == '\t') // If white space is found or tabulation
			{
				// We must always remember index of the
				// last gap in case we have to return to it:
				if(!LastGapIndex || bValidCharacter)	// If this is the first continuous white space
				{
					LastGapIndex = Index;	// Remember the index
					bValidCharacter = false; // This is to avoid remebering index of a repititious gap
				}
				nGaps ++;
			}
			else // Any valid character
			{
				// When a valid character is encountered we must remember that
				// fact to reset LastGapIndex next time a gap is met:
				bValidCharacter = true;

				nGaps = 0; // No current gaps
			}

			if(nMaxWidth) // If symbols must be countered
			{
				// We must allow counting repeated gaps, if required so:
				if(tthOptions & tthDuplicateGaps)
					nSymbols ++;
				else // Otherwise, by default, no repeated gaps
				{
					if(nGaps <= 1) // Only if current number of gaps is 1 or none
						nSymbols ++;
				}
			}

			Index ++;	// Next character from the source
		}

		// Verify whether we have to reverse the index here
		// or roll it forward:
		if(nMaxWidth && nSymbols > nMaxWidth) // If reached the limit
		{
			nGaps = 0;
			if(LastGapIndex)
				Index = LastGapIndex;
		}

		// Other reasons for which we might end the loop are:
		// 1. Index == Length
		// 2. pText[Index] == '\n'

		/////////////////////////////////////////////////////////
		// Step 3: Copying the text and replacing symbols
		/////////////////////////////////////////////////////////
		// As an entry here we have bookmark and index pointing at the
		// right text, i.e. the text is in Mid(Bookmark, Index - Bookmark);
		// We must replace &, <, > and '\t' with correct symbols;
		// Remove extra spaces/tabs between words, if required

		strBuffer.Empty();

		bool bGap = false;
		// The loop is to ignore gaps at the end;
		for(int I = Bookmark; I < Index - nGaps; I++)
		{
			switch(pText[I])
			{
			case ' ':
				{
					if(bGap)
					{
						if(tthOptions & tthDuplicateGaps)
							strBuffer += ' ';
					}
					else
					{
						strBuffer += ' ';
						bGap = true;
					}
					break;
				}
			case '\t':
				{
					if(bGap)
					{
						if(tthOptions & tthDuplicateGaps)
						{
							if(tthOptions & tthTabsAsSpaces)
								strBuffer += ' ';
							else
								strBuffer += _T("&#09;");
						}
					}
					else
					{
						if(tthOptions & tthTabsAsSpaces)
							strBuffer += ' ';
						else
							strBuffer += _T("&#09;");
						bGap = true;
					}
					break;
				}
			case '<':
				{
					bGap = false;
					strBuffer += _T("&lt;");
					break;
				}
			case '>':
				{
					bGap = false;
					strBuffer += _T("&gt;");
					break;
				}
			case '&':
				{
					bGap = false;
					strBuffer += _T("&amp;");
					break;
				}
			default:
				{
					bGap = false;
					strBuffer += pText[I];
					break;
				}
			}
		}

		if(nMaxWidth && !bJustTruncated && !strResult.IsEmpty())
		{
			if(nRows && nRowCounter == nRows)
			{
				bTruncated = true;
				break;
			}
			else
			{
				nRowCounter ++;
				strResult += _T("<BR>");
			}
		}

		bJustTruncated = false;

		strResult += strBuffer;

		Bookmark = Index - nGaps;

		/////////////////////////////////////////////////////////
		Index ++;

	}
	nRows = nRowCounter;
	if(bTruncated && !(tthOptions & tthAppendNoDots))
		strResult += _T("...");
	return strResult;
}


// Function name	: CString::TextToHTML
// Description	    : Converts plain text into HTML using function
//					  TextToHTMLEx (see above)
// Return type		: CString
// Argument         : int nMaxWidth
CString CString::TextToHTML(int nMaxWidth)
{
	long nRows = 0;
	bool bTruncated;
	return TextToHTMLEx(nMaxWidth, nRows, bTruncated);
}
