/*
 * VXPLib COMPONENT
 *
 * Module: String.h - Declaration of class CString
 *
 * Synopsis: Life in C++ without CString is a bit of a hurdle.
 * Without using MFC I had to implement my own CString, plus lots
 * of extra functionality as per working with code pages, HTML
 * resources, better memory management,...etc, etc, etc.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#if !defined(AFX_CSTRING_H__AFD1577E_7495_4078_B30D_50521A2D0EC8__INCLUDED_)
#define AFX_CSTRING_H__AFD1577E_7495_4078_B30D_50521A2D0EC8__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "tchar.h"
#include "VXPLib.h"

#ifdef _tcsncpy_s // Visual Studio 2005

	#define _copynstr(d, s, n) _tcsncpy_s(d, n + 1, s, n)
	#define _make_upper(str) _tcsupr_s(str, _tcslen(str) + 1)
	#define _make_lower(str) _tcslwr_s(str, _tcslen(str) + 1)

#else	// Visual Studio 6/2002/2003

	#define _copynstr(d, s, n) _tcsncpy(d, s, n)
	#define _make_upper(str) _tcsupr(str)
	#define _make_lower(str) _tcslwr(str)

#endif

// Maximum size of a text string in a resource file:
#define MAX_STRING_SIZE	1024

class CString
{
public:

	// Converts a simple text string into HTML:
	CString TextToHTML(int nMaxWidth = 0);

	// Extended conversion of text into HTML:
	CString TextToHTMLEx(int nMaxWidth, long & nRows, bool & bTruncated, VXPTextToHTML tthOptions = tthDefault);

	inline void ShiftLeft(int FirstPos, int nSymbols);
	void SmartTrim(bool bCutSpaces = false);
	void AdjustCodePage(long CodePage);
	void PostAllocate(int size = -1);
	int Count(TCHAR symbol);
	// Constructors:
	CString();
	CString(LPCTSTR txt);
	CString(UINT nID);
	CString(CString & object);
	explicit CString(BSTR txt);
	// Destructor:
	virtual ~CString();
		
	CString Left(int nCount);
	CString Right(int nCount);
	CString Mid(int nFirst, int nCount);

	// Formatting the text string
	// Returns true, if successful
	bool FormatMessage(LPCTSTR lpszFormat, ...);
	bool FormatMessage(UINT ResID, ...);

	// Shrinks the actual buffer to the size of the string
	void Shrink();

	// Shrinks the actual buffer to the size of the string
	// if the actual buffer is at least twice as long as the
	// string length, and the text length is at least 16 symbols
	// (8 for Unicode)
	void ShrinkEffective();

	// This function is to allocate some memory for the pointer
	// in order to provide for faster memory operations.
	// It returns the buffer pointer.
	// Call PostAllocate after this method to set the correct string size.
	// Use sequential call to Shrink or ShrinkEffective after this
	// to avoid unnecessary memory being allocated.
	LPTSTR PreAllocate(int size);

	BSTR SysAllocString();

	bool IsEmpty();
	void Empty();
	int GetLength();

	// Compares two strings. Returns:
	// < 0, if this string is less than parameter;
	// < 0, if this string is bigger than the parameter;
	//   0 - if the strings are equal.
	int Compare(LPCTSTR lpsz);
	int CompareNoCase(LPCTSTR lpsz);

	bool LoadString(UINT nID);
	// Will load a text string from the resource file while
	// controlling the length of the string.
	// Passing 0 or a value in access of MAX_STRING_SIZE won't work
	// for the length, and it will be treated just as MAX_STRING_SIZE:
	bool LoadString(HINSTANCE inst, UINT nID, WORD Length = MAX_STRING_SIZE);

	bool LoadHTML(UINT nID);
	bool LoadHTML(HINSTANCE inst, UINT nID);

	int FindOneOf(LPCTSTR lpszCharSet);
	int Find(TCHAR ch);
	int Find(TCHAR ch, int nStart);
	int Find(LPCTSTR lpszSub);
	int Find(LPCTSTR lpszSub, int nStart);
	int ReverseFind(TCHAR ch);

	TCHAR GetAt(int nIndex);
	void SetAt(int nIndex, TCHAR ch);

	TCHAR operator[](int nIndex);
	CString & operator = (LPCTSTR txt);
	CString & operator = (BSTR txt);
	CString & operator = (CString & object);
	CString & operator += (LPCTSTR txt);
	CString & operator += (TCHAR symbol);
	bool operator == (LPCTSTR txt);
	bool operator != (LPCTSTR txt);

	operator LPCTSTR();

	CString SpanIncluding(LPCTSTR lpszCharSet);
	CString SpanExcluding(LPCTSTR lpszCharSet);

	void MakeUpper();
	void MakeLower();
	void MakeReverse();

	int Replace(TCHAR chOld, TCHAR chNew);
	int Remove(TCHAR chRemove);
	int Delete(int nIndex, int nCount = 1);

	void TrimLeft(LPCTSTR lpszTargets);
	void TrimLeft(TCHAR chTarget);
	void TrimLeft();
	void TrimRight(LPCTSTR lpszTargetList);
	void TrimRight(TCHAR chTarget);
	void TrimRight();

	int TakeFromANSI(LPCSTR buffer, int Length, long CodePage);
	void TakeFromBSTR(BSTR text, long CodePage);

// My own functions ****************************************************
	
	// Retrieve a text string which doesn't have any of the
	// exclusion symbols, and set 'nStart' and 'nLength'
	// accordingly.
	// The search begins with index 'nStart'
	CString GetExcluding(LPCTSTR exclusion, int & nStart, int & nLength);

private:

	LPCTSTR Strcpy(LPCTSTR text);
	LPCTSTR Strncpy(LPCTSTR text, int nSymbols);
	LPCTSTR Strcpy(BSTR text);
	void Release();
	
	// Adds txt to the existing text
	void AddText(LPCTSTR txt, int Length);
	inline int SafeStrlen(LPCTSTR lpsz){return (lpsz==NULL)?0:lstrlen(lpsz);}

private:

	LPTSTR m_pText;
	int m_uSize;
	int m_uBufferSize;
};

#endif // !defined(AFX_CSTRING_H__AFD1577E_7495_4078_B30D_50521A2D0EC8__INCLUDED_)
