/*
 * VXPLib COMPONENT
 *
 * Module: TableMatrix.h - Declaration of classes CMatrixCell
 * and CTableMatrix;
 *
 * Synopsis: The QHTML parser implements matrix analysis of tag <TABLE>
 * Two classes CMatrixCell and CTableMatrix implement logic of syntax analysis
 * for tag <TABLE> of one level of depth.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#if !defined(AFX_TABLEMATRIX_H__5B040542_A133_4928_B0C0_D3FF16112C4B__INCLUDED_)
#define AFX_TABLEMATRIX_H__5B040542_A133_4928_B0C0_D3FF16112C4B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TagObject.h"
#include "Size.h"	// Added by ClassView

// This class os to contain:
// 1. Pointer to the first cell element (type CTagObject*);
// 2. Pointer to the last cell element (type CTagObject*);
class CMatrixCell
{
public:
	bool IsEmpty();
	// Constructor
	CMatrixCell();

	// Sets the two variables
	void Init(CTagObject * first, CTagObject * last);

	CSize m_contentSize;	// Size of the contained sequence of objects;
	CSize m_cellSize;		// The real size of the cell
	
	CSize m_RealContents;	// Unlike m_contentSize this variable doesn't
							// include width and height if they were forced

	CSize m_MinSize;		// This variable is to store minimum size
							// as set by the intial TD: width, height,
							// which initiated the cell
	// Properties:
	CTagObject * m_pFirst;
	CTagObject * m_pLast;

	CTagObject * m_pTR;	// TR object that initialized this cell
	CTagObject * m_pTD; // TD object that initialized this cell
};

class CTableMatrix
{
public:
	int GetMaxColsRow();
	CTableMatrix(CTagObject * start, CTagObject * end);
	virtual ~CTableMatrix();

	void SetBorderPos(int ZeroX, int ZeroY, int width, int height, int border, int cellspacing);
	CMatrixCell * GetCell(int col, int row);
	CSize GetMatrixSize();

private:
	void PopulateRow(CTagObject * start, CTagObject * end, int nCols, int row);
	void PopulateMatrix();
	CSize CalculateSize();
	void ReleaseMatrix();
	void InitMatrix();
	
	CSize m_size;	// cx = columns, cy = rows

	CMatrixCell *** m_pCells; // m_pCells[row][col]

	// Starting end ending objects passed to the constructor
	CTagObject * m_pStart;
	CTagObject * m_pEnd;

	// Index of the row, which contains the maximum number of columns
	int m_MaxColsRow;
};

#endif // !defined(AFX_TABLEMATRIX_H__5B040542_A133_4928_B0C0_D3FF16112C4B__INCLUDED_)
