/*
 * VXPLib COMPONENT
 *
 * Module: Tag.cpp - Tag-containing class
 *
 * Synopsis: This file contains implementation of class CTag
 * that represents properties (attributes) of a single tag
 * that QHTML parser singles out from its QHTML document.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "Tag.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

// The only extra symbol is &tab. Totally, 253 entries
// Corresponds to HTML 4.0 standard.

CTag::CCode CTag::m_pCodeList[] = {

{_T("AElig"), 198},
{_T("Aacute"), 193},
{_T("Acirc"), 194},
{_T("Agrave"), 192},
{_T("Alpha"), 913},
{_T("Aring"), 197},
{_T("Atilde"), 195},
{_T("Auml"), 196},
{_T("Beta"), 914},
{_T("Ccedil"), 199},
{_T("Chi"), 935},
{_T("Dagger"), 8225},
{_T("Delta"), 916},
{_T("ETH"), 208},
{_T("Eacute"), 201},
{_T("Egrave"), 200},
{_T("Eirc"), 202},
{_T("Epsilon"), 917},
{_T("Eta"), 919},
{_T("Euml"), 203},
{_T("Gamma"), 915},
{_T("Iacute"), 205},
{_T("Icirc"), 206},
{_T("Igrave"), 204},
{_T("Iota"), 921},
{_T("Iuml"), 207},
{_T("Kappa"), 922},
{_T("Lambda"), 923},
{_T("Mu"), 924},
{_T("Ntilde"), 209},
{_T("Nu"), 925},
{_T("OElig"), 338},
{_T("Oacute"), 211},
{_T("Ocirc"), 212},
{_T("Ograve"), 210},
{_T("Omega"), 937},
{_T("Omicron"), 927},
{_T("Oslash"), 216},
{_T("Otilde"), 213},
{_T("Ouml"), 214},
{_T("Phi"), 934},
{_T("Pi"), 928},
{_T("Prime"), 8243},
{_T("Psi"), 936},
{_T("Rho"), 929},
{_T("Scaron"), 352},
{_T("Sigma"), 931},
{_T("THORN"), 222},
{_T("Tau"), 932},
{_T("Theta"), 920},
{_T("Uacute"), 218},
{_T("Ucirc"), 219},
{_T("Ugrave"), 217},
{_T("Upsilon"), 933},
{_T("Uuml"), 220},
{_T("Xi"), 926},
{_T("Yacute"), 221},
{_T("Yuml"), 376},
{_T("Zeta"), 918},
{_T("aacute"), 225},
{_T("acirc"), 226},
{_T("acute"), 180},
{_T("aelig"), 230},
{_T("agrave"), 224},
{_T("alefsym"), 8501},
{_T("alpha"), 945},
{_T("amp"), 38},
{_T("and"), 8743},
{_T("ang"), 8736},
{_T("aring"), 229},
{_T("asymp"), 8776},
{_T("atilde"), 227},
{_T("auml"), 228},
{_T("bdquo"), 8222},
{_T("beta"), 946},
{_T("brvbar"), 166},
{_T("bull"), 8226},
{_T("cap"), 8745},
{_T("ccedil"), 231},
{_T("cedil"), 184},
{_T("cent"), 162},
{_T("chi"), 967},
{_T("circ"), 710},
{_T("clubs"), 9827},
{_T("cong"), 8773},
{_T("copy"), 169},
{_T("crarr"), 8629},
{_T("cup"), 8746},
{_T("curren"), 164},
{_T("dArr"), 8659},
{_T("dagger"), 8224},
{_T("darr"), 8595},
{_T("deg"), 176},
{_T("delta"), 948},
{_T("diams"), 9830},
{_T("divide"), 247},
{_T("eacute"), 233},
{_T("ecirc"), 234},
{_T("egrave"), 232},
{_T("empty"), 8709},
{_T("emsp"), 8195},
{_T("ensp"), 8194},
{_T("epsilon"), 949},
{_T("equiv"), 8801},
{_T("eta"), 951},
{_T("eth"), 240},
{_T("euml"), 235},
{_T("euro"), 8364},
{_T("exist"), 8707},
{_T("fnof"), 402},
{_T("forall"), 8704},
{_T("frac12"), 189},
{_T("frac14"), 188},
{_T("frac34"), 190},
{_T("frasl"), 8260},
{_T("gamma"), 947},
{_T("ge"), 8805},
{_T("gt"), 62},
{_T("hArr"), 8660},
{_T("harr"), 8596},
{_T("hearts"), 9829},
{_T("hellip"), 8230},
{_T("iacute"), 237},
{_T("icirc"), 238},
{_T("iexcl"), 161},
{_T("igrave"), 236},
{_T("image"), 8465},
{_T("infin"), 8734},
{_T("int"), 8747},
{_T("iota"), 953},
{_T("iquest"), 191},
{_T("isin"), 8712},
{_T("iuml"), 239},
{_T("kappa"), 954},
{_T("lArr"), 8656},
{_T("lambda"), 955},
{_T("lang"), 9001},
{_T("laquo"), 171},
{_T("larr"), 8592},
{_T("lceil"), 8968},
{_T("ldquo"), 8220},
{_T("le"), 8804},
{_T("lfloor"), 8970},
{_T("lowast"), 8727},
{_T("loz"), 9674},
{_T("lrm"), 8206},
{_T("lsaquo"), 8249},
{_T("lsquo"), 8216},
{_T("lt"), 60},
{_T("macr"), 175},
{_T("mdash"), 8212},
{_T("micro"), 181},
{_T("middot"), 183},
{_T("minus"), 8722},
{_T("mu"), 956},
{_T("nabla"), 8711},
{_T("nbsp"), 160},
{_T("ndash"), 8211},
{_T("ne"), 8800},
{_T("ni"), 8715},
{_T("not"), 172},
{_T("notin"), 8713},
{_T("nsub"), 8836},
{_T("ntilde"), 241},
{_T("nu"), 957},
{_T("oacute"), 243},
{_T("ocirc"), 244},
{_T("oelig"), 339},
{_T("ograve"), 242},
{_T("oline"), 8254},
{_T("omega"), 969},
{_T("omicron"), 959},
{_T("oplus"), 8853},
{_T("or"), 8744},
{_T("ordf"), 170},
{_T("ordm"), 186},
{_T("oslash"), 248},
{_T("otilde"), 245},
{_T("otimes"), 8855},
{_T("ouml"), 246},
{_T("para"), 182},
{_T("part"), 8706},
{_T("permil"), 8240},
{_T("perp"), 8869},
{_T("phi"), 966},
{_T("pi"), 960},
{_T("piv"), 982},
{_T("plusmn"), 177},
{_T("pound"), 163},
{_T("prime"), 8242},
{_T("prod"), 8719},
{_T("prop"), 8733},
{_T("psi"), 968},
{_T("quot"), 34},
{_T("rArr"), 8658},
{_T("radic"), 8730},
{_T("rang"), 9002},
{_T("raquo"), 187},
{_T("rarr"), 8594},
{_T("rceil"), 8969},
{_T("rdquo"), 8221},
{_T("real"), 8476},
{_T("reg"), 174},
{_T("rfloor"), 8971},
{_T("rho"), 961},
{_T("rlm"), 8207},
{_T("rsaquo"), 8250},
{_T("rsquo"), 8217},
{_T("sbquo"), 8218},
{_T("scaron"), 353},
{_T("sdot"), 8901},
{_T("sect"), 167},
{_T("shy"), 173},
{_T("sigma"), 963},
{_T("sigmaf"), 962},
{_T("sim"), 8764},
{_T("spades"), 9824},
{_T("sub"), 8834},
{_T("sube"), 8838},
{_T("sum"), 8721},
{_T("sup"), 8835},
{_T("sup1"), 185},
{_T("sup2"), 178},
{_T("sup3"), 179},
{_T("supe"), 8839},
{_T("szlig"), 223},
{_T("tab"), 9},
{_T("tau"), 964},
{_T("there4"), 8756},
{_T("theta"), 952},
{_T("thetasym"), 977},
{_T("thinsp"), 8201},
{_T("thorn"), 254},
{_T("tilde"), 732},
{_T("times"), 215},
{_T("trade"), 8482},
{_T("uArr"), 8657},
{_T("uacute"), 250},
{_T("uarr"), 8593},
{_T("ucirc"), 251},
{_T("ugrave"), 249},
{_T("uml"), 168},
{_T("upsih"), 978},
{_T("upsilon"), 965},
{_T("uuml"), 252},
{_T("weierp"), 8472},
{_T("xi"), 958},
{_T("yacute"), 253},
{_T("yen"), 165},
{_T("yuml"), 255},
{_T("zeta"), 950},
{_T("zwj"), 8205},
{_T("zwnj"), 8204}

					};

CTag::attribute::attribute(LPCTSTR name, LPCTSTR value)
{
	m_name = name;
	m_value = value;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTag::CTag(CString tag)
{
	m_Attr = TA_TEXT;
	m_pAttributes = NULL;
	Init(tag);
}

CTag::CTag()
{
	m_Attr = TA_TEXT;
	m_pAttributes = NULL;
}

CTag::~CTag()
{
	DELETE_COLLECTION(m_pAttributes)
}

#define DEBUG_NEW new

// We assume that the tag doesn't contain
// symbols <, > or any which are not separators (see below)
// neither normal symbols.
bool CTag::Init(CString tag)
{
	DELETE_COLLECTION(m_pAttributes)
	m_TagName.Empty();
	m_Attr = TA_TEXT;
	CString OldTag = tag;
	if(ReadTagName(tag, m_TagName))
	{
		// If it is a comment block <!...> or a sound block <&...>
		if(!m_TagName.IsEmpty() && (m_TagName[0] == '!' || m_TagName[0] == '&'))
		{
			if(m_TagName[0] == '!')
				m_Attr = TA_COMMENT;
			else
				m_Attr = TA_SOUND;

			OldTag.TrimLeft();
			m_TagName = OldTag.Right(OldTag.GetLength() - 1);
			return true;
		}
		m_TagName.MakeUpper();
		CString name;
		CString value;
		while(GetNextAttribute(tag, name, value))
		{
			name.MakeUpper();
			ADD_COLLECTION(m_pAttributes, new attribute(name, value))
		}
		return true;
	}
	return false;
}

// This set of symbols separate words in tags
CString CTag::m_separators(_T(" \n\r\t\'\"=;,"));

bool CTag::IsSeparator(TCHAR symbol)
{
	return (m_separators.Find(symbol) != -1);
}

CString CTag::GetTagName()
{
	return m_TagName;
}

bool CTag::ReadTagName(CString & tag, CString & name)
{
	int length;
	int start = 0;
	name = tag.GetExcluding(m_separators, start, length);
	if(length)
	{
		tag.Delete(0, start + length);
		return true;
	}
	tag.Empty();
	return false;
}

// Valid properties:
// property value can be included into '' or "", or
// it can be as an open value
// Each property must be followed by '=' to indicate
// the beginning of the property's value
bool CTag::GetNextAttribute(CString & tag, CString & name, CString & value)
{
	int length;
	int start;
	do
	{
		start = 0;
		name = tag.GetExcluding(m_separators, start, length);
		if(!length)	
		{
			tag.Empty();
			break;
		}
		// Otherwise we have a correct property name
		tag.Delete(0, start + length);
		int L = tag.GetLength();
		if(!L)
			break;

		// Here we're got our tag name, and we know there some
		// other symbols after.
		int i = 0;
		while(IsSeparator(tag[i]) && i < L)
		{
			if(tag[i] == '=')
			{
				int begin = i + 1;
				int end;
				value = GetPropertyValue(tag, begin, end);
				if(value.IsEmpty())
				{
					// none found, means to more properties too
					tag.Empty();
					return false;
				}
				else
				{
					tag.Delete(0, end);
					return true;
				}
			}
			i ++;
		}
		if(i == L) // then none found
		{
			tag.Empty();
			break;
		}
		// Otherwise, we found a non-separator symbol, which means
		// we have to start the whole procedure from the beginning:
		tag.Delete(0, i);
	}
	while(!tag.IsEmpty());
	return false;
}

// Notes:
// The value can be within '' or "", or it can be open
CString CTag::GetPropertyValue(CString tag, int begin, int & end)
{
	CString result;
	int L = tag.GetLength();
	if(begin >= L)
		return result;
	
	int i = begin;

	int first = -1;
	int second = -1;

	TCHAR parenthesis = 0;

	while((IsSeparator(tag[i]) || first != -1)&& i < L)
	{
		if(tag[i] == '\'' || tag[i] == '\"')
		{
			if(first == -1)//none found before
			{
				first = i;
				parenthesis = tag[i];
			}
			else
			{
				if(tag[i] == parenthesis)
				{
					second = i;
					break;
				}
			}
		}
		i ++;
	}
	if(i == L)
		return result;
	// Otherwise, we found a valid text or something in parenthesis
	if(second != -1) // Means we found something in parenhtesis
	{
		end = second + 1;
		if(second - first > 1) // if there's something in between them
			result = tag.Mid(first + 1, second - first - 1);
		return result;
	}
	// Otherwise we found an open valid text
	// so we must start looking for the end of such
	int k = i;
	while(!IsSeparator(tag[k]) && k < L)
		k ++;

	end = k;
	if(k == L) // the rest of 'tag' is the property value
		result = tag.Right(L - i);
	else
		result = tag.Mid(i, k - i);
	return result;
}

// This function reads 'text', and changes all
// &#xxxx and &name; into appropriate symbols
// The reference is taken for speed optimization only
CString CTag::SetSpecialSymbols(CString & text)
{
	int index = text.Find('&');
	if(index == -1)
		return text;

	CString result;
	// This will optimize work with memory:
	result.PreAllocate(text.GetLength());

	int i = 0;
	int L = text.GetLength();
	int start = 0;
	do
	{
		index = text.Find('&', start);
		if(index == -1)
		{
			result += text.Mid(start, L - start);
			result.ShrinkEffective();
			return result;
		}
		if(index > start)
			result += text.Mid(start, index - start);

		i = index + 1;
		while(i < L && !IsSeparator(text[i]))
			i ++;
	
		if(i == L)	// no following text found
		{
			CString tmp = text.Right(i - index - 1);
			TCHAR symbol;
			if(ConvertCodeIntoSymbol(tmp, symbol))
				result += symbol;
			else
				result += text.Right(i - index);
		}
		else // some following text was found
		{
			CString tmp = text.Mid(index + 1, i - index - 1);
			TCHAR symbol;
			if(ConvertCodeIntoSymbol(tmp, symbol))
			{
				result += symbol;
				if(text[i] == ' ')
					result += ' ';
			}
			else
				result += text.Mid(index, i - index + 1);
			start = i + 1;
		}
	}
	while(i < L && start < L);
	
	result.ShrinkEffective();
	return result;
}

// This function converts 'code' into symbol, if it is
// possible, and returns true, or it returns false otherwise.
// The first parameter is taken by reference for speed
// optimization only.
bool CTag::ConvertCodeIntoSymbol(CString & code, TCHAR & symbol)
{
	if(code.IsEmpty())
		return false;
	int L = code.GetLength();
	if(L > 1 && code[0] == '#')
	{
		int s = _ttoi((LPCTSTR)code + 1);
		if(s)
		{
			symbol = s;
			return true;
		}
	}

// Optimized algorithm to find the symbol in the table
// in the quickest possible way.
	int index = sizeof(m_pCodeList)/sizeof(CCode) / 2;
	float step = (float)(sizeof(m_pCodeList)/sizeof(CCode)) / 4;
	while(step >= 0.25)
	{
		int cmp = code.Compare(m_pCodeList[index].m_name);
		if(!cmp) // If found the code
		{
			symbol = m_pCodeList[index].m_symbol;
			return true;
		}
		if(cmp < 0)
			index -= Int(step);
		else
			index += Int(step);
		
		step /= 2;
	}
	return false;
}

CString CTag::GetAttrTextValue(LPCTSTR attr, bool * found)
{
	CString value;
	if(found)
		*found = false;
	if(m_pAttributes)
	{
		attribute * A = m_pAttributes;
		while(A)
		{
			if(A->m_name == attr)
			{
				if(found)
					*found = true;
				return A->m_value;
			}
			A = (attribute*)A->Next;
		}
	}
	return value;
}

int CTag::GetAttrIntValue(LPCTSTR attr, bool * found)
{
	if(found)
		*found = false;
	bool result;
	CString txt = GetAttrTextValue(attr, &result);
	if(result)
	{
		if(found)
			*found = true;
		return _ttoi(txt);
	}
	return 0;
}

int CTag::GetAttrIntSignedValue(LPCTSTR attr, int & sign, bool * found)
{
	bool HasText;
	CString txt = GetAttrTextValue(attr, &HasText);
	sign = 0; // No sign
	if(!HasText)
	{
		if(found)
			*found = false;
		return 0;
	}
	if(found)
		*found = true;
	txt.TrimLeft();
	if(txt[0] == '+')
		sign = 1;
	else
		if(txt[0] == '-')
			sign = -1;
	return abs(_ttoi(txt));
}

COLORREF CTag::GetAttrColorValue(LPCTSTR attr, bool & IsValid)
{
	CString txt = GetAttrTextValue(attr);
	CleanText(txt);
	if(txt.IsEmpty())
	{
		IsValid = false;
		return RGB(0, 0, 0);
	}
	IsValid = true;
	return GetColorFromText(txt);
}

struct CColorElement
{
	LPCTSTR m_ColorName;	// Name of the color
	COLORREF m_ColorValue;	// Color value
};

struct CSysColorIndex
{
	short m_TableIndex;		// Index of a system color in the global color table
	int m_ColorIndex;		// Code of the color as described in Windows API
};

// This table of colors defines only all named colors and
// system colors. When safe colors are needed, such colors
// must be selected using their hex form. Totally, 172 colors.
// System colors in the table are marked as such:

CColorElement g_ColorTable[] =		{

{_T("3DDKSHADOW"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("3DFACE"), RGB(0x00, 0x00, 0x00)},						// System color
{_T("3DHIGHLIGHT"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("3DHILIGHT"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("3DLIGHT"), RGB(0x00, 0x00, 0x00)},						// System color
{_T("3DSHADOW"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("ACTIVEBORDER"), RGB(0x00, 0x00, 0x00)},				// System color		
{_T("ACTIVECAPTION"), RGB(0x00, 0x00, 0x00)},				// System color		
{_T("ALICEBLUE"), RGB(0xF0, 0xF8, 0xFF)},
{_T("ANTIQUEWHITE"), RGB(0xFA, 0xEB, 0xD7)},
{_T("APPWORKSPACE"), RGB(0x00, 0x00, 0x00)},				// System color		
{_T("AQUA"), RGB(0x00, 0xFF, 0xFF)},
{_T("AQUAMARINE"), RGB(0x7F, 0xFF, 0xD4)},
{_T("AZURE"), RGB(0xF0, 0xFF, 0xFF)},
{attrBACKGROUND, RGB(0x00, 0x00, 0x00)},					// System color	
{_T("BEIGE"), RGB(0xF5, 0xF5, 0xDC)},
{_T("BISQUE"), RGB(0xFF, 0xE4, 0xC4)},
{_T("BLACK"), RGB(0x00, 0x00, 0x00)},						
{_T("BLANCHEDALMOND"), RGB(0xFF, 0xEB, 0xCD)},						
{_T("BLUE"), RGB(0x00, 0x00, 0xFF)},						
{_T("BLUEVIOLET"), RGB(0x8A, 0x2B, 0xE2)},
{_T("BROWN"), RGB(0xA5, 0x2A, 0x2A)},						
{_T("BTNFACE"), RGB(0x00, 0x00, 0x00)},						// System color
{_T("BTNHIGHLIGHT"), RGB(0x00, 0x00, 0x00)},				// System color		
{_T("BTNHILIGHT"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("BTNSHADOW"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("BTNTEXT"), RGB(0x00, 0x00, 0x00)},						// System color
{_T("BURLYWOOD"), RGB(0xDE, 0xB8, 0xA7)},						
{_T("CADETBLUE"), RGB(0x5F, 0x9E, 0xA0)},						
{_T("CAPTIONTEXT"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("CHARTREUSE"), RGB(0x7F, 0xFF, 0x00)},						
{_T("CHOCOLATE"), RGB(0xD2, 0x69, 0x1E)},						
{_T("COMFLOWERBLUE"), RGB(0x64, 0x95, 0xED)},						
{_T("COMSILK"), RGB(0xFF, 0xF8, 0xDC)},						
{_T("CORAL"), RGB(0xFF, 0x7F, 0x50)},						
{_T("CRIMSON"), RGB(0xDC, 0x14, 0x3C)},						
{_T("DARKBLUE"), RGB(0x00, 0x00, 0x8B)},						
{_T("DARKCYAN"), RGB(0x00, 0x8B, 0x8B)},						
{_T("DARKGOLDENROD"), RGB(0xB8, 0x86, 0x0B)},
{_T("DARKGRAY"), RGB(0xA9, 0xA9, 0xA9)},
{_T("DARKGREEN"), RGB(0x00, 0x64, 0x00)},
{_T("DARKKHAKI"), RGB(0xBD, 0xB7, 0x6B)},
{_T("DARKMAGENTA"), RGB(0x8B, 0x00, 0x8B)},
{_T("DARKOLIVEGREEN"), RGB(0x55, 0x6B, 0x2F)},
{_T("DARKORANGE"), RGB(0xFF, 0xBC, 0x00)},
{_T("DARKORCHID"), RGB(0x99, 0x32, 0xCC)},
{_T("DARKRED"), RGB(0x8B, 0x00, 0x00)},
{_T("DARKSALMON"), RGB(0xE9, 0x96, 0x7A)},
{_T("DARKSEAGREEN"), RGB(0x8F, 0xBC, 0x8F)},
{_T("DARKSLATEBLUE"), RGB(0x48, 0x3D, 0x8B)},
{_T("DARKSLATEGRAY"), RGB(0x2F, 0x4F, 0x4F)},
{_T("DARKTURQUOISE"), RGB(0x00, 0xCE, 0xD1)},						
{_T("DARKVIOLET"), RGB(0x94, 0x00, 0xD3)},						
{_T("DEEPPINK"), RGB(0xFF, 0x14, 0x93)},						
{_T("DEEPSKYBLUE"), RGB(0x00, 0xBF, 0xFF)},						
{_T("DESKTOP"), RGB(0x00, 0x00, 0x00)},						// System color
{_T("DIMGRAY"), RGB(0x69, 0x69, 0x69)},						
{_T("DODGERBLUE"), RGB(0x1E, 0x90, 0xFF)},						
{_T("FIREBRICK"), RGB(0xB2, 0x22, 0x22)},						
{_T("FLORALWHITE"), RGB(0xFF, 0xFA, 0xF0)},						
{_T("FORESTGREEN"), RGB(0x22, 0x8B, 0x22)},						
{_T("FUCHSIA"), RGB(0xFF, 0x00, 0xFF)},
{_T("GAC"), RGB(0x00, 0x00, 0x00)},				// System color
{_T("GAINSBORO"), RGB(0xDC, 0xDC, 0xDC)},						
{_T("GHOSTWHITE"), RGB(0xF8, 0xF8, 0xFF)},						
{_T("GIC"), RGB(0x00, 0x00, 0x00)},				// System color
{_T("GOLD"), RGB(0xFF, 0xD7, 0x00)},						
{_T("GOLDENROD"), RGB(0xDA, 0xA5, 0x20)},
{_T("GRADIENTACTIVECAPTION"), RGB(0x00, 0x00, 0x00)},		// System color				
{_T("GRADIENTINACTIVECAPTION"), RGB(0x00, 0x00, 0x00)},		// System color				
{_T("GRAY"), RGB(0x80, 0x80, 0x80)},						
{_T("GRAYTEXT"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("GREEN"), RGB(0x00, 0x80, 0x00)},						
{_T("GREENYELLOW"), RGB(0xAD, 0xFF, 0x2F)},						
{_T("HIGHLIGHT"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("HIGHLIGHTTEXT"), RGB(0x00, 0x00, 0x00)},				// System color		
{_T("HONEYDEW"), RGB(0xF0, 0xFF, 0xF0)},						
{_T("HOTLIGHT"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("HOTPINK"), RGB(0xFF, 0x69, 0xB4)},						
{_T("INACTIVEBORDER"), RGB(0x00, 0x00, 0x00)},				// System color		
{_T("INACTIVECAPTION"), RGB(0x00, 0x00, 0x00)},				// System color		
{_T("INACTIVECAPTIONTEXT"), RGB(0x00, 0x00, 0x00)},			// System color			
{_T("INDIANRED"), RGB(0xCD, 0x5C, 0x5C)},						
{_T("INDIGO"), RGB(0x4B, 0x00, 0x82)},						
{_T("INFOBK"), RGB(0x00, 0x00, 0x00)},						// System color
{_T("INFOTEXT"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("IVORY"), RGB(0xFF, 0xFF, 0xF0)},						
{_T("KHAKI"), RGB(0xF0, 0xE6, 0x8C)},						
{_T("LAVENDER"), RGB(0xE6, 0xE6, 0xFA)},						
{_T("LAVENDERBLUSH"), RGB(0xFF, 0xF0, 0xF5)},						
{_T("LAWNGREEN"), RGB(0x7C, 0xFC, 0x00)},						
{_T("LEMONCHIFFON"), RGB(0xFF, 0xFA, 0xCD)},						
{_T("LIGHTBLUE"), RGB(0xAD, 0xD8, 0xE6)},						
{_T("LIGHTCORAL"), RGB(0xF0, 0x80, 0x80)},						
{_T("LIGHTCYAN"), RGB(0xE0, 0xFF, 0xFF)},						
{_T("LIGHTGOLDENRODYELLOW"), RGB(0xFA, 0xFA, 0xD2)},						
{_T("LIGHTGRAY"), RGB(0xD3, 0xD3, 0xD3)},						
{_T("LIGHTGREEN"), RGB(0x90, 0xEE, 0x90)},						
{_T("LIGHTPINK"), RGB(0xFF, 0xB6, 0xC1)},						
{_T("LIGHTSALMON"), RGB(0xFF, 0xA0, 0x7A)},						
{_T("LIGHTSEAGREEN"), RGB(0x20, 0xBA, 0xAA)},						
{_T("LIGHTSKYBLUE"), RGB(0x87, 0xCE, 0xFA)},						
{_T("LIGHTSLATEGRAY"), RGB(0x77, 0x88, 0x99)},						
{_T("LIGHTSTEELBLUE"), RGB(0xB0, 0xC4, 0xDE)},						
{_T("LIGHTYELLOW"), RGB(0xFF, 0xFF, 0xE0)},						
{_T("LIME"), RGB(0x00, 0xFF, 0x00)},						
{_T("LIMEGREEN"), RGB(0x32, 0xCD, 0x32)},						
{_T("LINEN"), RGB(0xFA, 0xF0, 0xE6)},						
{_T("MAROON"), RGB(0x80, 0x00, 0x00)},						
{_T("MEDIUMAQUAMARINE"), RGB(0x66, 0xCD, 0xAA)},						
{_T("MEDIUMBLUE"), RGB(0x00, 0x00, 0xCD)},						
{_T("MEDIUMORCHID"), RGB(0xBA, 0x55, 0xD3)},						
{_T("MEDIUMPURPLE"), RGB(0x93, 0x70, 0xDB)},						
{_T("MEDIUMSEAGREEN"), RGB(0x3C, 0xB3, 0x71)},						
{_T("MEDIUMSLATEBLUE"), RGB(0x7B, 0x68, 0xEE)},						
{_T("MEDIUMSPRINGGREEN"), RGB(0x00, 0xFA, 0x9A)},						
{_T("MEDIUMTURQUOISE"), RGB(0x48, 0xD1, 0xCC)},						
{_T("MEDIUMVIOLETRED"), RGB(0xC7, 0x15, 0x85)},						
{_T("MENU"), RGB(0x00, 0x00, 0x00)},						// System color
{_T("MENUTEXT"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("MIDNIGHTBLUE"), RGB(0x19, 0x19, 0x70)},						
{_T("MINTCREAM"), RGB(0xF5, 0xFF, 0xFA)},						
{_T("MISTYROSE"), RGB(0xFF, 0xE4, 0xE1)},						
{_T("MOCCASIN"), RGB(0xFF, 0xE4, 0xB5)},						
{_T("NAVAJOWHITE"), RGB(0xFF, 0xDE, 0xAD)},						
{_T("NAVY"), RGB(0x00, 0x00, 0x80)},						
{_T("OLDLACE"), RGB(0xFD, 0xF5, 0xE6)},						
{_T("OLIVE"), RGB(0x80, 0x80, 0x00)},						
{_T("OLIVEDRAB"), RGB(0x6B, 0x8E, 0x23)},						
{_T("ORANGE"), RGB(0xFF, 0xA5, 0x00)},						
{_T("ORANGERED"), RGB(0xFF, 0x45, 0x00)},						
{_T("ORCHID"), RGB(0xDA, 0x70, 0xD6)},						
{_T("PALEGOLDENROD"), RGB(0xEE, 0xE8, 0xAA)},						
{_T("PALEGREEN"), RGB(0x98, 0xFB, 0x98)},						
{_T("PALETURQUOISE"), RGB(0xAF, 0xEE, 0xEE)},						
{_T("PALEVIOLETRED"), RGB(0xDB, 0x70, 0x93)},						
{_T("PAPAYAWHIP"), RGB(0xFF, 0xEF, 0xD5)},						
{_T("PEACHPUFF"), RGB(0xFF, 0xDA, 0xB9)},						
{_T("PERU"), RGB(0xCD, 0x85, 0x3F)},						
{_T("PINK"), RGB(0xFF, 0xC0, 0xCB)},						
{_T("PLUM"), RGB(0xDD, 0xA0, 0xDD)},						
{_T("POWDERBLUE"), RGB(0xB0, 0xE0, 0xE6)},						
{_T("PURPLE"), RGB(0x80, 0x00, 0x80)},						
{_T("RED"), RGB(0xFF, 0x00, 0x00)},						
{_T("ROSYBROWN"), RGB(0xBC, 0x8F, 0x8F)},						
{_T("ROYALBLUE"), RGB(0x41, 0x69, 0xE1)},						
{_T("SADDLEBROWN"), RGB(0x8B, 0x45, 0x13)},						
{_T("SALMON"), RGB(0xFA, 0x80, 0x72)},						
{_T("SANDYBROWN"), RGB(0xF4, 0xA4, 0x60)},						
{_T("SCROLLBAR"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("SEAGREEN"), RGB(0x2E, 0x8B, 0x57)},						
{_T("SEASHELL"), RGB(0xFF, 0xF5, 0xEE)},						
{_T("SIENNA"), RGB(0xA0, 0x52, 0x2D)},						
{_T("SILVER"), RGB(0xC0, 0xC0, 0xC0)},						
{_T("SKYBLUE"), RGB(0x87, 0xCE, 0xEB)},						
{_T("SLATEBLUE"), RGB(0x6A, 0x5A, 0xCD)},						
{_T("SLATEGRAY"), RGB(0x70, 0x80, 0x90)},						
{_T("SNOW"), RGB(0xFF, 0xFA, 0xFA)},						
{_T("SPRINGGREEN"), RGB(0x00, 0xFF, 0x7F)},						
{_T("STEELBLUE"), RGB(0x46, 0x82, 0xB4)},						
{_T("TAN"), RGB(0xD2, 0xB4, 0x8C)},						
{_T("TEAL"), RGB(0x00, 0x80, 0x80)},						
{_T("THISTLE"), RGB(0xD8, 0xBF, 0xD8)},						
{_T("TOMATO"), RGB(0xFF, 0x63, 0x47)},						
{_T("TURQUOISE"), RGB(0x40, 0xE0, 0xD0)},
{_T("VIOLET"), RGB(0xEE, 0x82, 0xEE)},
{_T("WHEAT"), RGB(0xF5, 0xDE, 0xB3)},
{_T("WHITE"), RGB(0xFF, 0xFF, 0xFF)},
{_T("WHITESMOKE"), RGB(0xF5, 0xF5, 0xF5)},
{_T("WINDOW"), RGB(0x00, 0x00, 0x00)},						// System color
{_T("WINDOWFRAME"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("WINDOWTEXT"), RGB(0x00, 0x00, 0x00)},					// System color	
{_T("YELLOW"), RGB(0xFF, 0xFF, 0x00)},
{_T("YELLOWGREEN"), RGB(0x9A, 0xCD, 0x32)}
						
									};
// Below is a table of indexes of system colors in the global
// table of colors along with color ID-s as described
// in Windows API:

CSysColorIndex g_sysColorTableIdx[] =		{

{0, COLOR_3DDKSHADOW},
{1, COLOR_3DFACE},
{2, COLOR_3DHIGHLIGHT},
{3, COLOR_3DHILIGHT},
{4, COLOR_3DLIGHT},
{5, COLOR_3DSHADOW},
{6, COLOR_ACTIVEBORDER},
{7, COLOR_ACTIVECAPTION},
{10, COLOR_APPWORKSPACE},
{14, COLOR_BACKGROUND},
{22, COLOR_BTNFACE},
{23, COLOR_BTNHIGHLIGHT},
{24, COLOR_BTNHILIGHT},
{25, COLOR_BTNSHADOW},
{26, COLOR_BTNTEXT},
{29, COLOR_CAPTIONTEXT},
{55, COLOR_DESKTOP},
{62, COLOR_GRADIENTACTIVECAPTION},
{65, COLOR_GRADIENTINACTIVECAPTION},
{68, COLOR_GRADIENTACTIVECAPTION},
{69, COLOR_GRADIENTINACTIVECAPTION},
{71, COLOR_GRAYTEXT},
{74, COLOR_HIGHLIGHT},
{75, COLOR_HIGHLIGHTTEXT}, // Windows 2000 constant
{77, COLOR_HOTLIGHT},
{79, COLOR_INACTIVEBORDER},
{80, COLOR_INACTIVECAPTION},
{81, COLOR_INACTIVECAPTIONTEXT},
{84, COLOR_INFOBK},
{85, COLOR_INFOTEXT},
{118, COLOR_MENU},
{119, COLOR_MENUTEXT},
{149, COLOR_SCROLLBAR},
{169, COLOR_WINDOW},
{170, COLOR_WINDOWFRAME},
{171, COLOR_WINDOWTEXT}

											};

COLORREF CTag::GetColorFromText(CString & text)
{
	text.MakeUpper();
	if(text[0] == '#')
	{
		text.Delete(0, 1);
		return GetColorFromHexText(text);
	}

// Optimized algorithm to find the color in the table
// in the quickest possible way.
	int index = sizeof(g_ColorTable)/sizeof(CColorElement) / 2;
	float step = (float)(sizeof(g_ColorTable)/sizeof(CColorElement)) / 4;
	while(step >= 0.25)
	{
		int cmp = text.Compare(g_ColorTable[index].m_ColorName);
		if(!cmp) // If found the color
			return g_ColorTable[index].m_ColorValue;
		if(cmp < 0)
			index -= Int(step);
		else
			index += Int(step);
		
		step /= 2;
	}
	return RGB(0,0,0);
}

// This function removes all junk symbols from the text
void CTag::CleanText(CString &text)
{
	text.Remove(' ');	// Remove spaces
	text.Remove('\n');	// Remove 0xa
	text.Remove('\r');	// Remove 0xd
	text.Remove('\t');	// Remove tabs
}

COLORREF CTag::GetColorFromHexText(CString & text)
{
	TCHAR HexTable[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
	int L = text.GetLength();
	DWORD color = 0;
	if(L)
	{
		if(L > 6)
			L = 6;	// Cannot understand more than 6 symbols
		int index = 0;
		while(L)
		{
			for(int i = 0;i < sizeof(HexTable)/sizeof(TCHAR);i ++)
				if(text[index] == HexTable[i])
				{
					color += i;
					if(L - 1 > 0)
						color *= 16;
					break;
				}
			index ++;
			L --;
		}
	}
	return RGB(LOBYTE(HIWORD(color)), HIBYTE(LOWORD(color)), LOBYTE(LOWORD(color)));
}

// This method returns percents
// If the value is more than 100, 100 is returned
// If the value is less then 0, 0 is returned
short CTag::GetAttrPercentValue(LPCTSTR attr, bool & isSpecified)
{
	CString txt = GetAttrTextValue(attr);
	txt.Remove('%');
	if(txt.IsEmpty())
	{
		isSpecified = false;
		return 0;
	}
	isSpecified = true;
	short result = _ttoi(txt);
	if(result > 100)
		result = 100;
	if(result < 0)
		result = 0;
	return result;
}

void CTag::UpdateSysColors()
{
	for(int i=0;i<sizeof(g_sysColorTableIdx)/sizeof(CSysColorIndex);i++)
		g_ColorTable[g_sysColorTableIdx[i].m_TableIndex].m_ColorValue = GetSysColor(g_sysColorTableIdx[i].m_ColorIndex);
}

int CTag::GetAttr()
{
	return m_Attr;
}

CTag & CTag::operator = (CTag & object)
{
	m_Attr = object.m_Attr;
	m_TagName = object.m_TagName;

	attribute * pAttributes = object.m_pAttributes;
	DELETE_COLLECTION(m_pAttributes)

	while(pAttributes)
	{
		ADD_COLLECTION(m_pAttributes, new attribute(pAttributes->m_name, pAttributes->m_value));
		pAttributes = (attribute*)pAttributes->Next;
	}

	return *this;
}
