/*
 * VXPLib COMPONENT
 *
 * Module: Tag.h - Tag-containing class
 *
 * Synopsis: This file contains declaration of class CTag
 * that implements properties (attributes) of a single tag
 * that QHTML parser singles out from its QHTML document.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#if !defined(AFX_TAG_H__54B21DB7_CD3A_416D_8D83_3F2B0CE3CFC2__INCLUDED_)
#define AFX_TAG_H__54B21DB7_CD3A_416D_8D83_3F2B0CE3CFC2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "String.h"
#include "HCollection.h"

#define Int(A) ((A - (int)A)>=0.5)?((int)A + 1):((int)A)

// Often used attributes:
const LPCTSTR attrBACKGROUND = _T("BACKGROUND");
const LPCTSTR attrBG = _T("BG");
const LPCTSTR attrBGCOLOR = _T("BGCOLOR");
const LPCTSTR attrBORDERCOLOR = _T("BORDERCOLOR");
const LPCTSTR attrBC = _T("BC");
const LPCTSTR attrBORDER = _T("BORDER");
const LPCTSTR attrALIGN = _T("ALIGN");
const LPCTSTR attrVALIGN = _T("VALIGN");
const LPCTSTR attrCOLOR = _T("COLOR");
const LPCTSTR attrSIZE = _T("SIZE");
const LPCTSTR attrWIDTH = _T("WIDTH");
const LPCTSTR attrHEIGHT = _T("HEIGHT");
const LPCTSTR attrRES = _T("RES");
const LPCTSTR attrFILE = _T("FILE");
const LPCTSTR attrPARAM = _T("PARAM");

// Table tag attributes:
#define TA_TABLE		1
#define TA_TABLE_END	2
#define TA_TR			3
#define TA_TR_END		4
#define TA_TD			5
#define TA_TD_END		6
#define TA_HR			7
#define TA_BR			8
#define TA_TEXT			9

// Comment tags defined using <!tag...>
#define TA_COMMENT		10

// Comment tags defined using <&tag...>
#define TA_SOUND		11

// Tag Control
#define TA_CONTROL		12

class CTag
{
	struct CCode
	{
		LPCTSTR m_name;
		TCHAR m_symbol;
	};

	class attribute:public HCollection
	{
	public:
		attribute(LPCTSTR name, LPCTSTR value);
		CString m_name; // Name of the attribute
		CString m_value; // Value of the attribute
	};
	
	// This function must find the tag name and cut the tag
	// up untill the next symbol after the tag name.
	// If no tag name found, it returns false, or true otherwise.
	bool ReadTagName(CString & tag, CString & name);

	// This function finds the next attribute and its value,
	// sets them into the 'name' and 'value', and cuts
	// the 'tag' to point to the next symbol
	// Returns true, only if found any valid property.
	// The rules for valid properties explained in the implementation.
	bool GetNextAttribute(CString & tag, CString & name, CString & value);

	// Searches the 'tag' for a property value from index 'begin'
	// If found, returns the value, and sets 'end' to the end
	// of the property value.
	// If not found, returns an empty string, and the value for
	// 'end' is indeterminate.
	CString GetPropertyValue(CString tag, int begin, int & end);

public:

	// Copy operator:
	CTag & operator = (CTag & object);

	int GetAttr();
	static void UpdateSysColors();
	CTag();
	CTag(CString tag);
	virtual ~CTag();

	// It returns true, if any string found that can be the tag name
	bool Init(CString tag);
	CString GetTagName();

	CString GetAttrTextValue(LPCTSTR attr, bool * found = NULL);
	int GetAttrIntValue(LPCTSTR attr, bool * found = NULL);
	short GetAttrPercentValue(LPCTSTR attr, bool & isSpecified);
	int GetAttrIntSignedValue(LPCTSTR attr, int & sign, bool * found = NULL);

	COLORREF GetAttrColorValue(LPCTSTR attr, bool & IsValid);

	// Static functions:
	static bool IsSeparator(TCHAR symbol);
	static CString SetSpecialSymbols(CString & text);
	static bool ConvertCodeIntoSymbol(CString & code, TCHAR & symbol);
private:	
	void CleanText(CString & text);
	COLORREF GetColorFromHexText(CString & text);

	attribute * m_pAttributes;
	CString m_TagName;

	static CCode m_pCodeList[];
	static CString m_separators;
	
	COLORREF GetColorFromText(CString & text);

	int m_Attr;	// Can be one of three values:
					// TA_TEXT - nothing
					// TA_COMMENT - comment
					// TA_SOUND - sound
	          

};

#endif // !defined(AFX_TAG_H__54B21DB7_CD3A_416D_8D83_3F2B0CE3CFC2__INCLUDED_)
