/*
 * VXPLib COMPONENT
 *
 * Module: TagLib.h - Declarations of QHTML tag classes
 *
 * Synopsis: This file contains declarations of all classes
 * that implement tag objects of QHTML syntax.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#ifndef Tag_Lib
#define Tag_Lib

#include "TagObject.h"
#include "GlobalObject.h"

#include "Picture.h"

TAG_OBJECT(CTOTableEnd)
TAG_OBJECT(CTOTableTREnd)
TAG_OBJECT(CTOTableTDEnd)

class CTOTable:public CTagObject
{
public:
	CTOTable(CTag * tag, CGlobalObject * globalObject);
	~CTOTable();
	
	COLORREF GetBorderColor();

protected:
	virtual bool IsTextSplitter(){return true;};
	virtual int GetBorderSize();
	virtual void OnDrawTag(HDC dc, int x, int y, int width, int height, int ZeroX, int ZeroY);
	virtual int GetCellPadding();
	virtual int GetCellSpacing();
private:

// Table tag attributes:
	int m_uBorder; // Border size of the table
	int m_uCellPadding;
	int m_uCellSpacing;
	COLORREF m_bgColor;
	bool m_bIsBGValid;	// Contains true, if the background must be filled

	COLORREF m_BorderColor;
	bool m_bIsBCValid;	// Contains true, if the background must be filled
	CPicture * m_pBackground;

// Gradient filling support:
	CGradientInfo m_grad;
};

class CTOTableTR:public CTagObject
{
public:
	CTOTableTR(CTag * tag, CGlobalObject * globalObject);
	~CTOTableTR();
	
	void SetLast();

protected:
	virtual bool IsTextSplitter(){return true;};
	virtual void OnDrawTag(HDC dc, int x, int y, int width, int height, int ZeroX, int ZeroY);
private:
	bool m_bIsBGValid;	// Contains true, if the background must be filled
	COLORREF m_bgColor;

	CPicture * m_pBackground;

// Gradient filling support:
	CGradientInfo m_grad;

	bool m_bLast; // Contains true, if the row is the last one in the table
};

class CTOTableTD:public CTagObject
{
public:
	CTOTableTD(CTag * tag, CGlobalObject * globalObject);
	~CTOTableTD();
protected:
	virtual bool IsTextSplitter(){return true;};
	virtual CSize GetForcedSize();
	virtual void OnDrawTag(HDC dc, int x, int y, int width, int height, int ZeroX, int ZeroY);
	virtual void OnDrawTDBorders(HDC dc, int ZeroX, int ZeroY);
private:

// Table tag attributes:
	CSize m_MinSize;
	
	bool m_bIsBGValid;	// Contains true, if the background must be filled
	COLORREF m_bgColor;

	CPicture * m_pBackground;

// Gradient filling support:
	CGradientInfo m_grad;
};

class CTOText:public CTagObject
{
public:
	CTOText(CString text, CGlobalObject * globalObject, bool bCutSpaces = false);
protected:
	virtual CSize GetSize(HDC);
	virtual void OnDrawTag(HDC dc, int x, int y, int width, int height, int ZeroX, int ZeroY);
	virtual int GetTopHeight(HDC dc);
	virtual CString GetText();
private:
	CString m_text;
	CGlobalObject * m_pGlobalObject;
	int m_tmDescent;
	int m_AveWidth;
};

class CTOComment:public CTagObject
{
public:
	CTOComment(CString text);
protected:
	virtual CString GetText();
private:
	CString m_text;
};

class CTOSound:public CTagObject
{
public:
	CTOSound(CString text);
protected:
	virtual CString GetText();
private:
	CString m_text;
};

#define HRS_RECT		1
#define HRS_ROUND		2
#define HRS_ELLIPSE		3

class CTO_HR:public CTagObject
{
public:
	CTO_HR(CTag * tag, CGlobalObject * globalObject);
protected:
	virtual CSize GetSize(HDC);
	virtual void OnDrawTag(HDC dc, int x, int y, int width, int height, int ZeroX, int ZeroY);
	virtual bool IsTextSplitter(){return true;};
private:
	short m_size;
	short m_width; // The width in percent
	short m_align;
	COLORREF m_color;
	short m_shape;
};

class CTO_BR:public CTagObject
{
public:
	CTO_BR(CTag * tag, CGlobalObject * globalObject);
protected:
	virtual CSize GetSize(HDC);
	virtual bool IsTextSplitter(){return true;};
private:
	CGlobalObject * m_pGlobalObject;
};

class CTOImage:public CTagObject
{
public:
	CTOImage(CTag * tag, CGlobalObject * globalObject);
	~CTOImage();
protected:
	virtual CSize GetSize(HDC);
	virtual void OnDrawTag(HDC dc, int x, int y, int width, int height, int ZeroX, int ZeroY);
	virtual bool IsTextSplitter(){return true;};
private:
	CPicture * m_pPicture;
	int m_Width;
	int m_Height;
	
	bool m_BGColorValid;
	COLORREF m_bgColor;
	short m_border;

	COLORREF m_BorderColor;
	bool m_BColorValid;
	CString m_alt;
	
	CGlobalObject * m_pGlobalObject;

// Gradient filling support:
	CGradientInfo m_grad;
};

#define TF_NONE		0	// No formatting applied
#define TF_SIMPLE	1	// Only carrage symbols will be formatted: \n\a\r\d
						// Useful to display an HTML text as is, where '<', '>'
						// are regular symbols.
						// It splits the text into a sequence of CTOText objects
						// and <BR> tags.
#define TF_HTML		2	// Default, applies both full html formatting + TF_SIMPLE
// When \n is met, if the following symbol is space or the one before, \n is removed
// Otherwise, it is replaced with space.

class CTOTxt:public CTagObject
{
public:
	CTOTxt(CTag * tag, CGlobalObject * globalObject);
	int m_nFormat;
	CString m_text;
	CTagObject * m_pList;
	
	long m_nSizeLimit; // The size limit in bytes

protected:

	virtual CSize GetSize(HDC);
	virtual void OnDrawTag(HDC dc, int x, int y, int width, int height, int ZeroX, int ZeroY);
	virtual int GetTopHeight(HDC dc);

private:
	CString GetTextFromFile(CTag * tag);
	CString GetTextFromRes(HINSTANCE hInst, CString & res);
	CString GetTextFromTag(CTag * tag);
	int GetFormatType(CTag * tag);
	
	void FormatSimple();
	void FormatHTML();
	void FormatNone();

	CGlobalObject * m_pGlobalObject;
	int m_tmDescent;
	long m_CodePage;
};

class CTODocument:public CTagObject
{
public:
	CTODocument(CTag * tag, CGlobalObject * globalObject);
	~CTODocument();
protected:
	virtual CSize GetSize(HDC);
	virtual void OnDrawTag(HDC dc, int x, int y, int width, int height, int ZeroX, int ZeroY);
	virtual bool IsTextSplitter(){return true;};
private:
	CGlobalObject * m_pGlobalObject;
	CSize m_docSize;
	CString m_strFile;
	CString m_strParam;
	int m_nBorder;
	bool m_bBGColorValid;
	COLORREF m_bgColor;
	COLORREF m_BorderColor;
	CPicture * m_pBackground;

// Gradient filling support:
	CGradientInfo m_grad;
};

class CTOControl:public CTagObject
{
public:
	CTOControl(CTag * tag, CGlobalObject * globalObject);

	long m_ID;

	CTag m_Tag;		// Copy of all attrbutes specified within the tag

protected:

	virtual CSize GetSize(HDC);
	virtual bool IsTextSplitter(){return true;};

private:

	CGlobalObject * m_pGlobalObject;
	
	CString m_strName;
	CSize m_ControlSize;
	CString m_strParam;
};

#endif
