/*
 * VXPLib COMPONENT
 *
 * Module: Timer.cpp - Implementation of class CTimer
 *
 * Synopsis: Standard windows timer has a few limitations in it,
 * which makes it unusable in this component. Therefore, here we
 * have class CTimer that provides us a simple implementation
 * of a timer similar to that used by windows but without the
 * problematic limitations.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "vxplib.h"
#include "Timer.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

#define MUTEX_DELAY	100	// 0.1 second

CTimer * g_Timer = NULL;

CTimer::CTimerClient * volatile CTimer::m_pClients = NULL;
HANDLE volatile CTimer::m_hMutex = NULL;
bool volatile CTimer::m_bTerminate = false;

CTimer::CTimerClient::CTimerClient(HWND hWnd)
{
	m_hWnd = hWnd;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTimer::CTimer()
{
	m_hThread = NULL;
}

CTimer::~CTimer()
{
	Done();
	if(m_pClients)
		delete m_pClients;
}

DWORD WINAPI CTimer::TimerThread(LPVOID lpParameter)
{
	DWORD LastTickCount = ::GetTickCount();
	do
	{
		DWORD NewTickCount = ::GetTickCount();
		if(abs(NewTickCount - LastTickCount) >= GLOBAL_TIMER_DELAY)
		{
			LastTickCount = NewTickCount;
			if(CTimer::m_hMutex && ::WaitForSingleObject(CTimer::m_hMutex, MUTEX_DELAY) == WAIT_OBJECT_0)
			{
				if(CTimer::m_pClients)
				{
					CTimerClient * A = (CTimerClient*)CTimer::m_pClients->GetFirst();
					while(A)
					{
						if(::IsWindow(A->m_hWnd))
							::PostMessage(A->m_hWnd, WM_VXPTIMER, 0, NULL);
						A = (CTimerClient*)A->Next;
					}
				}
				::ReleaseMutex(CTimer::m_hMutex);
			}
		}
		Sleep(GLOBAL_TIMER_DELAY);
	}
	while(CTimer::m_hMutex && !CTimer::m_bTerminate);
	CTimer::m_bTerminate = false;
	return 0;
}

bool CTimer::SetTimer(HWND hWnd)
{
	if(!::IsWindow(hWnd))
		return false;

	Init();

	if(m_hMutex && ::WaitForSingleObject(m_hMutex, MUTEX_DELAY) == WAIT_OBJECT_0)
	{
		ADD_COLLECTION(m_pClients, new CTimerClient(hWnd))
		::ReleaseMutex(m_hMutex);
		return true;
	}

	return false;
}

bool CTimer::KillTimer(HWND hWnd)
{
	if(m_hMutex && ::WaitForSingleObject(m_hMutex, MUTEX_DELAY) == WAIT_OBJECT_0)
	{
		if(m_pClients)
		{
			CTimerClient * A = (CTimerClient*)m_pClients->GetFirst();
			while(A)
			{
				if(A->m_hWnd == hWnd)
					break;
				A = (CTimerClient*)A->Next;
			}
			if(A)
			{
				CTimerClient * B = (CTimerClient*)A->GetChain();
				if(A == B)
				{
					delete m_pClients;
					m_pClients = NULL;
					Done();
				}
				else
				{
					m_pClients = B;
					A->Remove();
				}
			}
		}
		::ReleaseMutex(m_hMutex);
		return true;
	}
	return false;
}

void CTimer::Init()
{
	if(!m_hMutex && !m_hThread)
	{
		m_hMutex = ::CreateMutex(NULL, FALSE, NULL);
#ifdef _MSLU
		DWORD ThreadID;
		m_hThread = ::CreateThread(NULL, 0, TimerThread, NULL, 0, &ThreadID);
#else
		m_hThread = ::CreateThread(NULL, 0, TimerThread, NULL, 0, NULL);
#endif
	}
}

void CTimer::Done()
{
	if(m_hMutex && m_hThread)
	{
		m_bTerminate = true;
		while(m_bTerminate) // wait until the thread is terminated
			Sleep(1);

		m_hThread = NULL; // The worker thread doesn't exist anymore
		
		// Destroying the mutex here:
		::CloseHandle(m_hMutex);
		m_hMutex = NULL;
	}
}
