/*
 * VXPLib COMPONENT
 *
 * Module: Timer.h - Declaration of class CTimer
 *
 * Synopsis: Standard windows timer has a few limitations in it,
 * which makes it unusable in this component. Therefore, here we
 * have class CTimer that provides us a simple implementation
 * of a timer similar to that used by windows but without the
 * problematic limitations.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#ifndef Custom_Timer
#define Custom_Timer

#include "HCollection.h"

#define TIMER_ID	0x9F519
#define WM_VXPTIMER	WM_USER + 100

// This class can have only a single instance in a process
class CTimer
{
public:
	class CTimerClient:public HCollection
	{
	public:
		CTimerClient(HWND hWnd);
		virtual ~CTimerClient(){};

		HWND m_hWnd;	// Client's window handle
	};

public:

	CTimer();
	virtual ~CTimer();

	bool KillTimer(HWND hWnd);
	bool SetTimer(HWND hWnd);

private:
	void Init();
	void Done();
	
	static DWORD WINAPI TimerThread(LPVOID lpParameter);
	
	static CTimerClient * volatile m_pClients;
	static HANDLE volatile m_hMutex;
	static bool volatile m_bTerminate;

	HANDLE m_hThread;
};

extern CTimer * g_Timer;

#endif

