/*
 * VXPLib COMPONENT
 *
 * Module: ToolWnd.cpp - Implementation of class CToolWnd
 *
 * Synopsis: Class CToolWnd implements all the logic of the
 * Tooltip Window with all its methods and properties,
 * while class CVXPTooltip merely forwards all its calls
 * into this class.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "ToolWnd.h"
#include "VXPTooltip.h"

// Multimonitor support:
#define COMPILE_MULTIMON_STUBS
#include "Multimon.h"

CToolWnd * g_ToolWindow = NULL;

#define MAX_MARGIN	500

CWndClassInfo CToolWnd::WithShadow_Class =
{
	{sizeof(WNDCLASSEX), 0, StartWindowProc, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL},
	TOOLTIPCLASS, NULL, NULL, TRUE, 0, _T("")
};

CWndClassInfo CToolWnd::WithoutShadow_Class =
{
	{sizeof(WNDCLASSEX), 0, StartWindowProc, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL},
	TOOLTIPCLASS2, NULL, NULL, TRUE, 0, _T("")
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CToolWnd::CToolWnd()
{
	m_LastPos.x = 0;
	m_LastPos.y = 0;

	m_hSyncWnd = NULL;

	m_MouseClickResponse = mcrNoResponse;
	m_bMouseHooked = false;

	m_bGotTimer = false;
	m_Timer = NULL;

	memset(m_Msg, 0, sizeof(m_Msg));

	m_bCanCall = false;

	m_LastMove.x = 0;
	m_LastMove.y = 0;

	m_bAutohide = true;
	m_bTraceMode = false;

	SetTheme(ttDefault);

// *** Sound variables: ************************************************
	m_bSoundPlaying = false; // No sound in the beginning
	m_FadeSound = 20; // Describes how many times the sound must
					  // be adjusted, while the tooltip is being shown
// **********************************************************************

	m_positioning = tpLeftTop;
	// Resetting coordinates of the last cursor position:
	m_LastCursor.x = 0;
	m_LastCursor.y = 0;

	m_bUpdatedRegion = true;

	m_MemoryBMP = NULL;

	// Setting external default parameters:
	m_bShowWhenEmpty = false;
	
	// This parameters, when not set to zero on Win2000
	// will not let choosing the tool tale or rounded borders.
	m_State = ttsHidden;
	m_BorderColor = GetSysColor(COLOR_WINDOWFRAME);

	m_hRgn = NULL;	// Region handle

	m_bEmpty = true;

	m_bSourceChanged = true;

	// Loading the function to support layers ***
	HMODULE hUser32 = GetModuleHandle(_T("USER32.DLL"));
	m_fnSetLayeredWindowAttributes = (FN_SET_LAYERED_WINDOW_ATTRIBUTES)::GetProcAddress(hUser32, "SetLayeredWindowAttributes");

	m_bLayered = m_fnSetLayeredWindowAttributes?true:false;
	// ******************************************

	m_bSupportsShadow = true;
}

LRESULT CToolWnd::OnPaint()
{
	LRESULT result = DefWindowProc();
	HDC dc = GetDC();

	bool bWasDrawn = false;

	VARIANT_BOOL HasBGPicture;
	m_pTool->m_pHTML->get_HasBackgroundPicture(&HasBGPicture);

	VARIANT_BOOL gfEnabled;
	m_pTool->m_pSettings->get_gfEnabled(&gfEnabled);
	if(gfEnabled)
		HasBGPicture = TRUE;

	int Shift_Y = 0;
	if(m_bHasTail && (m_positioning == tpLeftTop || m_positioning == tpRightTop))
		Shift_Y = TP_HEIGHT;

	if(m_bSourceChanged || m_bUpdatedRegion)
	{
		m_bSourceChanged = false;
		m_bUpdatedRegion = false;

		// Drawing into the memory bitmap
		if(m_MemoryBMP)
		{
			DeleteObject(m_MemoryBMP);
			m_MemoryBMP = NULL;
		}

		if(HasBGPicture)
		{
			HDC memDC = CreateCompatibleDC(dc);
			m_MemoryBMP = CreateCompatibleBitmap(dc, m_wndSize.cx, m_wndSize.cy);
			HGDIOBJ old = SelectObject(memDC, m_MemoryBMP);

			DrawBackground(memDC, true);

			short Transparent;
			m_pTool->m_pHTML->get_Transparent(&Transparent);
			m_pTool->m_pHTML->put_Transparent(TRUE);

			VARIANT_BOOL success;
			m_pTool->m_pHTML->Draw((long)memDC, m_LeftMargin + (m_bHasBorder?1:0), m_TopMargin + (m_bHasBorder?1:0) + Shift_Y, &success);

			m_pTool->m_pHTML->put_Transparent(Transparent);

			SelectObject(memDC, old);
			DeleteDC(memDC);
		}
		else
		{
			DrawBackground(dc, false);
			VARIANT_BOOL success;
			m_pTool->m_pHTML->Draw((long)dc, m_LeftMargin + (m_bHasBorder?1:0), m_TopMargin + (m_bHasBorder?1:0) + Shift_Y, &success);
		}
		bWasDrawn = true;
	}

	if(m_MemoryBMP && HasBGPicture)
	{
		HDC memDC = CreateCompatibleDC(dc);
		HGDIOBJ old = SelectObject(memDC, m_MemoryBMP);
		BitBlt((HDC)dc, 0, 0, m_wndSize.cx, m_wndSize.cy, memDC, 0, 0, SRCCOPY);
		SelectObject(memDC, old);
		DeleteDC(memDC);
	}
	else
	{
		if(!bWasDrawn)
		{
			DrawBackground(dc, false);
			VARIANT_BOOL success;
			m_pTool->m_pHTML->Draw((long)dc, m_LeftMargin + (m_bHasBorder?1:0), m_TopMargin + (m_bHasBorder?1:0) + Shift_Y, &success);
		}
	}

	ReleaseDC(dc);
	return result;
}

LRESULT CToolWnd::OnVXPTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled)
{
	POINT pos;
	GetCursorPos(&pos);

	if(m_bAutohide)
	{
		if(pos.x != m_LastCursor.x || pos.y != m_LastCursor.y)
			HideToolTip();
	}
	
	if(m_bTraceMode && m_State != ttsHidden)
		MoveToolTip((short)pos.x, (short)pos.y);

	if(m_State == ttsShowing)
	{
		m_uFade += MAX_VISIBILITY * GLOBAL_TIMER_DELAY / (float)m_FadeShow;
		if(m_uFade > m_Transparency)
		{
			m_uFade = m_Transparency;
			if(m_fnSetLayeredWindowAttributes && m_bLayered)
				m_fnSetLayeredWindowAttributes(m_hWnd, 0, m_Transparency, LWA_ALPHA);
			SetState(ttsShown);
		}
		else
			if(m_fnSetLayeredWindowAttributes && m_bLayered)
				m_fnSetLayeredWindowAttributes(m_hWnd, 0, (BYTE)m_uFade, LWA_ALPHA);
	}

	if(m_State == ttsHiding)
	{
		m_uFade -= MAX_VISIBILITY * GLOBAL_TIMER_DELAY / (float)m_FadeHide;
		if(m_uFade <= 0)
		{
			m_uFade = 0;
			SoundStop();
			ShowWindow(SW_HIDE);
			SetState(ttsHidden);
		}
		else
			if(m_fnSetLayeredWindowAttributes && m_bLayered)
				m_fnSetLayeredWindowAttributes(m_hWnd, 0, (BYTE)m_uFade, LWA_ALPHA);
	}
	
	if(m_hSyncWnd) // If need to syncronize the tooltip window
	{
		bool bShow = (::IsWindow(m_hSyncWnd) && ::IsWindowVisible(m_hSyncWnd));
		if(bShow)
		{
			HWND hActiveWnd = ::GetActiveWindow();
			if(hActiveWnd != m_hSyncWnd && hActiveWnd != m_hWnd)
				bShow = false;
		}
		if(bShow && m_State == ttsHidden)
			ShowToolTip((short)m_LastPos.x, (short)m_LastPos.y);
		else
			if(!bShow && m_State == ttsShown)
				HideToolTip();
	}

	// Adjusting the sound depending on the current state of the tootlip:
	SoundAdjust();

	return 0;
}

LRESULT CToolWnd::OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled)
{
	if(!m_bGotTimer && ::IsWindow(m_hWnd))
		m_bGotTimer = g_Timer->SetTimer(m_hWnd);

	return 0;
}

// We create window here depending on what Shadow effect is set.
// If it effects something, then we destroy the current window and
// create the new one. We also reregister the class.
BOOL CToolWnd::Create()
{
	if(m_hWnd)
		return FALSE;

	g_ToolWindow = this;

	RECT r = {0, 0, 0, 0};

	long ExStyle = WS_EX_TOOLWINDOW|WS_EX_TOPMOST|WS_EX_CONTROLPARENT;

	if(m_bLayered && !g_bWin95orNT4)
		ExStyle |= WS_EX_LAYERED;

	CWindowImpl<CToolWnd, CWindow>::Create(NULL, r, NULL, WS_POPUP|WS_TABSTOP, ExStyle);
	if(!m_hWnd) // Then the window class got unregistered by another instance of IVXPTooltip
	{
		CreateTooltipClasses();
		CWindowImpl<CToolWnd, CWindow>::Create(NULL, r, NULL, WS_POPUP|WS_TABSTOP, ExStyle);
	}

	m_Timer = SetTimer(TIMER_ID, 1000); // Call every second
	m_bGotTimer = g_Timer->SetTimer(m_hWnd);

	if(m_hWnd)
		return TRUE;

	return FALSE;
}

void CToolWnd::CreateTooltipClasses()
{
// When used inside the component itself the classes already may
// have been registered. This is why we need to verify it:

	WNDCLASS info;
	if(GetClassInfo(_Module.GetModuleInstance(), TOOLTIPCLASS2, &info))
		return;

// *************************************************************

	WNDCLASSEX wcx; 
	wcx.cbSize = sizeof(wcx);
	wcx.style = CS_DBLCLKS|CS_SAVEBITS;
	wcx.style |= CS_DROPSHADOW;
	wcx.lpfnWndProc = ::DefWindowProc;
	wcx.cbClsExtra = 0;
	wcx.cbWndExtra = 0;
	wcx.hInstance = _Module.GetModuleInstance();
	wcx.hIcon = NULL;
	wcx.hCursor = LoadCursor(NULL, IDC_ARROW);
	
	// The value of NULL for background provides transparency
	wcx.hbrBackground = GetSysColorBrush(COLOR_WINDOW);
	wcx.lpszMenuName =  NULL;
	wcx.lpszClassName = TOOLTIPCLASS;
	wcx.hIconSm = NULL;

	if(!RegisterClassEx(&wcx))
	{
		wcx.style &= ~CS_DROPSHADOW;
		RegisterClassEx(&wcx);
		// We'll indicate to the client that the shadow effect
		// is not supported on this OS
		m_bSupportsShadow = false;
		m_bHasShadow = false;
	}
	
	// Registering another class like that, only certainly without shadow
	wcx.style &= ~CS_DROPSHADOW;
	wcx.lpszClassName = TOOLTIPCLASS2;
	RegisterClassEx(&wcx);
}

void CToolWnd::Construct(CVXPTooltip * pTool)
{
	m_pTool = pTool;
	
	// Creating tooltip window classes
	CreateTooltipClasses();
}

// When called, the tooltip can be in any possible state,
// which must be taken into account.
void CToolWnd::Destruct()
{
	// Destroying the tooltip window
	Destroy();

	// Destroying all created classes
	UnregisterClass(TOOLTIPCLASS, _Module.GetModuleInstance());
	UnregisterClass(TOOLTIPCLASS2, _Module.GetModuleInstance());
}

// This method is to draw the entire tooltip
// without its contents.
void CToolWnd::DrawBackground(HDC dc, bool HasBackground)
{
// Reading background color for this object
// Note: In CVXPTooltip::OnSetDefaults() we set this color
// to COLOR_INFOBK - the right one, so it can be changed only
// by a programmer overriding OnSetDefaults()

	HRGN hRgn = GetNewRegion();

	if(HasBackground)
	{
		VARIANT_BOOL result;
		m_pTool->m_pHTML->DrawBackground((long)dc, 0, 0, m_wndSize.cx, m_wndSize.cy, TRUE, &result);
	}
	else
	{
		COLORREF bgColor;
		m_pTool->m_pSettings->get_bgColor(&bgColor);
		HBRUSH brush = CreateSolidBrush(bgColor);
		FillRgn(dc, hRgn, brush);
		DeleteObject(brush);
	}

	if(m_bHasBorder)
	{
		HBRUSH BorderBrush = CreateSolidBrush(m_BorderColor);
		FrameRgn(dc, hRgn, BorderBrush, 1, 1);
		DeleteObject(BorderBrush);
	}
	DeleteObject(hRgn);
}

void CToolWnd::UpdateSizes()
{
// Step1: Updating the screen size
	m_screen.SetSize(GetSystemMetrics(SM_CXVIRTUALSCREEN), GetSystemMetrics(SM_CYVIRTUALSCREEN));

// Initializing m_MainScreen and m_VirtPos for multi-monitor support:

// Getting size of the main screen:
	m_MainScreen.SetSize(GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN));
// Getting size of the virtual screen:
	m_VirtPos.SetSize(GetSystemMetrics(SM_XVIRTUALSCREEN), GetSystemMetrics(SM_YVIRTUALSCREEN));

// Step2: Updating the size of the HTML area:
	short width, height;
	m_pTool->m_pHTML->get_Width(&width);
	m_pTool->m_pHTML->get_Height(&height);

	// Controlling minimum sizes for the tooltip when it
	// has a tail.
	
	m_bEmpty = (!width || !height);

	if(m_bHasTail && width < MIN_WIDTH)
		width = MIN_WIDTH;
	if(m_bHasTail && height < MIN_HEIGHT)
		height = MIN_HEIGHT;

	m_htmlSize.SetSize(width, height);

// Step3: Updating the size of the tooltip window,
// meanwhile, controlling margins:
// For now, ignore the tail size and rounded tooltip

	m_pTool->m_pMargins->AdjustMarginsByMax(MAX_MARGIN, MAX_MARGIN, MAX_MARGIN, MAX_MARGIN);

	int MarginShift = (int)(((float)m_Round)*0.1464466);

	if(m_Round)
		MarginShift ++;

	m_pTool->m_pMargins->AdjustMarginsByMin(0, 0, 0, 0);
	short right, bottom;
	m_pTool->m_pMargins->get_Right(&right);
	m_pTool->m_pMargins->get_Bottom(&bottom);
	m_pTool->m_pMargins->get_Left(&m_LeftMargin);
	m_pTool->m_pMargins->get_Top(&m_TopMargin);

	m_LeftMargin += MarginShift;
	m_TopMargin += MarginShift;

	m_wndSize.SetSize(MarginShift + m_LeftMargin + m_htmlSize.cx + right + (m_bHasBorder?2:0), MarginShift + m_TopMargin + m_htmlSize.cy + bottom + (m_bHasBorder?2:0) + m_TailHeight);
}

BOOL CToolWnd::Destroy()
{
	BOOL result = FALSE;
	if(m_hWnd)
	{
		if(m_Timer)
		{
			KillTimer(m_Timer);
			m_Timer = NULL;
		}
		if(m_bGotTimer)
		{
			g_Timer->KillTimer(m_hWnd);
			m_bGotTimer = false;
		}

		if(m_hRgn)
		{
			SetWindowRgn(NULL);
			m_hRgn = NULL;
		}

		HideToolTip(true);
		DestroyWindow();
		m_hWnd = NULL;
		result = TRUE;
	}

	if(m_MemoryBMP)
	{
		DeleteObject(m_MemoryBMP);
		m_MemoryBMP = NULL;
	}
	return result;
}


void CToolWnd::SetState(VXPTooltipState state)
{
	if(m_State != state)
	{
		m_State = state;
		m_pTool->Fire_OnStateChanged(state);

		// Setting up or removing the mouse hook:
		if((m_State == ttsShowing || m_State == ttsShown) && !m_bMouseHooked)
			SetupMouseHook();
		else
			if((m_State == ttsHiding || m_State == ttsHidden) && m_bMouseHooked)
				RemoveMouseHook();
	}
}

void CToolWnd::UpdateRegion()
{
	if(!m_bSourceChanged && m_hRgn && !m_bHasTail)
		return;

	if(m_hRgn)
	{
		SetWindowRgn(NULL);
		m_hRgn = NULL;
	}
	m_hRgn = GetNewRegion();
	SetWindowRgn(m_hRgn);
	m_bUpdatedRegion = true;
}

// This method can be called from any state
// of the tooltip
void CToolWnd::HideToolTip(bool NoFadingEffect)
{
	if(NoFadingEffect && m_State != ttsHidden)
	{
		SoundStop();
		ShowWindow(SW_HIDE);
		SetState(ttsHidden);
		return;
	}

	if(m_State == ttsShown || m_State == ttsShowing)
	{
		if(m_FadeHide && m_fnSetLayeredWindowAttributes && m_bLayered)
		{
			if(m_State == ttsShown)
				m_uFade = m_Transparency;
			SetState(ttsHiding);
		}
		else
		{
			SoundStop();
			ShowWindow(SW_HIDE);
			SetState(ttsHidden);
		}
	}
	if(m_State == ttsHidden) //new
		m_uFade = 0; //new
}

// This method is called from the interface at any moment,
// even when the tooltip is shown or being animated (faded).
void CToolWnd::ShowToolTip(short x, short y)
{
	Create(); // Creating the tooltip

	if(!m_bAutohide && !m_bTraceMode)
		MoveToolTip(x, y);

	if(m_State == ttsHidden || m_State == ttsHiding)
	{
		if(m_State == ttsHidden)
		{
			m_uFade = 0; // very new

			if(!Display(x, y)) // If refused to display for some reasons
				return;
		}
		if(m_fnSetLayeredWindowAttributes)
		{
			if(m_FadeShow)
				SetState(ttsShowing);
			else
			{
				if(m_State == ttsHiding && m_bLayered)
					m_fnSetLayeredWindowAttributes(m_hWnd, 0, m_Transparency, LWA_ALPHA);
				SetState(ttsShown);
			}
		}
		else
			SetState(ttsShown);
	}
	
	// The tooltip is garanteed to become visible
	// after this, so we start playing sound:
	SoundPlay();
}

void CToolWnd::MoveToolTip(short x, short y)
{
	if(m_State != ttsHidden)
	{
		bool Updated = UpdatePositioning(x, y);
		if(Updated)
		{
			// We need first to hide the tooltip, if fading effect is not used.
			// This is to fix a glitch of when the tooltip window flips from
			// top to the bottom, and it has tail but no transparency such
			// window would leave a trace on the screen.
			bool bHidden = true;
			if(m_fnSetLayeredWindowAttributes && m_bLayered)
				bHidden = false;
			else
				ShowWindow(SW_HIDE);
			// ***************************************************************

			if(!bHidden)
				SetWindowPos(HWND_BOTTOM, 0, 0, 0, 0, SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOMOVE);

			UpdateRegion();

			// Now we can show the tooltip window again, if it was hidden:
			if(bHidden)
				SetWindowPos(HWND_TOPMOST, 0, 0, 0, 0, SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOMOVE|SWP_SHOWWINDOW);
		}
		if(m_LastMove.x != x || m_LastMove.y != y)
		{
			m_LastMove.x = x;
			m_LastMove.y = y;
			SetWindowPos(HWND_TOPMOST, x, y, 0, 0, SWP_NOACTIVATE|SWP_NOSIZE);
		}
		if(Updated)
			Invalidate();
	}
}

bool CToolWnd::Display(short x, short y)
{
	m_LastPos.x = x;
	m_LastPos.y = y;

	UpdateSizes();

	// Verify if such tooltip should be displayed at all.
	if(!m_bShowWhenEmpty && m_bEmpty)
		return false;

// Step1: Make sure the coordinates are within the screen

	if(x > m_screen.cx)
		x = m_screen.cx;
	if(y > m_screen.cy)
		y = m_screen.cy;

// Step2: Saving the last cursor position:
	
	GetCursorPos(&m_LastCursor);

	if(m_fnSetLayeredWindowAttributes && m_bLayered)
	{
		if(m_FadeShow)
		{
			if(m_State == ttsHidden)
				m_uFade = 0; // Initiate the fading counter
			m_fnSetLayeredWindowAttributes(m_hWnd, 0, (BYTE)m_uFade, LWA_ALPHA);
		}
		else
			m_fnSetLayeredWindowAttributes(m_hWnd, 0, m_Transparency, LWA_ALPHA);
	}

// Step3: Showing the window:

// Trying to avoid recreating the regions when 

	if(!m_hRgn || m_LastWndSize != m_wndSize)
	{
		m_LastWndSize = m_wndSize;
		UpdateRegion();
	}

	if(UpdatePositioning(x, y))
		UpdateRegion();
	
	SetWindowPos(HWND_TOPMOST, x, y, m_wndSize.cx, m_wndSize.cy, SWP_NOACTIVATE|SWP_SHOWWINDOW|SWP_NOCOPYBITS);
	
	return true;
}

#define SOME_OFFSET	1

void CToolWnd::SetRegions(LPPOINT TailPoints, LPRECT BaseRect)
{
	int x = m_wndSize.cx - TP_OFFSET - TP_WIDTH;
	int y = min(m_wndSize.cx, m_wndSize.cy - TP_HEIGHT);
	if(x > y / 2)
		x = y / 2;

	switch(m_positioning)
	{
	case tpLeftTop:
		{
			TailPoints[0].x = TP_OFFSET;
			TailPoints[0].y = TP_HEIGHT + x;

			TailPoints[1].x = TP_OFFSET;
			TailPoints[1].y = 0;

			TailPoints[2].x = TP_OFFSET + TP_WIDTH + x;
			TailPoints[2].y = TP_HEIGHT + x;

			BaseRect->left = 0;
			BaseRect->top = m_TailHeight;
			BaseRect->right = m_wndSize.cx + SOME_OFFSET;
			BaseRect->bottom = m_wndSize.cy + SOME_OFFSET;

			break;
		}
	case tpRightTop:
		{
			TailPoints[0].x = m_wndSize.cx - TP_OFFSET;
			TailPoints[0].y = TP_HEIGHT + x;

			TailPoints[1].x = m_wndSize.cx - TP_OFFSET;
			TailPoints[1].y = 0;

			TailPoints[2].x = m_wndSize.cx - TP_OFFSET - TP_WIDTH - x;
			TailPoints[2].y = TP_HEIGHT + x;

			BaseRect->left = 0;
			BaseRect->top = m_TailHeight;
			BaseRect->right = m_wndSize.cx + SOME_OFFSET;
			BaseRect->bottom = m_wndSize.cy + SOME_OFFSET;

			break;
		}

	case tpLeftBottom:
		{
			TailPoints[0].x = TP_OFFSET;
			TailPoints[0].y = m_wndSize.cy - TP_HEIGHT - x;

			TailPoints[1].x = TP_OFFSET;
			TailPoints[1].y = m_wndSize.cy;

			TailPoints[2].x = TP_OFFSET + TP_WIDTH + x;
			TailPoints[2].y = m_wndSize.cy - TP_HEIGHT - x;

			BaseRect->left = 0;
			BaseRect->top = 0;
			BaseRect->right = m_wndSize.cx + SOME_OFFSET;
			BaseRect->bottom = m_wndSize.cy - m_TailHeight + SOME_OFFSET;

			break;
		}

	case tpRightBottom:
		{
			TailPoints[0].x = m_wndSize.cx - TP_OFFSET;
			TailPoints[0].y = m_wndSize.cy - TP_HEIGHT - x;

			TailPoints[1].x = m_wndSize.cx - TP_OFFSET;
			TailPoints[1].y = m_wndSize.cy;

			TailPoints[2].x = m_wndSize.cx - TP_OFFSET - TP_WIDTH - x;
			TailPoints[2].y = m_wndSize.cy - TP_HEIGHT - x;

			BaseRect->left = 0;
			BaseRect->top = 0;
			BaseRect->right = m_wndSize.cx + SOME_OFFSET;
			BaseRect->bottom = m_wndSize.cy - m_TailHeight + SOME_OFFSET;

			break;
		}

	default:
		break;
	}
}

// This function must create a new region according to m_bHasTail,
// positioning on the screen
HRGN CToolWnd::GetNewRegion()
{
	POINT TailPoints[3];
	RECT BaseRect;
	SetRegions(TailPoints, &BaseRect);

	if(!m_bHasTail)
	{
#ifdef _MSLU
		if(m_Round || g_bNTCompitable)
			return CreateRoundRectRgn(BaseRect.left, BaseRect.top, BaseRect.right, BaseRect.bottom, m_Round, m_Round);
		return CreateRectRgn(BaseRect.left, BaseRect.top, BaseRect.right - 1, BaseRect.bottom - 1);
#else
		return CreateRoundRectRgn(BaseRect.left, BaseRect.top, BaseRect.right, BaseRect.bottom, m_Round, m_Round);
#endif
	}
		
	HRGN TailRegion = CreatePolygonRgn(TailPoints, 3, ALTERNATE);
#ifdef _MSLU
	HRGN BaseRegion;
	if(m_Round || g_bNTCompitable)
		BaseRegion = CreateRoundRectRgn(BaseRect.left, BaseRect.top, BaseRect.right, BaseRect.bottom, m_Round, m_Round);
	else
		BaseRegion = CreateRectRgn(BaseRect.left, BaseRect.top, BaseRect.right - 1, BaseRect.bottom - 1);
#else
	HRGN BaseRegion = CreateRoundRectRgn(BaseRect.left, BaseRect.top, BaseRect.right, BaseRect.bottom, m_Round, m_Round);
#endif
	HRGN ResultingRegion = CreateRectRgn(0,0,0,0);

	CombineRgn(ResultingRegion, BaseRegion, TailRegion, RGN_OR);

	DeleteObject(TailRegion);
	DeleteObject(BaseRegion);

	return ResultingRegion;
}

// Updates information about positioning and returns true,
// if the positioning has changed.
// It takes the cursor position and changes to the real
// positions for the tooltip window.
bool CToolWnd::UpdatePositioning(short & x, short & y)
{
	bool bRightMost = false;	// Assuming this is not the right-most monitor
	bool bBottomMost = false;	// Assuming this is not the bottom-most monitor

 	if(GetSystemMetrics(SM_CMONITORS) > 1) // If there's more than one monitor
	{
		if(m_screen.cx + m_VirtPos.cx <= m_MainScreen.cx) // If this is the right-most monitor
			bRightMost = true;
		if(m_screen.cy + m_VirtPos.cy <= m_MainScreen.cy) // If this is the bottom-most monitor
			bBottomMost = true;
	}

	CTailPositioning OldPositioning = m_positioning;
	bool X_Changed = false;
	m_positioning = tpLeftTop;
	
	int shift = 0;
	if(OldPositioning == tpLeftTop || OldPositioning == tpRightTop)
		shift = CURSOR_WIDTH;

	long VirtualScreenWidth = m_screen.cx;
	if(bRightMost)
		VirtualScreenWidth += m_VirtPos.cx;

	if((x + m_wndSize.cx - TP_OFFSET + shift) > VirtualScreenWidth && x > VirtualScreenWidth / 2)
	{
		m_positioning = tpRightTop;
		X_Changed = true;
	}

	long VirtualScreenHeight = m_screen.cy;
	if(bBottomMost)
		VirtualScreenHeight += m_VirtPos.cy;

	if(y + m_wndSize.cy + CURSOR_HEIGHT > VirtualScreenHeight && y > VirtualScreenHeight / 2)
	{
		if(X_Changed)
			m_positioning = tpRightBottom;
		else
			m_positioning = tpLeftBottom;
	}

	switch(m_positioning)
	{
	case tpLeftTop:
		{
			x -= TP_OFFSET - CURSOR_WIDTH;
			y += CURSOR_HEIGHT;
			break;
		}
	case tpRightTop:
		{
			x += TP_OFFSET - m_wndSize.cx + CURSOR_WIDTH;
			y += CURSOR_HEIGHT;
			break;
		}
	case tpLeftBottom:
		{
			x -= TP_OFFSET;
			y -= m_wndSize.cy + CURSOR_SHIFT;
			break;
		}
	case tpRightBottom:
		{
			x += TP_OFFSET - m_wndSize.cx;
			y -= m_wndSize.cy + CURSOR_SHIFT;
			break;
		}
	default:
		break;
	}

	return (m_positioning != OldPositioning);
}

void CToolWnd::UpdateShadowAttribute(bool HasShadow)
{
	if(HasShadow == m_bHasShadow || (HasShadow && !m_bSupportsShadow))
		return;

	m_bHasShadow = HasShadow;
	
	Destroy();

	Create();
}

void CToolWnd::OnSourceChangedNotify()
{
	m_bSourceChanged = true;
	HideToolTip(true);
}

void CToolWnd::SoundStop()
{
	if(m_bSoundPlaying)
	{
		m_bSoundPlaying = false;
		m_pTool->Fire_OnSoundStop(m_pTool->m_pSound);
		m_LastFadingPercent = 0;
	}
}

void CToolWnd::SoundAdjust()
{
	if(m_bSoundPlaying)
	{
		// Calculating the fading percent:
		float FadingPercent = 0;
		if(m_uFade >= m_Transparency)
			FadingPercent = 100;
		else
			if(m_Transparency)
				FadingPercent = m_uFade * 100/m_Transparency;
		
		// Calculating whether we can call the event,
		// depending on the value of m_FadeSound:

		float difference = FadingPercent - m_LastFadingPercent;
		if(difference < 0)
			difference = -difference;
		bool cancall = (difference * m_FadeSound >= 100) || (difference > 0 && (FadingPercent == 100));
		if(cancall)
		{
			m_LastFadingPercent = FadingPercent;
			m_pTool->Fire_OnSoundAdjust(m_pTool->m_pSound, FadingPercent);
		}
	}
}

void CToolWnd::SoundPlay()
{
	if(!m_bSoundPlaying)
	{
		m_LastFadingPercent = 0;
		VARIANT_BOOL started_playing = FALSE;
		m_pTool->Fire_OnSoundPlay(m_pTool->m_pSound, &started_playing);
		if(started_playing)
			m_bSoundPlaying = true;
	}
}

void CToolWnd::SetTheme(VXPTooltipTheme NewTheme)
{
	struct CTooltipTheme
	{
		bool m_bHasBorder;
		bool m_bHasShadow;
		bool m_bHasTail;
		short m_FadeShow;
		short m_FadeHide;
		BYTE m_Transparency;
		BYTE m_Round;
	};

	CTooltipTheme Default =	{true, true, false, 500, 500, MAX_VISIBILITY, 0};
	CTooltipTheme Zephyr  =	{false, true, false, 1000, 1000, 185, 0};
	CTooltipTheme Classic =	{true, false, false, 0, 0, MAX_VISIBILITY, 0};
	CTooltipTheme Callout =	{true, true, true, 0, 0, MAX_VISIBILITY, 15};
	CTooltipTheme Slick =	{true, true, true, 1, 1, MAX_VISIBILITY, 3};

	CTooltipTheme * pTheme = NULL;
	switch(NewTheme)
	{
	case ttDefault:
		{
			pTheme = &Default;
			break;
		}
	case ttZephyr:
		{
			pTheme = &Zephyr;
			break;
		}
	case ttClassic:
		{
			pTheme = &Classic;
			break;
		}
	case ttCallout:
		{
			pTheme = &Callout;
			break;
		}
	case ttSlick:
		{
			pTheme = &Slick;
			break;
		}
	case ttCustom:
		{
			m_Theme = ttCustom;
			break;
		}
	default:
		break;
	}

	if(!pTheme)
		return;

	m_bHasBorder = pTheme->m_bHasBorder;
	m_FadeShow = pTheme->m_FadeShow;
	m_FadeHide = pTheme->m_FadeHide;
	m_Transparency = pTheme->m_Transparency;
	m_Round = pTheme->m_Round;
	m_bHasTail = pTheme->m_bHasTail;
	m_TailHeight = m_bHasTail?TP_HEIGHT:0;

	m_Theme = NewTheme;

	if(m_hWnd) // Because is first called from the constructor
	{
		HideToolTip(true);
		OnSourceChangedNotify();
		UpdateShadowAttribute(pTheme->m_bHasShadow);
	}
	else
		m_bHasShadow = pTheme->m_bHasShadow;
}

LRESULT CToolWnd::OnWndMsg(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled)
{
	m_bCanCall = true;
	if(!IsSubscribed(uMsg))
	{
		if(uMsg == WM_ERASEBKGND)
			return 1;
		return CallDefWndProc(uMsg, wParam, lParam);
	}

	VARIANT_BOOL bCallDefWndProc = TRUE;
	LRESULT result = 0;
	m_pTool->Fire_OnWndMsg(uMsg, wParam, lParam, &bCallDefWndProc, &result);
	if(!bCallDefWndProc)
	{
		if(uMsg == WM_ERASEBKGND)
			return 1;
		return result;
	}
	if(m_bCanCall)
	{
		if(uMsg == WM_ERASEBKGND)
			return 1;
		return CallDefWndProc(uMsg, wParam, lParam);
	}
	if(uMsg == WM_ERASEBKGND)
		return 1;
	return result;
}

LRESULT CToolWnd::CallDefWndProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if(m_bCanCall)
	{
		m_bCanCall = false;
		if(GetCurrentMessage()->message == WM_PAINT)
			return OnPaint();
		return DefWindowProc(uMsg, wParam, lParam);
	}
	return 0L;
}

BOOL CToolWnd::Subscribe(UINT uMsg)
{
	int i = 0;
	while(i < MSG_SIZE && m_Msg[i])
		i ++;
	if(i == MSG_SIZE)
		return FALSE;
	m_Msg[i] = uMsg;
	return TRUE;
}

BOOL CToolWnd::UnSubscribe(UINT uMsg)
{
	int i = 0;
	if(!uMsg)
	{
		for(i;i < MSG_SIZE;i++)
			m_Msg[i] = 0;
		return TRUE;
	}
	while(i < MSG_SIZE && m_Msg[i] != uMsg)
		i ++;
	if(i == MSG_SIZE)
		return FALSE;
	
	m_Msg[i] = 0;

	int k = MSG_SIZE - 1;
	while(!m_Msg[k] && k)
		k --;
	if(k)
	{
		m_Msg[i] = m_Msg[k];
		m_Msg[k] = 0;
	}
	return TRUE;
}

bool CToolWnd::IsSubscribed(UINT uMsg)
{
	int i = 0;
	while(i < MSG_SIZE && m_Msg[i] != uMsg && m_Msg[i])
		i ++;
	if(i == MSG_SIZE || !m_Msg[i])
		return false;
	return true;
}

void CToolWnd::UpdateLayering()
{
	if((m_FadeHide || m_FadeShow || (m_Transparency!=255)) && !m_bLayered)
	{
		if(::IsWindow(m_hWnd))
			ModifyStyleEx(0, WS_EX_LAYERED);
		m_bLayered = true;
	}
	else
		if(!m_FadeHide && !m_FadeShow && (m_Transparency==255) && m_bLayered)
		{
			if(::IsWindow(m_hWnd))
				ModifyStyleEx(WS_EX_LAYERED, 0);
			m_bLayered = false;
		}
}

CToolWnd * CToolWnd::g_pMouseHookClients[MOUSEHOOK_CLIENT_NUMBER] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
HHOOK CToolWnd::g_hMouseHook = NULL;
long CToolWnd::g_nMouseHooks = 0; // The number of current mouse hooks

#ifndef WM_XBUTTONDOWN
#define WM_XBUTTONDOWN                  0x020B
#define WM_XBUTTONUP                    0x020C
#define WM_XBUTTONDBLCLK                0x020D
#endif

LRESULT CALLBACK CToolWnd::MouseHookProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	if(nCode < 0)
		return CallNextHookEx(g_hMouseHook, nCode, wParam, lParam);

	// Processing only mouse-click events:
	if(nCode==HC_ACTION && (wParam == WM_LBUTTONDBLCLK || wParam == WM_LBUTTONDOWN || wParam == WM_LBUTTONUP ||
		wParam == WM_MBUTTONDBLCLK || wParam == WM_MBUTTONDOWN || wParam == WM_MBUTTONUP ||
		wParam == WM_RBUTTONDBLCLK || wParam == WM_RBUTTONDOWN || wParam == WM_RBUTTONUP ||
		wParam == WM_XBUTTONDBLCLK || wParam == WM_XBUTTONDOWN || wParam == WM_XBUTTONUP))
	{
		MOUSEHOOKSTRUCT * pMouseInfo = (MOUSEHOOKSTRUCT*)lParam;

		// Now getting the correct hit-test code, as pMouseInfo->wHitTestCode
		// happens not to contain the correct code:
		LRESULT HitTestCode = ::SendMessage(pMouseInfo->hwnd, WM_NCHITTEST, 0, MAKELONG(pMouseInfo->pt.x, pMouseInfo->pt.y));

		int I = 0;
		while(I < MOUSEHOOK_CLIENT_NUMBER)
		{
			if(g_pMouseHookClients[I])
				g_pMouseHookClients[I]->OnMouseClick(pMouseInfo->pt.x, pMouseInfo->pt.y, wParam, pMouseInfo->hwnd, HitTestCode);
			I ++;
		}
	}

	return CallNextHookEx(g_hMouseHook, nCode, wParam, lParam);
}

void CToolWnd::SetupMouseHook()
{
	if(g_nMouseHooks == MOUSEHOOK_CLIENT_NUMBER) // Cannot connect more than 10 clients
		return;

	if(!g_hMouseHook)
		g_hMouseHook = SetWindowsHookEx(WH_MOUSE, MouseHookProc, NULL, GetCurrentThreadId());
	int I = 0;
	while(g_pMouseHookClients[I] && I < MOUSEHOOK_CLIENT_NUMBER)
		I ++;

	if(I < MOUSEHOOK_CLIENT_NUMBER)
	{
		g_pMouseHookClients[I] = this;
		m_bMouseHooked = true;
		g_nMouseHooks ++;
	}
}

void CToolWnd::RemoveMouseHook()
{
	int I = 0;
	while(g_pMouseHookClients[I] != this && I < MOUSEHOOK_CLIENT_NUMBER)
		I ++;

	if(I < MOUSEHOOK_CLIENT_NUMBER)
	{
		g_pMouseHookClients[I] = NULL;
		m_bMouseHooked = false;
		g_nMouseHooks --;
	}

	if(!g_nMouseHooks)
	{
		UnhookWindowsHookEx(g_hMouseHook);
		g_hMouseHook = NULL;
	}
}

void CToolWnd::OnMouseClick(long x, long y, UINT uMsg, HWND hWnd, UINT wHitTestCode)
{
	VARIANT_BOOL bProcessed = FALSE;
	m_pTool->Fire_OnMouseClick(x, y, uMsg, (long)hWnd, wHitTestCode, &bProcessed);
	if(bProcessed)
		return;
	switch(m_MouseClickResponse)
	{
	case mcrNoResponse:
		{
			break;
		}
	case mcrHideWhenShown:
		{
			if(m_State == ttsShown)
				HideToolTip();
			break;
		}
	case mcrHideAlways:
		{
			HideToolTip();
			break;
		}
	case mcrMoveToCursor:
		{
			POINT p;
			GetCursorPos(&p);
			ShowToolTip((short)p.x, (short)p.y);
			break;
		}
	default:
		break;
	}
}
