/*
 * VXPLib COMPONENT
 *
 * Module: ToolWnd.h - Declaration of class CToolWnd
 *
 * Synopsis: Class CToolWnd implements all the logic of the
 * Tooltip Window with all its methods and properties,
 * while class CVXPTooltip merely forwards all its calls
 * into this class.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#if !defined(AFX_TOOLWND_H__0B25360B_F820_48F4_BA52_80A037A66360__INCLUDED_)
#define AFX_TOOLWND_H__0B25360B_F820_48F4_BA52_80A037A66360__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "atlwin.h"
#include "String.h"
#include "VXPLib.h"
#include "Size.h"
#include "Timer.h"

#define MSG_SIZE	20	// Maximum number of messages that we can subscribe for

#define MAX_VISIBILITY	255	// The maximum layer visibility

// Tooltip tail parameters:
#define TP_OFFSET	10	// Offset for the tooltip window width

// Note: it might be not possible to keep the correct
// functionality, if TP_WIDTH and TP_HEIGHT are not equal.
#define TP_WIDTH	20	// The tail width
#define TP_HEIGHT	20	// The tail height

#define CURSOR_HEIGHT	21
#define CURSOR_WIDTH	10
#define CURSOR_SHIFT	6	// Horizontal shift for the tooltip when the meouse cursor
							// is underneath the tooltip

// Minimum width and hight for a tooltip when it has m_bHasTail set ON.
#define MIN_WIDTH	TP_OFFSET * 2 + TP_WIDTH
#define MIN_HEIGHT	1

#define MOUSEHOOK_CLIENT_NUMBER	10

class CToolWnd;

extern CToolWnd * g_ToolWindow;

class CToolWnd:public CWindowImpl<CToolWnd, CWindow>
{
	enum CTailPositioning
	{
		tpLeftTop, tpRightTop, tpLeftBottom, tpRightBottom
	};

	friend class CVXPTooltip;
	friend class CVXPTooltipManager;
public:
	void UpdateLayering();
	void OnSourceChangedNotify();
	void Destruct();
	void Construct(CVXPTooltip * pTool);

	static CWndClassInfo WithShadow_Class;
	static CWndClassInfo WithoutShadow_Class;

	static CWndClassInfo & GetWndClassInfo()
	{

		if(g_ToolWindow->m_bHasShadow)
			return WithShadow_Class;
		else
			return WithoutShadow_Class;
	}

	CToolWnd();

	void ShowToolTip(short x, short y);
	BOOL Create();
	BOOL Destroy();
	void HideToolTip(bool NoFadingEffect = false);
	void SetTheme(VXPTooltipTheme NewTheme);

protected:

	LRESULT OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled);
	LRESULT OnVXPTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled);
	LRESULT OnPaint();
	LRESULT OnWndMsg(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled);

BEGIN_MSG_MAP(CToolWnd)

	MESSAGE_HANDLER(WM_TIMER, OnTimer)
	MESSAGE_HANDLER(WM_VXPTIMER, OnVXPTimer)

	bHandled = TRUE;
	lResult = OnWndMsg(uMsg, wParam, lParam, bHandled);
	if(bHandled)
		return TRUE;

END_MSG_MAP()

private:
	void OnMouseClick(long x, long y, UINT uMsg, HWND hWnd, UINT wHitTestCode);
	static LRESULT CALLBACK MouseHookProc(int nCode, WPARAM wParam, LPARAM lParam);

	void RemoveMouseHook();
	void SetupMouseHook();
	void SoundPlay();
	void SoundAdjust();
	void SoundStop();
	void UpdateShadowAttribute(bool HasShadow);
	void SetRegions(LPPOINT TailPoints, LPRECT BaseRect);
	bool UpdatePositioning(short & x, short & y);
	HRGN GetNewRegion();
	void MoveToolTip(short x, short y);
	bool Display(short x, short y);
	void UpdateRegion();
	void SetState(VXPTooltipState state);
	void UpdateSizes();
	void DrawBackground(HDC dc, bool HasBackground);
	LRESULT CallDefWndProc(UINT uMsg, WPARAM wParam, LPARAM lParam);
	typedef BOOL (WINAPI* FN_SET_LAYERED_WINDOW_ATTRIBUTES)(HWND,COLORREF,BYTE,DWORD);

	BOOL Subscribe(UINT uMsg);
	BOOL UnSubscribe(UINT uMsg);
	bool IsSubscribed(UINT uMsg);

	FN_SET_LAYERED_WINDOW_ATTRIBUTES m_fnSetLayeredWindowAttributes;

	static CToolWnd * g_pMouseHookClients[MOUSEHOOK_CLIENT_NUMBER];
	static HHOOK g_hMouseHook;
	static long g_nMouseHooks; // The number of current mouse hooks

	void CreateTooltipClasses();
	POINT m_LastCursor;

	float m_uFade;

// These three parameters are updated inside UpdateSizes() only;
// They are necessary to provide multi-monitor support
	CSize m_screen;		// Screen size in pixels of the virtual screen
	CSize m_MainScreen;	// Size of the main monitor screen
	CSize m_VirtPos;	// Position of the virtual screen relative to the main monitor
	
	CSize m_wndSize;	// Size of the whole tooltip window, including
						// the tail and whatever else.

	CSize m_htmlSize;	// Size of the HTML context
// ************************************************************

// Drawing margins calculated within UpdateSizes().
// The two reasons for having them here locally instead of
// reading everytime from IVXPMargins:
// 1. Drawing speed
// 2. The user can change the margins at any moment, so we won't
// even know about it. And that must not cause the tooltip to redraw
// itself, because it would need IVXPQuickHTML to recalculate everything,
// which is too slow then.
	short m_LeftMargin;
	short m_TopMargin;

	CVXPTooltip * m_pTool;

	HRGN m_hRgn;		// The window region

	bool m_bSoundPlaying; // Contains true, if the sound is being played.
	float m_LastFadingPercent;

// These are to be set and read by the interface class:

	VXPMouseClickResponse m_MouseClickResponse;
	bool m_bShowWhenEmpty;
	bool m_bTraceMode;
	short m_FadeSound;
	bool m_bAutohide;
	short m_FadeShow;
	short m_FadeHide;
	BYTE m_Transparency;
	bool m_bHasTail;
	BYTE m_Round;	// How much the window is rounded. The value of 0
					// inidicates no rounding at all

	bool m_bMouseHooked;
	
	HWND m_hSyncWnd;	// The window to syncronize showing/hiding the tooltip;
	POINT m_LastPos;	// {x, y} of where the tooltip was shown the last time;

	VXPTooltipTheme m_Theme;

	VXPTooltipState m_State;
	bool m_bHasBorder;
	COLORREF m_BorderColor;
	bool m_bHasShadow;

	CTailPositioning m_positioning;
	
	POINT m_LastMove;	// This variable helps avoid moving the tooltip
						// when m_bTraceMode is true, but the mouse cursor
						// position doesn't change.
	CSize m_LastWndSize;

	bool m_bEmpty;

	short m_TailHeight;
	
	bool m_bSupportsShadow;

	bool m_bSourceChanged; // Indicates whether the HTML source contents has changed
	
	bool m_bUpdatedRegion;

	HBITMAP m_MemoryBMP;

	bool m_bCanCall;

	UINT m_Msg[MSG_SIZE];

	bool m_bLayered; // Contains true when the tooltip window is layered

	bool m_bGotTimer;	// Contains True, if the timer has succeeded to initialize
	UINT m_Timer;		// Timer identificator returned by function SetTimer
};

#endif // !defined(AFX_TOOLWND_H__0B25360B_F820_48F4_BA52_80A037A66360__INCLUDED_)
