/*
 * VXPLib COMPONENT
 *
 * Module: VXPCmdToolList.cpp - Implementation of class CVXPCmdToolList
 *
 * Synopsis: Class CVXPCmdToolList implements methods and properties
 * of interface IVXPCmdToolList.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "VXPLib.h"
#include "VXPCmdToolList.h"
#include "VXPTooltipManager.h"

/////////////////////////////////////////////////////////////////////////////
// CVXPCmdToolList

HRESULT CVXPCmdToolList::FinalConstruct()
{
	m_pList = NULL;
	return S_OK;
}

HRESULT CVXPCmdToolList::FinalRelease()
{
	DELETE_COLLECTION(m_pList)
	return S_OK;
}

STDMETHODIMP CVXPCmdToolList::Add(long CommandID, BSTR ToolTip)
{
	VXPLIB_BEGIN

	ADD_COLLECTION(m_pList, new CToolCmdList(CommandID, ToolTip))
	
	VXPLIB_END
}

STDMETHODIMP CVXPCmdToolList::Clear()
{
	VXPLIB_BEGIN

	DELETE_COLLECTION(m_pList)
	
	VXPLIB_END
}

STDMETHODIMP CVXPCmdToolList::HasTool(long CommandID, VARIANT_BOOL *result)
{
	VXPLIB_BEGIN

	CToolCmdList * A = m_pList;
	*result = FALSE;
	while(A)
	{
		if(A->m_ID == CommandID)
		{
			*result = TRUE;
			break;
		}
		A = (CToolCmdList*)A->Next;
	}

	VXPLIB_END
}

STDMETHODIMP CVXPCmdToolList::IsActive(long CommandID, VARIANT_BOOL *result)
{
	VXPLIB_BEGIN

	CToolCmdList * A = m_pList;
	*result = FALSE;
	while(A)
	{
		if(A->m_ID == CommandID)
		{
			*result = A->m_bActive?TRUE:FALSE;
			break;
		}
		A = (CToolCmdList*)A->Next;
	}

	VXPLIB_END
}

STDMETHODIMP CVXPCmdToolList::Modify(long CommandID, BSTR ToolTip, VARIANT_BOOL *success)
{
	VXPLIB_BEGIN

	CToolCmdList * A = m_pList;
	*success = FALSE;
	while(A)
	{
		if(A->m_ID == CommandID)
		{
			*success = TRUE;
			A->m_text = ToolTip;
			m_pManager->ToolModifiedNotify(A);
			break;
		}
		A = (CToolCmdList*)A->Next;
	}

	VXPLIB_END
}

STDMETHODIMP CVXPCmdToolList::Remove(long CommandID, VARIANT_BOOL *success)
{
	VXPLIB_BEGIN

	CToolCmdList * A = m_pList;
	*success = FALSE;
	while(A)
	{
		if(A->m_ID == CommandID)
		{
			*success = TRUE;
			CToolCmdList * B = (CToolCmdList*)A->GetChain();
			if(A == B)
				DELETE_COLLECTION(m_pList)
			else
			{
				m_pList = B;
				A->Remove();
			}
			break;
		}
		A = (CToolCmdList*)A->Next;
	}

	VXPLIB_END
}

STDMETHODIMP CVXPCmdToolList::SetActive(long CommandID, VARIANT_BOOL Active, VARIANT_BOOL *success)
{
	VXPLIB_BEGIN

	CToolCmdList * A = m_pList;
	*success = FALSE;
	while(A)
	{
		if(A->m_ID == CommandID)
		{
			A->m_bActive = Active?true:false;
			*success = TRUE;
			break;
		}
		A = (CToolCmdList*)A->Next;
	}

	VXPLIB_END
}

STDMETHODIMP CVXPCmdToolList::get_Size(short *pVal)
{
	VXPLIB_BEGIN

	if(m_pList)
		*pVal = m_pList->CountAll();
	else
		*pVal = 0;

	VXPLIB_END
}
