/*
 * VXPLib COMPONENT
 *
 * Module: VXPFont.h - Declaration of class CVXPFont
 *
 * Synopsis: Class CVXPFont implements methods and properties
 * of interface IVXPFont.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#ifndef __VXPFONT_H_
#define __VXPFONT_H_

#include "resource.h"       // main symbols
#include "String.h"

/////////////////////////////////////////////////////////////////////////////
// CVXPFont
class ATL_NO_VTABLE CVXPFont : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CVXPFont, &CLSID_VXPFont>,
	public ISupportErrorInfo,
	public IDispatchImpl<IVXPFont, &IID_IVXPFont, &LIBID_VXPLibrary>,
	public CVXPException<&CLSID_VXPFont, &IID_IVXPFont>
{
public:
	CVXPFont();
	~CVXPFont();

DECLARE_REGISTRY_RESOURCEID(IDR_VXPFONT)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CVXPFont)
	COM_INTERFACE_ENTRY(IVXPFont)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IVXPFont
public:
	STDMETHOD(SetFromHandle)(/*[in]*/ long hFont, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(get_BackColor)(/*[out, retval]*/ OLE_COLOR *pVal);
	STDMETHOD(put_BackColor)(/*[in]*/ OLE_COLOR newVal);
	STDMETHOD(get_Color)(/*[out, retval]*/ OLE_COLOR *pVal);
	STDMETHOD(put_Color)(/*[in]*/ OLE_COLOR newVal);
	STDMETHOD(SetFromSystemFont)(/*[in]*/ VXPSysFont FontIndex, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(get_PitchAndFamily)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_PitchAndFamily)(/*[in]*/ short newVal);
	STDMETHOD(get_Quality)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_Quality)(/*[in]*/ short newVal);
	STDMETHOD(get_ClipPrecision)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_ClipPrecision)(/*[in]*/ short newVal);
	STDMETHOD(get_OutPrecision)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_OutPrecision)(/*[in]*/ short newVal);
	STDMETHOD(get_CharSet)(/*[out, retval]*/ VXPCharSet *pVal);
	STDMETHOD(put_CharSet)(/*[in]*/ VXPCharSet newVal);
	STDMETHOD(get_Escapement)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_Escapement)(/*[in]*/ short newVal);
	STDMETHOD(get_Orientation)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_Orientation)(/*[in]*/ short newVal);
	STDMETHOD(get_StrikeOut)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_StrikeOut)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_Underlined)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_Underlined)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_Italic)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_Italic)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_Weight)(/*[out, retval]*/ VXPFontWeight *pVal);
	STDMETHOD(put_Weight)(/*[in]*/ VXPFontWeight newVal);
	STDMETHOD(get_Size)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_Size)(/*[in]*/ short newVal);
	STDMETHOD(SetDefaults)();
	STDMETHOD(get_hFont)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_Name)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_Name)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_Transparent)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_Transparent)(/*[in]*/ VARIANT_BOOL newVal);

private:
	BOOL SetSystemFont(VXPSysFont FontIndex);
	void SetDefaultParameters();
	void UpdateFont();

	bool m_bChanged;		// True, if the font information has changed
	bool m_bTransparent;	// True, if the font has no background color
	COLORREF m_Color;		// The text color
	COLORREF m_bkColor;		// BackColor of the text, when m_bTransparent = false.
	LOGFONT m_LogFont;		// The font information.
	HFONT m_hFont;			// The font handle.
};

#endif //__VXPFONT_H_
