/*
 * VXPLib COMPONENT
 *
 * Module: VXPLib.cpp - Implementation of the project's exports
 *
 * Synopsis: Implementation of all the functions exported
 * by the component.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "resource.h"
#include <initguid.h>
#include "VXPLib.h"

#include "VXPLib_i.c"
#include "VXPQuickHTML.h"
#include "VXPTooltip.h"
#include "VXPSettings.h"
#include "VXPFont.h"
#include "VXPMargins.h"
#include "VXPTooltipManager.h"
#include "VXPWndToolList.h"
#include "VXPCmdToolList.h"
#include "VXPSound.h"
#include "VXPMenuHook.h"

CComModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
OBJECT_ENTRY(CLSID_VXPQuickHTML, CVXPQuickHTML)
OBJECT_ENTRY(CLSID_VXPTooltip, CVXPTooltip)
OBJECT_ENTRY(CLSID_VXPSettings, CVXPSettings)
OBJECT_ENTRY(CLSID_VXPFont, CVXPFont)
OBJECT_ENTRY(CLSID_VXPMargins, CVXPMargins)
OBJECT_ENTRY(CLSID_VXPTooltipManager, CVXPTooltipManager)
OBJECT_ENTRY(CLSID_VXPWndToolList, CVXPWndToolList)
OBJECT_ENTRY(CLSID_VXPCmdToolList, CVXPCmdToolList)
OBJECT_ENTRY(CLSID_VXPSound, CVXPSound)
OBJECT_ENTRY(CLSID_VXPMenuHook, CVXPMenuHook)
END_OBJECT_MAP()

/////////////////////////////////////////////////////////////////////////////
// DLL Entry Point

HMODULE hGraphics = NULL;
GRADIENT_FILL g_pGradientFillFunc = NULL;
SetMenuInfo g_pSetMenuInfo = NULL;
GetMenuBarInfo g_pGetMenuBarInfo = NULL;
GetLongPathNameWide g_pGetLongPathNameW = NULL;

extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
	InitializeVersionInfo();
    if (dwReason == DLL_PROCESS_ATTACH)
    {
		// Initializing the timer:
		g_Timer = new CTimer;

		// Initializing ATL:
        _Module.Init(ObjectMap, hInstance, &LIBID_VXPLibrary);

		// Disabling simultaneous calls from several threads:
        DisableThreadLibraryCalls(hInstance);

		// We load library MSIMG32.DLL dynamically just to avoid
		// linking with it:
		hGraphics = ::LoadLibrary(_T("msimg32.dll"));
		if(hGraphics)
			g_pGradientFillFunc = (GRADIENT_FILL)GetProcAddress(hGraphics, "GradientFill");

		HMODULE hModule = ::GetModuleHandle(_T("User32.dll"));
		if(hModule)
		{
			g_pSetMenuInfo = (SetMenuInfo)GetProcAddress(hModule, "SetMenuInfo");
			g_pGetMenuBarInfo = (GetMenuBarInfo)GetProcAddress(hModule, "GetMenuBarInfo");
		}
		hModule = ::GetModuleHandle(_T("Kernel32.dll"));
		if(hModule)
			g_pGetLongPathNameW = (GetLongPathNameWide)GetProcAddress(hModule, "GetLongPathNameW");
    }
    else
		if (dwReason == DLL_PROCESS_DETACH)
		{
			g_pGradientFillFunc = NULL;
			::FreeLibrary(hGraphics);
			_Module.Term();
			delete g_Timer;
			g_Timer = NULL;
		}
    return TRUE;    // ok
}

/////////////////////////////////////////////////////////////////////////////
// Used to determine whether the DLL can be unloaded by OLE

STDAPI DllCanUnloadNow(void)
{
    return (_Module.GetLockCount()==0) ? S_OK : S_FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Returns a class factory to create an object of the requested type

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
    return _Module.GetClassObject(rclsid, riid, ppv);
}

/////////////////////////////////////////////////////////////////////////////
// DllRegisterServer - Adds entries to the system registry

STDAPI DllRegisterServer(void)
{
    // registers object, typelib and all interfaces in typelib
    return _Module.RegisterServer(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// DllUnregisterServer - Removes entries from the system registry

STDAPI DllUnregisterServer(void)
{
    return _Module.UnregisterServer(TRUE);
}


