/*
 * VXPLib COMPONENT
 *
 * Module: VXPMargins.cpp - Implementation of class CVXPMargins
 *
 * Synopsis: Class CVXPMargins implements methods and properties
 * of interface IVXPMargins.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "VXPLib.h"
#include "VXPMargins.h"

/////////////////////////////////////////////////////////////////////////////
// CVXPMargins

STDMETHODIMP CVXPMargins::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IVXPMargins
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

CVXPMargins::CVXPMargins()
{
	SetDefaultParameters();
}

HRESULT CVXPMargins::FinalConstruct()
{
	return S_OK;
}

HRESULT CVXPMargins::FinalRelease()
{
	return S_OK;
}

STDMETHODIMP CVXPMargins::get_Left(short *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_nLeft;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMargins::put_Left(short newVal)
{
	VXPLIB_BEGIN

	m_nLeft = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMargins::get_Top(short *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_nTop;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMargins::put_Top(short newVal)
{
	VXPLIB_BEGIN

	m_nTop = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMargins::get_Right(short *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_nRight;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMargins::put_Right(short newVal)
{
	VXPLIB_BEGIN

	m_nRight = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMargins::get_Bottom(short *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_nBottom;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMargins::put_Bottom(short newVal)
{
	VXPLIB_BEGIN

	m_nBottom = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMargins::AdjustMarginsByMin(short MinLeft, short MinTop, short MinRight, short MinBottom)
{
	VXPLIB_BEGIN

	if(m_nLeft < MinLeft)
		m_nLeft = MinLeft;

	if(m_nTop < MinTop)
		m_nTop = MinTop;

	if(m_nRight < MinRight)
		m_nRight = MinRight;

	if(m_nBottom < MinBottom)
		m_nBottom = MinBottom;

	VXPLIB_END
}

STDMETHODIMP CVXPMargins::AdjustMarginsByMax(short MaxLeft, short MaxTop, short MaxRight, short MaxBottom)
{
	VXPLIB_BEGIN

	if(m_nLeft > MaxLeft)
		m_nLeft = MaxLeft;

	if(m_nTop > MaxTop)
		m_nTop = MaxTop;

	if(m_nRight > MaxRight)
		m_nRight = MaxRight;

	if(m_nBottom > MaxBottom)
		m_nBottom = MaxBottom;

	VXPLIB_END
}

STDMETHODIMP CVXPMargins::SetDefaults()
{
	VXPLIB_BEGIN

	SetDefaultParameters();
	
	VXPLIB_END
}

void CVXPMargins::SetDefaultParameters()
{
	m_nLeft = 0;
	m_nTop = 0;
	m_nRight = 0;
	m_nBottom = 0;
}
