/*
 * VXPLib COMPONENT
 *
 * File: VXPMenuHook.cpp - Implementation of the CVXPMenuHook class
 *
 * Synopsis: This file contains implementation for methods and properties
 * of interface IVXPMenuHook.
 *
 * Copyright (c) 2002-2003, Tooltips.Net L.L.C.
 *
 * Permission to use, copy, modify, but NOT distribute this software without
 * fee is hereby granted, subject to the following terms and conditions also
 * stated in the END USER LICENSE AGREEMENT for VXPLib COM Server DLL.
 * 
 * 1. Source Code of this software may only be used solely for internal
 *    business needs including but not limited to preparation of derivative
 *    works. Under no circumstances the Source Code, either in original or 
 *    modified form, may be distributed or transferred outside of the business
 *    entity or individual person capacity to which this software was licensed.
 *
 * 2. This software and derivative works in its compiled (binary object code)
 *    form may be distributed either commercially or in not-for-profit fashion
 *    to any third party including retail, provided that its use in the 
 *    distributing software package is limited to accessory, secondary means
 *    and neither constitute a main part not determine a purpose of such
 *    distribution. 
 *
 * 3. All copies of the software, derivative works or modified versions,
 *    and any portions thereof, must include this entire copyright and
 *    permission notice, without modification. The full notice must also
 *    appear in supporting documentation.
 *
 */

#include "stdafx.h"
#include "VXPLib.h"
#include "VXPMenuHook.h"

CHookList * CVXPMenuHook::m_pHooks = NULL;

// This flag is to help highlight the very first menu item
// for Main Menu and System Menu when keyboard is used for
// initiating a menu. It is not used for context menus.
#define ID_CODE		0x40000000

/////////////////////////////////////////////////////////////////////////////
// CVXPMenuHook

// Implementation of exceptions support by the interface:
STDMETHODIMP CVXPMenuHook::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IVXPMenuHook
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

// Implementation of the class constructor:
CVXPMenuHook::CVXPMenuHook()
{
	m_hHook = NULL;

	m_WindowTransparency = 255;
	m_MenuTransparency = 255;

	m_MenuBarExtenderID = 0;

	m_OldWndProc = NULL;
	m_hOldMenuHandler = NULL;
	m_pDrawInfo = NULL;

	m_hBGBrush = NULL;
	m_bgColor = GetSysColor(COLOR_MENU);

	m_hSelBrush = NULL;
	m_selColor = GetSysColor(COLOR_HIGHLIGHT);

	m_capColor = GetSysColor(COLOR_BTNFACE);
	m_hCapBrush = NULL;

	m_bStartedDrawing = false;
	m_bStartedMenu = false;

	m_bCaptionSelected = false;

	// Loading the function to support layers ***
	HMODULE hUser32 = GetModuleHandle(_T("USER32.DLL"));
	m_fnSetLayeredWindowAttributes = (FN_SET_LAYERED_WINDOW_ATTRIBUTES)::GetProcAddress(hUser32, "SetLayeredWindowAttributes");
	// ******************************************
}

// Destructor:
CVXPMenuHook::~CVXPMenuHook()
{
	if(m_hBGBrush)
		DeleteObject(m_hBGBrush);
	if(m_hSelBrush)
		DeleteObject(m_hSelBrush);
	if(m_hCapBrush)
		DeleteObject(m_hCapBrush);
}

LRESULT CALLBACK CVXPMenuHook::MenuHook(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	CVXPMenuHook * pHook = CVXPMenuHook::FindHook(hwnd);
	if(pHook)
		return pHook->ProcessHook(hwnd, uMsg, wParam, lParam);
	return 0;
}

CVXPMenuHook * g_Hooker = NULL;

LRESULT CALLBACK CVXPMenuHook::CallWndProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	if(nCode < 0)
		return CallNextHookEx(g_Hooker->m_hHook, nCode, wParam, lParam);

	CWPSTRUCT * pInfo = (CWPSTRUCT*)lParam;
	if((pInfo->message == WM_CREATE || pInfo->message == WM_NCPAINT) && GetCurrentThreadId() == GetWindowThreadProcessId(pInfo->hwnd, NULL))
	{
		CString name;
		name.PostAllocate(GetClassName(pInfo->hwnd, name.PreAllocate(10), 10));
		if(name == _T("#32768")) // If it is a menu window
		{
			UnhookWindowsHookEx(g_Hooker->m_hHook);
			g_Hooker->m_hHook = NULL;
			if(g_Hooker->m_fnSetLayeredWindowAttributes)
			{
				long style = GetWindowLong(pInfo->hwnd, GWL_EXSTYLE);
				if(!(style & WS_EX_LAYERED))
					SetWindowLong(pInfo->hwnd, GWL_EXSTYLE, style | WS_EX_LAYERED);
				g_Hooker->m_fnSetLayeredWindowAttributes(pInfo->hwnd, 0, g_Hooker->m_MenuTransparency, LWA_ALPHA);
			}
		}
	}

	return 0;
}

LRESULT CVXPMenuHook::ProcessHook(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_CONTEXTMENU:
		{
			Fire_OnContextMenu(wParam, LOWORD(lParam), HIWORD(lParam));
			break;
		}
		case WM_INITMENU:
		{
			Fire_OnMenuInit(wParam, -1, true);
			break;
		}
		case WM_INITMENUPOPUP:
		{
			m_bStartedMenu = true;
			if(m_hBGBrush && !g_bWin95orNT4) // If color was set dynamically, and not old OS
			{
				MENUINFO info;
				info.cbSize = sizeof(MENUINFO);
				info.fMask = MIM_BACKGROUND;
				info.hbrBack = m_hBGBrush;
				g_pSetMenuInfo((HMENU)wParam, &info);
			}
			Fire_OnMenuInit(wParam, LOWORD(lParam), HIWORD(lParam));
			g_Hooker = this;
			if(!m_hHook && m_MenuTransparency != 255)
				m_hHook = SetWindowsHookEx(WH_CALLWNDPROC, CallWndProc, NULL, GetCurrentThreadId());

			break;
		}
		case WM_MENUSELECT:
		{
			if(m_bStartedMenu)
			{
				m_bStartedMenu = false;
				if(!(HIWORD(wParam) & MF_MOUSESELECT))
					m_bStartedDrawing = true;
			}
			MENUITEMINFO info;
			info.cbSize = sizeof(MENUITEMINFO);
			info.fMask = MIIM_DATA|MIIM_ID;
			::GetMenuItemInfo((HMENU)lParam, LOWORD(wParam), (HIWORD(wParam)&MF_POPUP)?TRUE:FALSE, &info);
			int ID = info.wID;
			bool bOwnerDrawn = (HIWORD(wParam) & MF_OWNERDRAW)?true:false;
			if(bOwnerDrawn)
			{
				ID = info.dwItemData;
				ID &= ~ID_CODE;
			}
			Fire_OnMenuSelect(lParam, bOwnerDrawn, (HIWORD(wParam)&MF_HILITE)?TRUE:FALSE, (HIWORD(wParam)&MF_POPUP)?TRUE:FALSE, ID);
			break;
		}
		case WM_UNINITMENUPOPUP:
		{
			Fire_OnMenuDone(wParam);
			break;
		}
		case WM_MEASUREITEM:
		{
			LPMEASUREITEMSTRUCT pInfo = (LPMEASUREITEMSTRUCT)lParam;
			if(pInfo->CtlType == ODT_MENU)
			{
				short width = pInfo->itemWidth;
				short height = pInfo->itemHeight;
				long ID = pInfo->itemData;
				ID &= ~ID_CODE;
				Fire_OnMeasureItem(ID, &width, &height);
				pInfo->itemWidth = width;
				pInfo->itemHeight = height;
				return TRUE;
			}
			break;
		}
		case WM_DRAWITEM:
		{
			LPDRAWITEMSTRUCT pInfo = (LPDRAWITEMSTRUCT)lParam;
			if(pInfo->CtlType == ODT_MENU)
			{
				short Flags = 0;
				if((pInfo->itemAction == ODA_SELECT && (pInfo->itemState & ODS_SELECTED)) || m_bStartedDrawing)
					Flags |= mfHiLite;
				if(pInfo->itemState & ODS_CHECKED)
					Flags |= mfChecked;
				if(pInfo->itemState & ODS_GRAYED)
					Flags |= mfDisabled;
				if(pInfo->itemState & ODS_DEFAULT)
					Flags |= mfDefault;
				
				if(pInfo->itemData & ID_CODE)
				{
					m_bCaptionSelected = true;
					if((pInfo->itemState & ODS_HOTLIGHT) || (g_bWin95orNT4 && (pInfo->itemState & ODS_SELECTED)))
						Flags |= mfHiLite;
				}

				m_pDrawInfo = pInfo;
				
				long ID = pInfo->itemData;
				ID &= ~ID_CODE;
				Fire_OnDrawItem((long)pInfo->hwndItem, (long)pInfo->hDC, ID, (VXPMenuItemFlags)Flags, (short)pInfo->rcItem.left, (short)pInfo->rcItem.top, (short)pInfo->rcItem.right, (short)pInfo->rcItem.bottom);
				
				m_pDrawInfo = NULL;

				if(m_MenuBarExtenderID && ID == m_MenuBarExtenderID && !(Flags&mfHiLite))
					DrawExtender(pInfo);

				return TRUE;
			}
			break;
		}
		case WM_SETTINGCHANGE:
			{
				if(!m_hBGBrush)
					m_bgColor = GetSysColor(COLOR_MENU);
				if(!m_hSelBrush)
					m_selColor = GetSysColor(COLOR_HIGHLIGHT);
				if(!m_hCapBrush)
					m_capColor = GetSysColor(COLOR_BTNFACE);

				Fire_OnSettingsChanged();
				break;
			}
		default:
			break;
	}

	LRESULT Result = ::CallWindowProc(m_OldWndProc, hwnd, uMsg, wParam, lParam);

	if(uMsg == WM_DESTROY)
		DetachWindow();

	return Result;
}

STDMETHODIMP CVXPMenuHook::HookWnd(long hWnd, VARIANT_BOOL * success)
{
	VXPLIB_BEGIN

	*success = FALSE;
	if(!hWnd || !::IsWindow((HWND)hWnd) || GetCurrentThreadId() != GetWindowThreadProcessId((HWND)hWnd, NULL))
		AddException(EX_INVALIDHANDLE);
	else
	{
		DetachWindow();
		m_hOldMenuHandler = (HWND)hWnd;
		m_OldWndProc = (WNDPROC)::GetWindowLong(m_hOldMenuHandler, GWL_WNDPROC);
		AddHook(m_hOldMenuHandler);

		::SetWindowLong(m_hOldMenuHandler, GWL_WNDPROC, (long)MenuHook);

		if(m_WindowTransparency != 255)
			put_WindowTransparency(m_WindowTransparency);

		*success = TRUE;
	}

	VXPLIB_END
}

void CVXPMenuHook::DetachWindow()
{
	if(m_hHook)
	{
		UnhookWindowsHookEx(m_hHook);
		m_hHook = NULL;
	}

	if(m_hOldMenuHandler && m_OldWndProc)
	{
		::SetWindowLong(m_hOldMenuHandler, GWL_WNDPROC, (long)m_OldWndProc);
		m_hOldMenuHandler = NULL;
		m_OldWndProc = NULL;
		RemoveHook();
	}
}

STDMETHODIMP CVXPMenuHook::UnHookWnd(long *hWnd)
{
	VXPLIB_BEGIN

	*hWnd = (long)m_hOldMenuHandler;
	DetachWindow();
	
	VXPLIB_END
}

HRESULT CVXPMenuHook::FinalRelease()
{
	DetachWindow();
	return S_OK;
}

CVXPMenuHook * CVXPMenuHook::FindHook(HWND hWnd)
{
	if(!m_pHooks)
		return NULL;

	CHookList * A = (CHookList*)m_pHooks->GetFirst();
	while(A)
	{
		if(A->m_hWnd == hWnd)
			return A->m_pHook;
		A = (CHookList*)A->Next;
	}

	return NULL;
}

void CVXPMenuHook::AddHook(HWND hWnd)
{
	ADD_COLLECTION(m_pHooks, new CHookList(this, hWnd))
}

void CVXPMenuHook::RemoveHook()
{
	if(m_pHooks)
	{
		CHookList * A = (CHookList*)m_pHooks->GetFirst();
		while(A)
		{
			if(A->m_pHook == this)
				break;
			A = (CHookList*)A->Next;
		}
		if(A)
		{
			CHookList * B = (CHookList*)A->GetChain();
			if(A == B)
			{
				delete m_pHooks;
				m_pHooks = NULL;
			}
			else
			{
				m_pHooks = B;
				A->Remove();
			}
		}
	}
}

STDMETHODIMP CVXPMenuHook::Override(long uItem, VARIANT_BOOL bByPos, long ItemID, long Sub1, long Sub2, long Sub3, long Sub4, long Sub5, long Sub6, long Sub7, long Sub8, long Sub9, VARIANT_BOOL * success)
{
	VXPLIB_BEGIN

	*success = FALSE;
	HMENU hMenu = ::GetMenu(m_hOldMenuHandler);
	if(hMenu)
	{
		long * SubIndex[] = {&Sub1, &Sub2, &Sub3, &Sub4, &Sub5, &Sub6, &Sub7, &Sub8, &Sub9};
		int I = 0;
		while(*SubIndex[I] != -1)
		{
			hMenu = ::GetSubMenu(hMenu, *SubIndex[I]);
			if(!hMenu)
			{
				AddException(EX_MENUFAILURE);
				break;
			}
			I ++;
		}
		if(!HasExceptions())
		{
			MENUITEMINFO mi;
			memset(&mi, 0, sizeof(MENUITEMINFO));
			mi.cbSize = sizeof(MENUITEMINFO);
			mi.fMask = MIIM_TYPE;

			// If by position, and no ItemID is specified
			// we need to retrieve Command ID:
			if(bByPos && !ItemID)
				mi.fMask |= MIIM_ID;

			GetMenuItemInfo(hMenu, uItem, bByPos, &mi);

			mi.fMask |= MIIM_DATA;
			mi.fType |= MFT_OWNERDRAW;

			// Now using that Command ID as ItemID:
			if(bByPos && !ItemID)
			{
				ItemID = mi.wID;
				mi.fMask &= ~MIIM_ID;
			}

			if(!bByPos && ItemID == 0)
				ItemID = uItem;
			if(bByPos && Sub1 == -1)
				ItemID |= ID_CODE;
			mi.dwItemData = ItemID;
			*success = SetMenuItemInfo(hMenu, uItem, bByPos, &mi);
		}
	}
	
	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::OverrideMenu(long hMenu, long uItem, VARIANT_BOOL bByPos, long ItemID, long Sub1, long Sub2, long Sub3, long Sub4, long Sub5, long Sub6, long Sub7, long Sub8, long Sub9, VARIANT_BOOL * success)
{
	VXPLIB_BEGIN

	*success = FALSE;
	HMENU menu = (HMENU)hMenu;
	if(menu)
	{
		long * SubIndex[] = {&Sub1, &Sub2, &Sub3, &Sub4, &Sub5, &Sub6, &Sub7, &Sub8, &Sub9};
		int I = 0;
		while(*SubIndex[I] != -1)
		{
			menu = ::GetSubMenu(menu, *SubIndex[I]);
			if(!menu)
			{
				AddException(EX_MENUFAILURE);
				break;
			}
			I ++;
		}
		if(!HasExceptions())
		{
			MENUITEMINFO mi;
			memset(&mi, 0, sizeof(MENUITEMINFO));
			mi.cbSize = sizeof(MENUITEMINFO);
			mi.fMask = MIIM_TYPE;

			// If by position, and no ItemID is specified
			// we need to retrieve Command ID:
			if(bByPos && !ItemID)
				mi.fMask |= MIIM_ID;

			GetMenuItemInfo(menu, uItem, (I > 0)?TRUE:FALSE, &mi);
			mi.fMask |= MIIM_DATA;
			mi.fType |= MFT_OWNERDRAW;

			// Now using that Command ID as ItemID:
			if(bByPos && !ItemID)
			{
				ItemID = mi.wID;
				mi.fMask &= ~MIIM_ID;
			}

			if(!bByPos && ItemID == 0)
				ItemID = uItem;
			mi.dwItemData = ItemID;
			*success = SetMenuItemInfo(menu, uItem, bByPos, &mi);
		}
	}
	else
		AddException(EX_INVALIDHANDLE);
	
	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::OverrideSys(long uItem, VARIANT_BOOL bByPos, long ItemID, long Sub1, long Sub2, long Sub3, VARIANT_BOOL * success)
{
	VXPLIB_BEGIN

	*success = FALSE;
	HMENU hMenu = ::GetSystemMenu(m_hOldMenuHandler, FALSE);
	if(hMenu)
	{
		long * SubIndex[] = {&Sub1, &Sub2, &Sub3};
		int I = 0;
		while(*SubIndex[I] != -1)
		{
			hMenu = ::GetSubMenu(hMenu, *SubIndex[I]);
			if(!hMenu)
			{
				AddException(EX_MENUFAILURE);
				break;
			}
			I ++;
		}
		if(!HasExceptions())
		{
			MENUITEMINFO mi;
			memset(&mi, 0, sizeof(MENUITEMINFO));
			mi.cbSize = sizeof(MENUITEMINFO);
			mi.fMask = MIIM_TYPE;

			// If by position, and no ItemID is specified
			// we need to retrieve Command ID:
			if(bByPos && !ItemID)
				mi.fMask |= MIIM_ID;

			GetMenuItemInfo(hMenu, uItem, (I > 0)?TRUE:FALSE, &mi);
			mi.fMask |= MIIM_DATA;
			mi.fType |= MFT_OWNERDRAW;

			// Now using that Command ID as ItemID:
			if(bByPos && !ItemID)
			{
				ItemID = mi.wID;
				mi.fMask &= ~MIIM_ID;
			}

			if(!bByPos && ItemID == 0)
				ItemID = uItem;
			if(bByPos && Sub1 == -1)
				ItemID |= ID_CODE;
			mi.dwItemData = ItemID;
			*success = SetMenuItemInfo(hMenu, uItem, bByPos?TRUE:FALSE, &mi);
		}
	}
	else
		AddException(EX_MENUFAILURE);

	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::DrawItemBackground(VARIANT_BOOL bXPSelection, VARIANT_BOOL * success)
{
	VXPLIB_BEGIN

	*success = DrawBackground(0, 0, bXPSelection?true:false);

	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::DrawItemBackgroundEx(short LeftMargin, short RightMargin, VARIANT_BOOL bXPSelection, VARIANT_BOOL * success)
{
	VXPLIB_BEGIN

	*success = DrawBackground(LeftMargin, RightMargin, bXPSelection?true:false);

	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::get_hWnd(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = (long)m_hOldMenuHandler;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::get_bgColor(OLE_COLOR *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_bgColor;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::put_bgColor(OLE_COLOR newVal)
{
	VXPLIB_BEGIN

	if(!g_bWin95orNT4)
	{
		m_bgColor = newVal;
		if(m_hBGBrush)
			DeleteObject(m_hBGBrush);
		m_hBGBrush = ::CreateSolidBrush(m_bgColor);
	}

	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::get_selColor(OLE_COLOR *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_selColor;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::put_selColor(OLE_COLOR newVal)
{
	VXPLIB_BEGIN

	m_selColor = newVal;
	if(m_hSelBrush)
		DeleteObject(m_hSelBrush);
	m_hSelBrush = ::CreateSolidBrush(m_selColor);
	
	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::ResetPalette()
{
	VXPLIB_BEGIN

	if(m_hSelBrush)
	{
		DeleteObject(m_hSelBrush);
		m_hSelBrush = NULL;
	}
	if(m_hCapBrush)
	{
		DeleteObject(m_hCapBrush);
		m_hCapBrush = NULL;
	}
	if(m_hBGBrush)
		DeleteObject(m_hBGBrush);

	m_bgColor = GetSysColor(COLOR_MENU);
	m_selColor = GetSysColor(COLOR_HIGHLIGHT);
	m_capColor = GetSysColor(COLOR_BTNFACE);

	m_hBGBrush = ::CreateSolidBrush(m_bgColor);

	VXPLIB_END
}

COLORREF CVXPMenuHook::BrightenColor(COLORREF Color)
{
	const double Factor = 0.75;
	
	BYTE Red, Green, Blue;

	Red = GetRValue(Color);
	Green = GetGValue(Color);
	Blue = GetBValue(Color);

	return RGB(Factor * (255 - Red) + Red, Factor * (255 - Green) + Green, Factor * (255 - Blue) + Blue);
}

STDMETHODIMP CVXPMenuHook::get_capColor(OLE_COLOR *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_capColor;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::put_capColor(OLE_COLOR newVal)
{
	VXPLIB_BEGIN

	m_capColor = newVal;
	if(m_hCapBrush)
		DeleteObject(m_hCapBrush);
	m_hCapBrush = ::CreateSolidBrush(m_capColor);

	VXPLIB_END
}

BOOL CVXPMenuHook::DrawBackground(short LeftMargin, short RightMargin, bool bXPSelection)
{
	if(!m_pDrawInfo)
		return FALSE;

	RECT r = m_pDrawInfo->rcItem;
	
	bool bSelected = (m_pDrawInfo->itemState & ODS_SELECTED);

	if(m_bStartedDrawing && (m_pDrawInfo->itemState & ODS_SELECTED))
	{
		bSelected = true;
		m_bStartedDrawing = false;
	}

	if(!bSelected && m_pDrawInfo->itemState & ODS_HOTLIGHT)
		bSelected = true;

	bool bDrawCaptionBackground = false;
	if(m_bCaptionSelected && !((m_pDrawInfo->itemState & ODS_HOTLIGHT) || (g_bWin95orNT4 && (m_pDrawInfo->itemState & ODS_SELECTED))))
	{
		m_bCaptionSelected = false;
		bDrawCaptionBackground = true;
	}

	HBRUSH hBrush = m_hBGBrush;
	if(bSelected)
		hBrush = m_hSelBrush;

	if(!hBrush)
		hBrush = GetSysColorBrush(bSelected?COLOR_HIGHLIGHT:COLOR_MENU);

	if(bDrawCaptionBackground && !bSelected)
	{
		if(m_hCapBrush)
			hBrush = m_hCapBrush;
		else
			hBrush = GetSysColorBrush(COLOR_BTNFACE);
	}

	if(bSelected && bXPSelection)
	{
		HPEN hPen = ::CreatePen(PS_SOLID, 1, m_selColor);

		if(m_pDrawInfo->itemState & ODS_GRAYED)
			hBrush = ::CreateSolidBrush(m_selColor);
		else
			hBrush = ::CreateSolidBrush(BrightenColor(m_selColor));				

		HGDIOBJ hOldBrush = SelectObject(m_pDrawInfo->hDC, hBrush);
		HGDIOBJ hOldPen = SelectObject(m_pDrawInfo->hDC, hPen);
		if(m_pDrawInfo->itemState & ODS_GRAYED)
		{
			RECT r2 = r;
			r2.left += LeftMargin;
			r2.right -= RightMargin;
			::FrameRect(m_pDrawInfo->hDC, &r2, hBrush);
		}
		else
			::Rectangle(m_pDrawInfo->hDC, r.left + LeftMargin, r.top, r.right - RightMargin, r.bottom);
		SelectObject(m_pDrawInfo->hDC, hOldPen);
		SelectObject(m_pDrawInfo->hDC, hOldBrush);
		DeleteObject(hBrush);
		DeleteObject(hPen);
	}
	else
	{
		r.left += LeftMargin;
		r.right -= RightMargin;
		FillRect(m_pDrawInfo->hDC, &r, hBrush);
	}

	return TRUE;
}

STDMETHODIMP CVXPMenuHook::DrawMenuBar(VARIANT_BOOL * success)
{
	VXPLIB_BEGIN

	*success = FALSE;

	if(::IsWindow(m_hOldMenuHandler))
		*success = ::DrawMenuBar(m_hOldMenuHandler)?TRUE:FALSE;

	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::get_MenuBarExtenderID(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_MenuBarExtenderID;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::put_MenuBarExtenderID(long newVal)
{
	VXPLIB_BEGIN

	m_MenuBarExtenderID = newVal;
	
	VXPLIB_END
}

void CVXPMenuHook::DrawExtender(LPDRAWITEMSTRUCT pInfo)
{
	if(g_bWin95orNT4)	// This function is not supported on Windows 95/NT4
		return;			// Though it is supported on NT4 + SP6

// Step1: Get the width of the menu bar:
	MENUBARINFO info;
	info.cbSize = sizeof(MENUBARINFO);
	g_pGetMenuBarInfo(m_hOldMenuHandler, OBJID_MENU, 0, &info);
	int BarWidth = info.rcBar.right - info.rcBar.left;

// Step2: Filling the extender:
	RECT r = {pInfo->rcItem.right, pInfo->rcItem.top, BarWidth + 3, pInfo->rcItem.bottom};
	if(m_hCapBrush)	// If it was ever changed
		::FillRect(pInfo->hDC, &r, m_hCapBrush);

// Step3: Drawing the extender:
	
	Fire_OnDrawMenuBarExtender((long)pInfo->hDC, (short)r.left, (short)r.top, (short)(r.right - r.left), (short)(r.bottom - r.top));
}

STDMETHODIMP CVXPMenuHook::get_MenuTransparency(BYTE *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_MenuTransparency;

	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::put_MenuTransparency(BYTE newVal)
{
	VXPLIB_BEGIN

	if(!g_bWin95orNT4)
		m_MenuTransparency = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::get_WindowTransparency(BYTE *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_WindowTransparency;
	
	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::put_WindowTransparency(BYTE newVal)
{
	VXPLIB_BEGIN

	if(!g_bWin95orNT4 && m_WindowTransparency != newVal && m_fnSetLayeredWindowAttributes && ::IsWindow(m_hOldMenuHandler))
	{
		m_WindowTransparency = newVal;
		long style = GetWindowLong(m_hOldMenuHandler, GWL_EXSTYLE);
		if(m_WindowTransparency == 255)
			SetWindowLong(m_hOldMenuHandler, GWL_EXSTYLE, style & ~WS_EX_LAYERED);
		else
		{
			if(!(style & WS_EX_LAYERED))
				SetWindowLong(m_hOldMenuHandler, GWL_EXSTYLE, style | WS_EX_LAYERED);
			m_fnSetLayeredWindowAttributes(m_hOldMenuHandler, 0, m_WindowTransparency, LWA_ALPHA);
		}
	}

	VXPLIB_END
}

STDMETHODIMP CVXPMenuHook::get_sysFadedMenus(VARIANT_BOOL *pVal)
{
	VXPLIB_BEGIN

	*pVal = FALSE;

	BOOL Effects;
	SystemParametersInfo(SPI_GETUIEFFECTS, NULL, &Effects, 0);
	if(Effects)
	{
		BOOL Animation;
		SystemParametersInfo(SPI_GETMENUANIMATION, NULL, &Animation, 0);
		if(Animation)
		{
			BOOL Fade;
			SystemParametersInfo(SPI_GETMENUFADE, NULL, &Fade, 0);
			*pVal = Fade;
		}
	}

	VXPLIB_END
}
