/*
 * VXPLib COMPONENT
 *
 * Module: VXPMenuHook.h - Declaration of the CVXPMenuHook class
 * (CTOxxx classes)
 *
 * Synopsis: Class CVXPMenuHook implements methods and properties
 * of interface IVXPMenuHook.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#ifndef __VXPMENUHOOK_H_
#define __VXPMENUHOOK_H_

#include "resource.h"       // main symbols
#include "HCollection.h"
#include "VXPLibCP.h"

// Predeclaration of the class to make it visible
// in class CHookList:
class CVXPMenuHook;

class CHookList:public HCollection
{
public:
	CHookList(CVXPMenuHook * pHook, HWND hWnd):m_pHook(pHook), m_hWnd(hWnd){}

	HWND m_hWnd;
	CVXPMenuHook * m_pHook;
};

/////////////////////////////////////////////////////////////////////////////
// CVXPMenuHook
class ATL_NO_VTABLE CVXPMenuHook : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CVXPMenuHook, &CLSID_VXPMenuHook>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CVXPMenuHook>,
	public IDispatchImpl<IVXPMenuHook, &IID_IVXPMenuHook, &LIBID_VXPLibrary>,
	public CProxy_IVXPMenuHookEvents< CVXPMenuHook >,
	public CVXPException<&CLSID_VXPMenuHook, &IID_IVXPMenuHook>
{
public:
	// Constructor:
	CVXPMenuHook();

	// Destructor:
	~CVXPMenuHook();

// ATL registration template for this interface:
DECLARE_REGISTRY_RESOURCEID(IDR_VXPMENUHOOK)

// Making sure that FinalConstruct
// is always called:
DECLARE_PROTECT_FINAL_CONSTRUCT()

// COM interfaces implemented by this class:
BEGIN_COM_MAP(CVXPMenuHook)
	COM_INTERFACE_ENTRY(IVXPMenuHook)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
END_COM_MAP()

// Map of connection points to handle events:
BEGIN_CONNECTION_POINT_MAP(CVXPMenuHook)
	CONNECTION_POINT_ENTRY(DIID__IVXPMenuHookEvents)
END_CONNECTION_POINT_MAP()

// Overriding a virtual method here:
	HRESULT FinalRelease();

// ISupportsErrorInfo:
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

public:

////////////////////////////
// Methods of the interface:
////////////////////////////

	STDMETHOD(HookWnd)(/*[in]*/ long hWnd, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(UnHookWnd)(/*[out, retval]*/ long * hWnd);
	STDMETHOD(Override)(/*[in]*/ long uItem, /*[in, defaultvalue(FALSE)]*/ VARIANT_BOOL bByPos, /*[in, defaultvalue(0)]*/ long ItemID, /*[in, defaultvalue(-1)]*/ long Sub1, /*[in, defaultvalue(-1)]*/ long Sub2, /*[in, defaultvalue(-1)]*/ long Sub3, /*[in, defaultvalue(-1)]*/ long Sub4, /*[in, defaultvalue(-1)]*/ long Sub5, /*[in, defaultvalue(-1)]*/ long Sub6, /*[in, defaultvalue(-1)]*/ long Sub7, /*[in, defaultvalue(-1)]*/ long Sub8, /*[in, defaultvalue(-1)]*/ long Sub9, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(OverrideMenu)(/*[in]*/ long hMenu, /*[in]*/ long uItem, /*[in, defaultvalue(FALSE)]*/ VARIANT_BOOL bByPos, /*[in, defaultvalue(0)]*/ long ItemID, /*[in, defaultvalue(-1)]*/ long Sub1, /*[in, defaultvalue(-1)]*/ long Sub2, /*[in, defaultvalue(-1)]*/ long Sub3, /*[in, defaultvalue(-1)]*/ long Sub4, /*[in, defaultvalue(-1)]*/ long Sub5, /*[in, defaultvalue(-1)]*/ long Sub6, /*[in, defaultvalue(-1)]*/ long Sub7, /*[in, defaultvalue(-1)]*/ long Sub8, /*[in, defaultvalue(-1)]*/ long Sub9, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(OverrideSys)(/*[in]*/ long uItem, /*[in, defaultvalue(FALSE)]*/ VARIANT_BOOL bByPos, /*[in, defaultvalue(0)]*/ long ItemID, /*[in, defaultvalue(-1)]*/ long Sub1, /*[in, defaultvalue(-1)]*/ long Sub2, /*[in, defaultvalue(-1)]*/ long Sub3, /*[in, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(DrawItemBackground)(/*[in, defaultvalue(FALSE)]*/ VARIANT_BOOL bXPSelection, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(DrawItemBackgroundEx)(/*[in, defaultvalue(0)]*/ short LeftMargin, /*[in, defaultvalue(0)]*/ short RightMargin, /*[in, defaultvalue(FALSE)]*/ VARIANT_BOOL bXPSelection, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(ResetPalette)();
	STDMETHOD(DrawMenuBar)(/*[out, retval]*/ VARIANT_BOOL * success);

//////////////////////////////
// Properties of the interface
//////////////////////////////

	STDMETHOD(get_selColor)(/*[out, retval]*/ OLE_COLOR *pVal);
	STDMETHOD(put_selColor)(/*[in]*/ OLE_COLOR newVal);
	STDMETHOD(get_sysFadedMenus)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(get_bgColor)(/*[out, retval]*/ OLE_COLOR *pVal);
	STDMETHOD(put_bgColor)(/*[in]*/ OLE_COLOR newVal);
	STDMETHOD(get_hWnd)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_MenuBarExtenderID)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_MenuBarExtenderID)(/*[in]*/ long newVal);
	STDMETHOD(get_capColor)(/*[out, retval]*/ OLE_COLOR *pVal);
	STDMETHOD(put_capColor)(/*[in]*/ OLE_COLOR newVal);
	STDMETHOD(get_WindowTransparency)(/*[out, retval]*/ BYTE *pVal);
	STDMETHOD(put_WindowTransparency)(/*[in]*/ BYTE newVal);
	STDMETHOD(get_MenuTransparency)(/*[out, retval]*/ BYTE *pVal);
	STDMETHOD(put_MenuTransparency)(/*[in]*/ BYTE newVal);

	LRESULT ProcessHook(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	static LRESULT CALLBACK MenuHook(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	static LRESULT CALLBACK CallWndProc(int nCode, WPARAM wParam, LPARAM lParam);
	static CVXPMenuHook * FindHook(HWND hWnd);
	static CHookList * m_pHooks;

private:
	
	void DrawExtender(LPDRAWITEMSTRUCT pInfo);
	BOOL DrawBackground(short LeftMargin, short RightMargin, bool bXPSelection);

// Properties:

	COLORREF m_capColor;
	COLORREF m_bgColor;
	COLORREF m_selColor;
	COLORREF m_gsColor; // Gradient Selection Color
	bool m_bGSEnabled;	// Contains true if m_gsColor was set after the last
						// invocation of ResetPalette();

	HBRUSH m_hBGBrush;
	HBRUSH m_hSelBrush;
	HBRUSH m_hCapBrush;

	DRAWITEMSTRUCT m_LastDrawnSelection;
	bool m_bStartedDrawing;
	bool m_bStartedMenu;
	bool m_bCaptionSelected;
	LPDRAWITEMSTRUCT m_pDrawInfo;
	WNDPROC m_OldWndProc;
	HWND m_hOldMenuHandler;

	long m_MenuBarExtenderID;

// Methods:

	COLORREF BrightenColor(COLORREF color);
	void AddHook(HWND hWnd);
	void RemoveHook();
	void DetachWindow();

	BYTE m_WindowTransparency;
	BYTE m_MenuTransparency;

	typedef BOOL (WINAPI* FN_SET_LAYERED_WINDOW_ATTRIBUTES)(HWND,COLORREF,BYTE,DWORD);
	FN_SET_LAYERED_WINDOW_ATTRIBUTES m_fnSetLayeredWindowAttributes;

	HHOOK m_hHook;
};

#endif //__VXPMENUHOOK_H_
