/*
 * VXPLib COMPONENT
 *
 * Module: VXPQuickHTML.h - Declaration of class CVXPQuickHTML
 *
 * Synopsis: Class CVXPQuickHTML implements methods and properties
 * of interface IVXPQuickHTML.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#ifndef __VXPQUICKHTML_H_
#define __VXPQUICKHTML_H_

#include "resource.h"       // main symbols
#include "String.h"
#include "TagObject.h"
#include "GlobalObject.h"
#include "VXPLibCP.h"
#include "Picture.h"
#include "VXPSound.h"

/////////////////////////////////////////////////////////////////////////////
// CVXPQuickHTML
class ATL_NO_VTABLE CVXPQuickHTML : 
	public CGlobalObject,
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CVXPQuickHTML, &CLSID_VXPQuickHTML>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CVXPQuickHTML>,
	public IDispatchImpl<IVXPQuickHTML, &IID_IVXPQuickHTML, &LIBID_VXPLibrary>,
	public CProxy_IVXPQuickHTMLEvents< CVXPQuickHTML >,
	public CVXPException<&CLSID_VXPQuickHTML, &IID_IVXPQuickHTML>
{
public:
	CVXPQuickHTML();
	~CVXPQuickHTML();

DECLARE_REGISTRY_RESOURCEID(IDR_VXPQUICKHTML)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CVXPQuickHTML)
	COM_INTERFACE_ENTRY(IVXPQuickHTML)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
END_COM_MAP()
BEGIN_CONNECTION_POINT_MAP(CVXPQuickHTML)
CONNECTION_POINT_ENTRY(DIID__IVXPQuickHTMLEvents)
END_CONNECTION_POINT_MAP()

	HRESULT FinalConstruct();
	HRESULT FinalRelease();

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IVXPQuickHTML
public:
	STDMETHOD(TextToHTMLEx)(/*[in]*/ BSTR strSource, /*[in]*/ long nMaxWidth, /*[in, out]*/ long * nRows, /*[in, out]*/ VARIANT_BOOL * bTruncated, /*[in, defaultvalue(tthDefault)]*/ enum VXPTextToHTML tthOptions, /*[out, retval]*/ BSTR * strDest);
	STDMETHOD(GetControlsInfo)(/*[in, out]*/ long * MinID, /*[in, out]*/ long * MaxID, /*[out, retval]*/ long * nControls);
	STDMETHOD(GetLicenses)(/*[out, retval]*/ long * nLicenses);
	STDMETHOD(TextToHTML)(/*[in]*/ BSTR strSource, /*[in, defaultvalue(0)]*/ long nMaxWidth, /*[out, retval]*/ BSTR * strDest);
	STDMETHOD(GetControlPos)(/*[in]*/ long ID, /*[in, out]*/ short * X, /*[in, out]*/ short * Y, /*[in, out]*/ short * W, /*[in, out]*/ short * H, /*[out, retval]*/ VARIANT_BOOL * bFound);
	STDMETHOD(GetBodyAttr)(/*[in]*/ BSTR attrName, /*[in, defaultvalue(taText)]*/ enum VXPTagAttribute attrType, /*[out, retval]*/ VARIANT * Value);
	STDMETHOD(GetControlAttr)(/*[in]*/ long ID, /*[in]*/ BSTR attrName, /*[in, defaultvalue(taText)]*/ enum VXPTagAttribute attrType, /*[out, retval]*/ VARIANT * Value);
	STDMETHOD(GetControlAttrIdx)(/*[in]*/ long Index, /*[in]*/ BSTR attrName, /*[in, defaultvalue(taText)]*/ enum VXPTagAttribute attrType, /*[out, retval]*/ VARIANT * Value);
	STDMETHOD(GradientFillRect)(/*[in]*/ long hDC, /*[in]*/ OLE_COLOR StartColor, /*[in]*/ OLE_COLOR EndColor, /*[in]*/ short x1, /*[in]*/ short y1, /*[in]*/ short x2, /*[in]*/ short y2, /*[in, defaultvalue(TRUE)]*/ VARIANT_BOOL bHorizontal, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(get_BodyCodePage)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_BodyCodePage)(/*[in]*/ long newVal);
	STDMETHOD(get_TxtCodePage)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_TxtCodePage)(/*[in]*/ long newVal);
	STDMETHOD(GetSpeechText)(/*[in]*/ VXPSpeechFormat format, /*[in]*/ VARIANT_BOOL bIncludeComments, /*[in]*/ VARIANT_BOOL bIncludeText, /*[out, retval]*/ BSTR * Text);
	STDMETHOD(get_HasBackgroundPicture)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(DrawBackground)(/*[in]*/ long dc, /*[in]*/ short x, /*[in]*/ short y, /*[in]*/ short width, /*[in]*/ short height, /*[in]*/ VARIANT_BOOL DrawBackColor, /*[in, out]*/ VARIANT_BOOL * success);
	STDMETHOD(AboutBox)();
	STDMETHOD(DrawStretched)(/*[in]*/ long dc, /*[in]*/ short x, /*[in]*/ short y, /*[in]*/ short width, /*[in]*/ short height, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(get_settings)(/*[out, retval]*/ IVXPSettings* *pVal);
	STDMETHOD(AddFilePath)(/*[in]*/ BSTR FilePath, /*[out, retval]*/ short * counter);
	STDMETHOD(SetDefaultRes)(/*[in]*/ BSTR FilePath, /*[out, retval]*/ VARIANT_BOOL * succeeded);
	STDMETHOD(get_Height)(/*[out, retval]*/ short *pVal);
	STDMETHOD(get_Width)(/*[out, retval]*/ short *pVal);
	STDMETHOD(get_Transparent)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_Transparent)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(Draw)(/*[in]*/ long dc, /*[in]*/ short x, /*[in]*/ short y, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(DrawState)(/*[in]*/ long dc, /*[in]*/ short x, /*[in]*/ short y, /*[in]*/ long dssFlags, /*[in, defaultvalue(0)]*/ OLE_COLOR Color, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(RecalculateLayout)(/*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(SetSourceText)(/*[in]*/ BSTR source);

private:
	bool Recalculate();
	void VariantClearEx(VARIANT * pVariant);
	void NormalizeSizes();

	VXPSysFont ReadSysFont(CString & name);
	void UpdateBitmap();
	void SetupSound(CTag * tag);
	void SetupBody(CTag * tag);
	// Document support virtual methods ***
	virtual CSize GetDocumentSize(CString & FilePath, CString & Param, short Width, short Height);
	virtual void DrawDocument(CString & FilePath, CString & Param, HDC dc, short x, short y, short width, short height);
	CSize GetDefIconSize();
	// ************************************

	// Controls support:
	CSize GetControlSize(long ID, CString & Name, short Width, short Height, CString & Param);

	static BOOL CALLBACK DrawStateProc(HDC hdc, LPARAM lData, WPARAM wData, int cx, int cy);

	// Attribute ID support for tag TXT:
	virtual CString OnGetText(CString & FilePath, CString & Res, long Size, long ID);

	virtual void AddError(long ExceptionFlags)
	{
		AddException(ExceptionFlags);
	}

	CTagObject * GetNextTagObject(CString & text);
	void ConvertTextIntoTags();
	void RecalculateWindowSize();
	void DrawTags(HDC dc, int x, int y, bool Transparent = false);
	bool GetNextPhrase(CString & text, CString & phrase, bool & isText);
	void DrawBuffered(HDC dc, short x, short y, short width, short height);

	VARIANT GetTagAttr(CTag * pTag, BSTR attrName, enum VXPTagAttribute attrType);
	CTOControl * FindControl(long ID, bool bIndex = false);

	CString m_text;
	CTagObject * m_pTags;
	HBITMAP m_MemoryBMP;
	CSize m_size;
	bool m_bTransparent;
	bool m_bSourceChanged;

// These two booleans are to indicate that they have been
// found at least once. Used for avoiding a second installation.
	bool m_bSetupBody;
	bool m_bSetupSound;

	IVXPSettings * m_pSettings;
	IVXPMargins * m_pMargins;
	CVXPSound * m_pSoundClass;

// This will contain a picture, if specified and found along with
// the BODY tag to be used as background.
	CPicture * m_pBackground;

	long m_Exception;

	CTagObject * m_pLastTag; // The very last tag the parser created

	CTag m_BodyTag; // Copy of all attrbutes specified within tag <BODY>
					// It is populated in method SetupBody()

	bool m_bRecalculating; // When True QHTML Document recalculation is in process


	// These variables are to speed up work of function GetControlsInfo
	// so that they wouldn't neet to be recalculated more than once:
	long m_nControls;
	long m_MinID;
	long m_MaxID;

};

#endif //__VXPQUICKHTML_H_
