/*
 * VXPLib COMPONENT
 *
 * Module: VXPSettings.cpp - Implementation of class CVXPSettings
 *
 * Synopsis: Class CVXPSettings implements methods and properties
 * of interface IVXPSettings.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "VXPLib.h"
#include "VXPSettings.h"

extern VXPOSVersion g_osVersion;

/////////////////////////////////////////////////////////////////////////////
// CVXPSettings

STDMETHODIMP CVXPSettings::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IVXPSettings
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

CVXPSettings::CVXPSettings()
{
// Setting static parameters, i.e. those which are not
// restored for each new document:
	m_DefIconWidth = 32;
	m_DefIconHeight = 32;
// ***************************************************

	SetDefaultParameters();
}

STDMETHODIMP CVXPSettings::get_bgColor(OLE_COLOR *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_bgColor;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::put_bgColor(OLE_COLOR newVal)
{
	VXPLIB_BEGIN

	m_bgColor = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::SetDefaults()
{
	VXPLIB_BEGIN

	SetDefaultParameters();

	m_pMargins->SetDefaults();
	m_pFont->SetDefaults();
	m_pSound->SetDefaults();

	VXPLIB_END
}

void CVXPSettings::SetDefaultParameters()
{
	m_gfColor = RGB(0, 0, 0);
	m_gfEnabled = FALSE;
	m_gfHorizontal = TRUE;

	m_bgColor = RGB(255, 255, 255);
}

HRESULT CVXPSettings::FinalConstruct()
{
	::CoCreateInstance(CLSID_VXPMargins, NULL, CLSCTX_ALL, IID_IVXPMargins, (void**)&m_pMargins);
	::CoCreateInstance(CLSID_VXPFont, NULL, CLSCTX_ALL, IID_IVXPFont, (void**)&m_pFont);
	::CoCreateInstance(CLSID_VXPSound, NULL, CLSCTX_ALL, IID_IVXPSound, (void**)&m_pSound);

	return S_OK;
}

HRESULT CVXPSettings::FinalRelease()
{
	m_pMargins->Release();
	m_pFont->Release();
	m_pSound->Release();

	return S_OK;
}

STDMETHODIMP CVXPSettings::get_margins(IVXPMargins ** pVal)
{
	VXPLIB_BEGIN

	*pVal = m_pMargins;
	m_pMargins->AddRef();
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::get_font(IVXPFont **pVal)
{
	VXPLIB_BEGIN

	*pVal = m_pFont;
	m_pFont->AddRef();
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::get_bgsound(IVXPSound ** pVal)
{
	VXPLIB_BEGIN

	*pVal = m_pSound;
	m_pSound->AddRef();
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::get_gfColor(OLE_COLOR *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_gfColor;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::put_gfColor(OLE_COLOR newVal)
{
	VXPLIB_BEGIN

	m_gfColor = newVal;
	m_gfEnabled = TRUE;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::get_gfHorizontal(VARIANT_BOOL *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_gfHorizontal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::put_gfHorizontal(VARIANT_BOOL newVal)
{
	VXPLIB_BEGIN

	m_gfHorizontal = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::get_gfEnabled(VARIANT_BOOL *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_gfEnabled;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::put_gfEnabled(VARIANT_BOOL newVal)
{
	VXPLIB_BEGIN

	m_gfEnabled = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::get_DefIconWidth(short *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_DefIconWidth;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::put_DefIconWidth(short newVal)
{
	VXPLIB_BEGIN

	m_DefIconWidth = newVal;

	VXPLIB_END
}

STDMETHODIMP CVXPSettings::get_DefIconHeight(short *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_DefIconHeight;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSettings::put_DefIconHeight(short newVal)
{
	VXPLIB_BEGIN

	m_DefIconHeight = newVal;

	VXPLIB_END
}

STDMETHODIMP CVXPSettings::get_osVersion(enum VXPOSVersion * pVal)
{
	VXPLIB_BEGIN

	*pVal = g_osVersion;

	VXPLIB_END
}
