/*
 * VXPLib COMPONENT
 *
 * Module: VXPSettings.h - Declaration of class CVXPSettings
 *
 * Synopsis: Class CVXPSettings implements methods and properties
 * of interface IVXPSettings.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#ifndef __VXPSETTINGS_H_
#define __VXPSETTINGS_H_

#include "resource.h"       // main symbols

/////////////////////////////////////////////////////////////////////////////
// CVXPSettings
class ATL_NO_VTABLE CVXPSettings : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CVXPSettings, &CLSID_VXPSettings>,
	public ISupportErrorInfo,
	public IDispatchImpl<IVXPSettings, &IID_IVXPSettings, &LIBID_VXPLibrary>,
	public CVXPException<&CLSID_VXPSettings, &IID_IVXPSettings>
{
public:
	CVXPSettings();

DECLARE_REGISTRY_RESOURCEID(IDR_VXPSETTINGS)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CVXPSettings)
	COM_INTERFACE_ENTRY(IVXPSettings)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()

	HRESULT FinalConstruct();
	HRESULT FinalRelease();

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IVXPSettings
public:
	STDMETHOD(get_DefIconHeight)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_DefIconHeight)(/*[in]*/ short newVal);
	STDMETHOD(get_DefIconWidth)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_DefIconWidth)(/*[in]*/ short newVal);
	STDMETHOD(get_gfEnabled)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_gfEnabled)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_gfHorizontal)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_gfHorizontal)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_gfColor)(/*[out, retval]*/ OLE_COLOR *pVal);
	STDMETHOD(put_gfColor)(/*[in]*/ OLE_COLOR newVal);
	STDMETHOD(get_bgsound)(/*[out, retval]*/ IVXPSound ** pVal);
	STDMETHOD(get_font)(/*[out, retval]*/ IVXPFont ** pVal);
	STDMETHOD(get_margins)(/*[out, retval]*/ IVXPMargins ** pVal);
	STDMETHOD(SetDefaults)();
	STDMETHOD(get_bgColor)(/*[out, retval]*/ OLE_COLOR *pVal);
	STDMETHOD(put_bgColor)(/*[in]*/ OLE_COLOR newVal);
	STDMETHOD(get_osVersion)(/*[out, retval]*/ enum VXPOSVersion * pVal);

private:
	
	IVXPMargins * m_pMargins;
	IVXPFont * m_pFont;
	IVXPSound * m_pSound;

	void SetDefaultParameters();
	
	COLORREF m_bgColor;

// Gradient filling support:
	COLORREF m_gfColor;
	BOOL m_gfHorizontal;
	BOOL m_gfEnabled;

// These are two sttaic parameters, i.e. they are not
// restored for each new document:
	short m_DefIconWidth;
	short m_DefIconHeight;
};

#endif //__VXPSETTINGS_H_
