/*
 * VXPLib COMPONENT
 *
 * Module: VXPSound.cpp - Implementation of class CVXPSound
 *
 * Synopsis: Class CVXPSound implements methods and properties
 * of interface IVXPSound.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "VXPLib.h"
#include "VXPSound.h"

/////////////////////////////////////////////////////////////////////////////
// CVXPSound

CVXPSound::CVXPSound()
{
	m_Handle = NULL;
	m_LastPlayFrom = pfCustom;
	m_bKeepSettings = false;
	m_bWasInitialized = false;
	GetTempPathW(1024, m_TempDir.PreAllocate(1024));
	m_TempDir.PostAllocate();
}

STDMETHODIMP CVXPSound::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IVXPSound
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CVXPSound::get_Volume(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_Volume;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_Volume(long newVal)
{
	VXPLIB_BEGIN

	m_Volume = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::SetDefaults()
{
	VXPLIB_BEGIN

	SetDefaultParameters();
	
	VXPLIB_END
}

void CVXPSound::SetDefaultParameters()
{
	if(m_bKeepSettings && m_bWasInitialized)
		return;

	UninitializeHandle();

	m_PlayFrom = pfCustom;
	m_FilePath.Empty();
	m_Res.Empty();
	m_Info.Empty();
	m_SizeLimit = 0;
	m_Param = 0;
	m_Volume = 0xffffffff; // Max sound volume
	m_Balance = 0;
	m_Frequency = 0;
	m_Pause = 0;
	m_Delay = 0;
	m_Loop = 1;
	m_StartPos = 0;
	m_EndPos = 0;
	
	m_bWasInitialized = true;
}

BYTE * CVXPSound::GetDataFromFileRes(int & nSize)
{
	CString res(m_Res);
	nSize = 0;
	res.Remove(' ');
	if(res.IsEmpty() && m_FilePath.IsEmpty())
		return NULL;

	if(!res.IsEmpty()) // Then we load the resource
	{
//*************************************************************
// Now parsing RES:
		int L = res.GetLength();
		int index = res.Find('=');
		if(index == -1)
			index = res.Find(':');
		if(index == -1 || !index || index == L - 1)	// Wrong contents
			return NULL;
		CString res_type = res.Left(index);
		CString res_name = res.Right(L - index - 1);
//**************************************************************
		HINSTANCE hInst;
		if(m_FilePath.IsEmpty())
			hInst = m_pGlobalObject->GetDefaultInstance();
		else
		{

		#ifdef _MSLU
			if(g_bNTCompitable)
				hInst = ::LoadLibraryEx(m_FilePath, NULL, LOAD_LIBRARY_AS_DATAFILE);
			else
				hInst = ::LoadLibrary(m_FilePath);
		#else
			hInst = ::LoadLibraryEx(m_FilePath, NULL, LOAD_LIBRARY_AS_DATAFILE);
		#endif

			if(!hInst)
				return NULL;
		}
//**************************************************************
		if(GetResource(hInst, res_name, res_type, NULL, nSize) && nSize > 0)
		{
			// Controlling the size of the loaded:
			if(m_SizeLimit > 0 && m_SizeLimit < nSize)
				nSize = m_SizeLimit;
			// ***********************************
			BYTE * pBuffer = new BYTE[nSize];
			if(GetResource(hInst, res_name, res_type, pBuffer, nSize))
			{
				if(!m_FilePath.IsEmpty())
					FreeLibrary(hInst);
				return pBuffer;
			}
			else
				delete []pBuffer;
		}
		if(!m_FilePath.IsEmpty())
			FreeLibrary(hInst);
		return NULL;
	}
	else	// Otherwise, load the sound from file
		return ReadFromFile(m_FilePath, nSize, m_SizeLimit);
}

void CVXPSound::SetupSound(CTag * tag, CGlobalObject * pGlobalObject)
{
	m_pGlobalObject = pGlobalObject;

	UninitializeHandle();
// *****************************************************************
	m_Volume = tag->GetAttrIntValue(_T("VOLUME"));
	m_Balance = tag->GetAttrIntValue(_T("BALANCE"));
	m_StartPos = tag->GetAttrIntValue(_T("STARTPOS"));
	m_EndPos = tag->GetAttrIntValue(_T("ENDPOS"));
	m_Loop = tag->GetAttrIntValue(_T("LOOP"));
	m_Delay = tag->GetAttrIntValue(_T("DELAY"));
	m_Pause = tag->GetAttrIntValue(_T("PAUSE"));
	m_Frequency = tag->GetAttrIntValue(_T("FREQUENCY"));
	m_Info = tag->GetAttrTextValue(_T("INFO"));
// *****************************************************************
	CString PlayFrom = tag->GetAttrTextValue(_T("PLAYFROM"));
	PlayFrom.Remove(' ');
	PlayFrom.MakeUpper();

struct CPlayFrom
{
	LPCTSTR m_Name;
	VXPPlayFrom m_Type;
};

	if(!PlayFrom.IsEmpty())
	{
		CPlayFrom play_from[] =
		{{_T("HEAP"), pfHeap}, {_T("GLOBAL"), pfGlobal}, {_T("STREAM"), pfStream}, {_T("VFILE"), pfVirtualFile},
						{_T("RESOURCE"), pfResource}, {attrFILE, pfFile}, {_T("CUSTOM"), pfCustom}};

		for(int i = 0;i < sizeof(play_from)/sizeof(CPlayFrom);i ++)
			if(PlayFrom == play_from[i].m_Name)
			{
				m_PlayFrom = play_from[i].m_Type;
				break;
			}
	}
// *****************************************************************
	m_SizeLimit = tag->GetAttrIntValue(attrSIZE);
	if(m_SizeLimit < 0)
		m_SizeLimit = 0;

	m_Param = tag->GetAttrIntValue(attrPARAM);
	m_Res = tag->GetAttrTextValue(attrRES);
	m_FilePath = tag->GetAttrTextValue(attrFILE);
	m_FilePath = pGlobalObject->GetValidFilePath(m_FilePath);

// *****************************************************************

	InitializeHandle();
}

// This function must analyze contents of FILE, RES and PLAYFROM
// to initialize contents of the handle.
void CVXPSound::InitializeHandle()
{
	m_LastPlayFrom = m_PlayFrom;
	
	switch(m_PlayFrom)
	{
	case pfFile:
		{
			// The file must exist and placed in FILE.
			// All the rest parameters can be ignored, including RES
			if(!m_FilePath.IsEmpty())
				FileExists(m_FilePath, &m_Handle);
			break;
		}
	case pfResource:
		{
			// The resource file instance must be returned.
			// RES is ignored.
			if(!m_FilePath.IsEmpty())
			{
			#ifdef _MSLU
				if(g_bNTCompitable)
					m_Handle = ::LoadLibraryEx(m_FilePath, NULL, LOAD_LIBRARY_AS_DATAFILE);
				else
					m_Handle = ::LoadLibrary(m_FilePath);
			#else
				m_Handle = ::LoadLibraryEx(m_FilePath, NULL, LOAD_LIBRARY_AS_DATAFILE);
			#endif
			}
				
			break;
		}
	case pfHeap:
		{
			// The result must be allocated in the local heap.
			// Data are to be taken from file/resource, depending
			// the RES/FILE combination.
			int nSize;
			BYTE * Buffer = GetDataFromFileRes(nSize);
			if(Buffer)
			{
				HANDLE hProcessHeap = GetProcessHeap();
				if(hProcessHeap)
				{
					LPVOID hHeap = HeapAlloc(hProcessHeap, 0, nSize);
					if(hHeap)
					{
						memcpy(hHeap, Buffer, nSize);
						m_Handle = hHeap;
					}
				}
				delete []Buffer;
			}
			break;
		}
	case pfGlobal:
		{
			// The result must be allocated in the global memory.
			// Data are to be taken from file/resource, depending
			// the RES/FILE combination.
			int nSize;
			BYTE * Buffer = GetDataFromFileRes(nSize);
			if(Buffer)
			{
				m_Handle = GlobalAlloc(GMEM_MOVEABLE, nSize);
				if(m_Handle)
				{
					void * pData = GlobalLock(m_Handle);
					memcpy(pData, Buffer, nSize);
					GlobalUnlock(m_Handle);
				}
				delete []Buffer;
			}
			break;
		}
	case pfStream:
		{
			// The result must be allocated in stream.
			// Data are to be taken from file/resource, depending
			// the RES/FILE combination.
			int nSize;
			BYTE * Buffer = GetDataFromFileRes(nSize);
			if(Buffer)
			{
				HGLOBAL hGlobal = GlobalAlloc(GMEM_MOVEABLE, nSize);
				if(hGlobal)
				{
					void * pData = GlobalLock(hGlobal);
					memcpy(pData, Buffer, nSize);
					GlobalUnlock(hGlobal);
					IStream * pStream = NULL;
					CreateStreamOnHGlobal(hGlobal, TRUE, &pStream);
					m_Handle = pStream;
				}
				delete []Buffer;
			}
			break;
		}
	case pfVirtualFile:
		{
			// Requires that RES be defined, so verify that first:
			CString resource = m_Res;
			resource.Remove(' ');
			bool succeeded = false;
			if(!resource.IsEmpty())
			{
				int nSize;
				BYTE * Buffer = GetDataFromFileRes(nSize);
				if(Buffer)
				{
					CString name;
					GetTempFileName(m_TempDir, _T("VXP"), 0, name.PreAllocate(MAX_PATH));
					name.PostAllocate();
					if(!name.IsEmpty())
					{
					#ifdef _MSLU
						if(g_bNTCompitable)
						{
							if(g_pGetLongPathNameW)
							{
								CString unicode_name(_T("\\\\?\\"));
								unicode_name += name;
								g_pGetLongPathNameW(unicode_name, m_FilePath.PreAllocate(MAX_PATH_LENGTH), MAX_PATH_LENGTH);
							}
							else
								m_FilePath = name;
						}
						else
							m_FilePath = name;
					#else
						CString unicode_name(_T("\\\\?\\"));
						unicode_name += name;
						GetLongPathNameW(unicode_name, m_FilePath.PreAllocate(MAX_PATH_LENGTH), MAX_PATH_LENGTH);
					#endif
						m_FilePath.PostAllocate();
						HANDLE hFile = ::CreateFileW(m_FilePath, GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

						if(hFile)
						{
							DWORD Written;
							WriteFile(hFile, Buffer, nSize, &Written, NULL);
							if(Written == (DWORD)nSize)
								succeeded = true;
							CloseHandle(hFile);
						}
					}
				}
				delete []Buffer;
			}
			if(succeeded)
				m_Handle = ::CreateFileW(m_FilePath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
			if(m_Handle)
			{
				m_TempFilePath = m_FilePath;
				m_FilePath.Delete(0, 4);
			}
			else
				m_FilePath.Empty();

			break;
		}
	case pfCustom:
		{
			// No processing whatsoever.
			break;
		}
	}
}

// This function must correctly destroy the handle
void CVXPSound::UninitializeHandle()
{
	if(m_Handle && m_LastPlayFrom != pfCustom)
		switch(m_LastPlayFrom)
		{
		case pfStream:
			{
				((IStream*)m_Handle)->Release();
				break;
			}
		case pfGlobal:
			{
				GlobalFree((HGLOBAL)m_Handle);
				break;
			}
		case pfHeap:
			{
				HANDLE hProcessHeap = GetProcessHeap();
				if(hProcessHeap)
					HeapFree(hProcessHeap, 0, m_Handle);
				break;
			}
		case pfResource:
			{
				FreeLibrary((HINSTANCE)m_Handle);
				break;
			}
		case pfFile:
			{
				CloseHandle(m_Handle);
				break;
			}
		case pfVirtualFile:
			{
				CloseHandle(m_Handle);
				DeleteFileW(m_TempFilePath);
				break;
			}
		default:
			break;
		}
	m_Handle = NULL;
	m_LastPlayFrom = pfCustom;
}

STDMETHODIMP CVXPSound::get_FilePath(BSTR *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_FilePath.SysAllocString();
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_FilePath(BSTR newVal)
{
	VXPLIB_BEGIN

	m_FilePath = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_Size(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_SizeLimit;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_Size(long newVal)
{
	VXPLIB_BEGIN

	m_SizeLimit = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_PlayFrom(VXPPlayFrom *pVal)
{
	*pVal = m_PlayFrom;
	return S_OK;
}

STDMETHODIMP CVXPSound::put_PlayFrom(VXPPlayFrom newVal)
{
	m_PlayFrom = newVal;
	return S_OK;
}

STDMETHODIMP CVXPSound::get_Res(BSTR *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_Res.SysAllocString();
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_Res(BSTR newVal)
{
	VXPLIB_BEGIN

	m_Res = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_Handle(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = (long)m_Handle;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_Param(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_Param;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_Param(long newVal)
{
	VXPLIB_BEGIN

	m_Param = newVal;
	
	VXPLIB_END
}

HRESULT CVXPSound::FinalConstruct()
{
	return S_OK;
}

HRESULT CVXPSound::FinalRelease()
{
	UninitializeHandle();
	return S_OK;
}

STDMETHODIMP CVXPSound::get_Balance(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_Balance;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_Balance(long newVal)
{
	VXPLIB_BEGIN

	m_Balance = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_Frequency(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_Frequency;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_Frequency(long newVal)
{
	VXPLIB_BEGIN

	m_Frequency = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_StartPos(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_StartPos;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_StartPos(long newVal)
{
	VXPLIB_BEGIN

	m_StartPos = newVal;

	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_EndPos(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_EndPos;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_EndPos(long newVal)
{
	VXPLIB_BEGIN

	m_EndPos = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_Loop(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_Loop;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_Loop(long newVal)
{
	VXPLIB_BEGIN

	m_Loop = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_Info(BSTR *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_Info.SysAllocString();
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_Info(BSTR newVal)
{
	VXPLIB_BEGIN

	m_Info = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_Delay(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_Delay;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_Delay(long newVal)
{
	VXPLIB_BEGIN

	m_Delay = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_Pause(long *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_Pause;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_Pause(long newVal)
{
	VXPLIB_BEGIN

	m_Pause = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::get_KeepSettings(VARIANT_BOOL *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_bKeepSettings;
	
	VXPLIB_END
}

STDMETHODIMP CVXPSound::put_KeepSettings(VARIANT_BOOL newVal)
{
	VXPLIB_BEGIN

	m_bKeepSettings = newVal?true:false;
	
	VXPLIB_END
}
