/*
 * VXPLib COMPONENT
 *
 * Module: VXPSound.h - Declaration of class CVXPSound
 *
 * Synopsis: Class CVXPSound implements methods and properties
 * of interface IVXPSound.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#ifndef __VXPSOUND_H_
#define __VXPSOUND_H_

#include "resource.h"       // main symbols
#include "Tag.h"
#include "GlobalObject.h"

/////////////////////////////////////////////////////////////////////////////
// CVXPSound
class ATL_NO_VTABLE CVXPSound : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CVXPSound, &CLSID_VXPSound>,
	public ISupportErrorInfo,
	public IDispatchImpl<IVXPSound, &IID_IVXPSound, &LIBID_VXPLibrary>,
	public CVXPException<&CLSID_VXPSound, &IID_IVXPSound>
{

public:
	CVXPSound();

	void SetupSound(CTag * tag, CGlobalObject * pGlobalObject);

DECLARE_REGISTRY_RESOURCEID(IDR_VXPSOUND)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CVXPSound)
	COM_INTERFACE_ENTRY_IID(CLSID_VXPSound, CVXPSound)
	COM_INTERFACE_ENTRY(IVXPSound)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
END_COM_MAP()

	HRESULT FinalConstruct();
	HRESULT FinalRelease();

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IVXPSound
public:
	STDMETHOD(get_KeepSettings)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_KeepSettings)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_Pause)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_Pause)(/*[in]*/ long newVal);
	STDMETHOD(get_Delay)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_Delay)(/*[in]*/ long newVal);
	STDMETHOD(get_Info)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_Info)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_Loop)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_Loop)(/*[in]*/ long newVal);
	STDMETHOD(get_EndPos)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_EndPos)(/*[in]*/ long newVal);
	STDMETHOD(get_StartPos)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_StartPos)(/*[in]*/ long newVal);
	STDMETHOD(get_Frequency)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_Frequency)(/*[in]*/ long newVal);
	STDMETHOD(get_Balance)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_Balance)(/*[in]*/ long newVal);
	STDMETHOD(get_Param)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_Param)(/*[in]*/ long newVal);
	STDMETHOD(get_Handle)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_Res)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_Res)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_PlayFrom)(/*[out, retval]*/ VXPPlayFrom *pVal);
	STDMETHOD(put_PlayFrom)(/*[in]*/ VXPPlayFrom newVal);
	STDMETHOD(get_Size)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_Size)(/*[in]*/ long newVal);
	STDMETHOD(get_FilePath)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_FilePath)(/*[in]*/ BSTR newVal);
	STDMETHOD(SetDefaults)();
	STDMETHOD(get_Volume)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_Volume)(/*[in]*/ long newVal);

private:

	// Pure sound parameters:
	long m_Volume;
	long m_Balance;
	long m_Param;
	long m_Frequency;
	long m_Pause;
	long m_StartPos;
	long m_EndPos;
	long m_Loop;
	long m_Delay;
	CString m_Info;
	// **********************

	// To manage temporary files:
	CString m_TempDir;
	CString m_TempFilePath; // Temporary file path in unicode, because m_FilePath can be changed.
	// *************************


	bool m_bKeepSettings; // Tells to keep all settings unless tag <BGSOUND> is met again.
	bool m_bWasInitialized;

	VXPPlayFrom m_PlayFrom;
	VXPPlayFrom m_LastPlayFrom;
	long m_SizeLimit;
	CString m_FilePath;
	CString m_Res;
	HANDLE m_Handle;
	
	CGlobalObject * m_pGlobalObject;

	void SetDefaultParameters();
	
	void InitializeHandle();
	void UninitializeHandle();

	// Returns sound data from pair FILE/RES, or NULL, if failed.
	// Returns the size of data as the parameter.
	BYTE * GetDataFromFileRes(int & nSize);
};

#endif //__VXPSOUND_H_
