/*
 * VXPLib COMPONENT
 *
 * Module: VXPTooltip.h - Declaration of class CVXPTooltip
 *
 * Synopsis: Class CVXPTooltip implements methods and properties
 * of interface IVXPTooltip.
 * The actual implementation of the tooltip window is provided
 * by class CToolWnd, while this class merely forwards all its calls
 * to that class.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#ifndef __VXPTOOLTIP_H_
#define __VXPTOOLTIP_H_

#include "resource.h"       // main symbols
#include "VXPLibCP.h"
#include "ToolWnd.h"

/////////////////////////////////////////////////////////////////////////////
// CVXPTooltip
class ATL_NO_VTABLE CVXPTooltip : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CVXPTooltip, &CLSID_VXPTooltip>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CVXPTooltip>,
	public IDispatchImpl<IVXPTooltip, &IID_IVXPTooltip, &LIBID_VXPLibrary>,
	public CProxy_IVXPTooltipEvents< CVXPTooltip >,
	public IDispEventImpl<0, CVXPTooltip, &DIID__IVXPQuickHTMLEvents, &LIBID_VXPLibrary, 1, 0>,
	public CVXPException<&CLSID_VXPTooltip, &IID_IVXPTooltip>
{
	friend class CToolWnd;
	friend class CVXPTooltipManager;
public:
	CVXPTooltip();

DECLARE_REGISTRY_RESOURCEID(IDR_VXPTOOLTIP)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CVXPTooltip)
	COM_INTERFACE_ENTRY_IID(CLSID_VXPTooltip, CVXPTooltip)
	COM_INTERFACE_ENTRY(IVXPTooltip)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(CVXPTooltip)
CONNECTION_POINT_ENTRY(DIID__IVXPTooltipEvents)
END_CONNECTION_POINT_MAP()

BEGIN_SINK_MAP(CVXPTooltip)
	SINK_ENTRY_EX(0, DIID__IVXPQuickHTMLEvents, 3, OnSetDefaults)
	SINK_ENTRY_EX(0, DIID__IVXPQuickHTMLEvents, 5, OnRecalculate)
END_SINK_MAP()

	HRESULT FinalConstruct();
	HRESULT FinalRelease();

	HRESULT __stdcall OnSetDefaults();
	HRESULT __stdcall OnRecalculate(VARIANT_BOOL Begin, VARIANT_BOOL SourceChanged);

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IVXPTooltip
public:
	STDMETHOD(get_SyncWnd)(/*[out, retval]*/ long * pVal);
	STDMETHOD(put_SyncWnd)(/*[in]*/ long newVal);
	STDMETHOD(get_MouseClickResponse)(/*[out, retval]*/ VXPMouseClickResponse * pVal);
	STDMETHOD(put_MouseClickResponse)(/*[in]*/ VXPMouseClickResponse newVal);
	STDMETHOD(Destroy)(/*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(Create)(/*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(UnSubscribe)(/*[in]*/ long uMsg, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(Subscribe)(/*[in]*/ long uMsg, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(DefWndProc)(/*[in]*/ long uMsg, /*[in]*/ long wParam, /*[in]*/ long lParam, /*[out, retval]*/ long * result);
	STDMETHOD(get_Theme)(/*[out, retval]*/ enum VXPTooltipTheme *pVal);
	STDMETHOD(put_Theme)(/*[in]*/ enum VXPTooltipTheme newVal);
	STDMETHOD(get_FadeSound)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_FadeSound)(/*[in]*/ short newVal);
	STDMETHOD(get_hWnd)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_ShowWhenEmpty)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_ShowWhenEmpty)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_TraceMode)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_TraceMode)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_HasShadow)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_HasShadow)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_BorderColor)(/*[out, retval]*/ OLE_COLOR *pVal);
	STDMETHOD(put_BorderColor)(/*[in]*/ OLE_COLOR newVal);
	STDMETHOD(get_HasBorder)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_HasBorder)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_State)(/*[out, retval]*/ VXPTooltipState *pVal);
	STDMETHOD(get_Round)(/*[out, retval]*/ BYTE *pVal);
	STDMETHOD(put_Round)(/*[in]*/ BYTE newVal);
	STDMETHOD(get_HasTail)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_HasTail)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_Transparency)(/*[out, retval]*/ BYTE *pVal);
	STDMETHOD(put_Transparency)(/*[in]*/ BYTE newVal);
	STDMETHOD(get_FadeHide)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_FadeHide)(/*[in]*/ short newVal);
	STDMETHOD(get_FadeShow)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_FadeShow)(/*[in]*/ short newVal);
	STDMETHOD(get_Autohide)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_Autohide)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(Hide)(/*[in]*/ VARIANT_BOOL NoFadingEffect, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(ShowAt)(/*[in]*/ short x, /*[in]*/ short y, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(Show)(/*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(get_html)(/*[out, retval]*/ IVXPQuickHTML ** pVal);

private:

	CToolWnd m_tooltip;
	IVXPQuickHTML * m_pHTML;

	IVXPSettings * m_pSettings;
	IVXPMargins * m_pMargins;
	IVXPFont * m_pFont;
	IVXPSound * m_pSound;

	bool m_bCalling; // True when one of the methods is being called: Show(), ShowAt(...) or Hide(...)
};

#endif //__VXPTOOLTIP_H_
