/*
 * VXPLib COMPONENT
 *
 * Module: VXPTooltipManager.cpp - Implementation of class CVXPTooltipManager
 *
 * Synopsis: Class CVXPTooltipManager implements methods and properties
 * of interface IVXPTooltipManager.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "VXPLib.h"
#include "VXPTooltipManager.h"
#include "commctrl.h"

_ATL_FUNC_INFO StateChangedInfo = {CC_STDCALL, VT_EMPTY, 1, {VT_I4}};

#define MAX_CLASS_NAME	255	// Maximum length for a class name

#define MAX_TEXT_LENGTH	30	// Maximum length of text lines to be
							// used in debug windows

/////////////////////////////////////////////////////////////////////////////
// CWndProcList
/////////////////////////////////////////////////////////////////////////////

// There can be only a single Manager attached to a menu at the same time:
//CVXPTooltipManager * g_MenuHandlingOwner = NULL;

CTMList * CVXPTooltipManager::m_pMenuHandlers = NULL;

/////////////////////////////////////////////////////////////////////////////
// CVXPTooltipManager
/////////////////////////////////////////////////////////////////////////////

CVXPTooltipManager::CVXPTooltipManager()
{
	m_bIgnoreMenus = false;

	m_bGotTimer = false;
	m_Timer = NULL;

	m_hActivator = NULL;

	m_Exception = 0;

	m_bUseMenuItemID = false;

	m_hLastWnd = NULL;

	// Resetting coordinates of the last cursor position:
	m_LastCursor.x = 0;
	m_LastCursor.y = 0;

	m_pMenuClasses = new CClassList(_T("#32768"));
	m_pToolbarClasses = new CClassList(_T("ToolbarWindow32"));

	m_uHideDelay = 0;
	m_nHideDelayCounter = 0;
	m_pForbiddenTool = NULL;
	m_pLastDisplayedTool = NULL;
	m_hContextMenuOwner = NULL;
	m_nMenuCommand = 0;

	m_hOldMenuHandler = NULL;

	m_bActive = true;	// Active by default
	
	m_uShowDelay = 500;
	m_bWaiting = false;
	m_nDelayed = 0;
	m_pCurrentTool = NULL;
	m_pPreviousTool = NULL;
	m_state = ttsHidden;

	m_bDebugMode = false;
	m_hGhostWnd = NULL;

	m_b_sfdMenuItems = true;
	m_b_sfdToolbarButtons = true;
}

CVXPTooltipManager::~CVXPTooltipManager()
{
	DELETE_COLLECTION(m_pMenuClasses)
	DELETE_COLLECTION(m_pToolbarClasses)
}

STDMETHODIMP CVXPTooltipManager::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IVXPTooltipManager
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

LRESULT CVXPTooltipManager::OnVXPTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled)
{
	// Do not do anything when the tooltip manager is passive,
	if(!m_bActive)
		return 0;

	// When activator is used, and the activating window is not active:
	if(m_hActivator && ::GetActiveWindow() != m_hActivator)
	{
		ResetAll();
		return 0;
	}

// Find out if the tooltip must get hidden after a delay:

	if((((m_uHideDelay || m_nHideDelayCounter) && m_state == ttsShown) || (m_nHideDelayCounter && m_state == ttsShowing))&& !m_bDebugMode)
	{
		m_nHideDelayCounter += GLOBAL_TIMER_DELAY;
		if(m_nHideDelayCounter >= m_uHideDelay)
		{
			m_nHideDelayCounter = 0;
			HideTool();
			m_pForbiddenTool = m_pCurrentTool;
		}
	}

//*******************************************************

	POINT cursor;
	GetCursorPos(&cursor);

	if(!::IsWindow(m_hContextMenuOwner) || !::IsWindowVisible(m_hContextMenuOwner))
		m_hContextMenuOwner = NULL;

	HookMenu(&cursor);

	bool cursor_changed = (cursor.x != m_LastCursor.x || cursor.y != m_LastCursor.y);

	GetCursorPos(&m_LastCursor);

	if(m_bWaiting)
	{
		if(m_state == ttsHidden || m_state == ttsHiding)
			m_nDelayed += GLOBAL_TIMER_DELAY;
		if(cursor_changed)
		{
			if(!m_pTooltipClass->m_tooltip.m_bTraceMode || m_bDebugMode || m_pCurrentTool == &m_GhostTool)
				HideTool();

			m_bWaiting = false;
			m_nDelayed = 0;
		}
		else
		{
			if(WindowFromPoint(cursor) != m_hLastWnd) //If it is a different window from when we set Wait=True
			{
				m_bWaiting = false;
				m_nDelayed = 0;
				HideTool();
			}
			else
				if(m_nDelayed >= m_uShowDelay && (m_state == ttsHidden || m_pCurrentTool == m_pPreviousTool) && (m_state == ttsHidden || !m_bDebugMode))
				{
					m_bWaiting = false;
					m_nDelayed = 0;
					m_pPreviousTool = m_pCurrentTool;
					if(m_state != ttsShown && m_state != ttsShowing)
						ShowCurrentTooltip(&cursor);
				}
		}
	}
	else
	{
		HWND wnd = WindowFromPoint(cursor);
		::ScreenToClient(wnd, &cursor);
		CToolList * pTL = m_pCurrentTool;
		m_pCurrentTool = GetToolFromWindow(wnd, &cursor);

		if(m_pCurrentTool == m_pForbiddenTool && m_pCurrentTool)
		{
			HideTool();
			return 0;
		}
		else
		{
			if(m_pForbiddenTool)
			{
				m_pForbiddenTool = NULL;
				m_nHideDelayCounter = 0;
			}
		}
		if(m_pCurrentTool && !m_pForbiddenTool)
		{
			// By this code we put a ban on a situation when both Autohide
			// and TraceMode are false, and yet the tooltip gets hidden when
			// the mouse cursor moves:

			if(m_pCurrentTool != pTL) // If a new tool found
				HideTool();
		
			m_hLastWnd = wnd;
			m_bWaiting = true;
			m_nDelayed = 0;
		}
		else
			HideTool();
	}
	return 0;
}

LRESULT CVXPTooltipManager::OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled)
{
	if(!m_bGotTimer && ::IsWindow(m_hWnd))
		m_bGotTimer = g_Timer->SetTimer(m_hWnd);

	return 0;
}

HRESULT CVXPTooltipManager::FinalConstruct()
{
	::CoCreateInstance(CLSID_VXPTooltip, NULL, CLSCTX_ALL, IID_IVXPTooltip, (void**)&m_pTooltip);
	m_pTooltip->QueryInterface(CLSID_VXPTooltip, (void**)&m_pTooltipClass);
	
	m_pTooltip->put_TraceMode(TRUE);
	m_pTooltip->put_Autohide(FALSE);

	::CoCreateInstance(CLSID_VXPWndToolList, NULL, CLSCTX_ALL, IID_IVXPWndToolList, (void**)&m_pWndList);
	m_pWndList->QueryInterface(CLSID_VXPWndToolList, (void**)&m_pWndListClass);

	::CoCreateInstance(CLSID_VXPCmdToolList, NULL, CLSCTX_ALL, IID_IVXPCmdToolList, (void**)&m_pCmdList);
	m_pCmdList->QueryInterface(CLSID_VXPCmdToolList, (void**)&m_pCmdListClass);

	m_pCmdListClass->m_pManager = this;
	m_pWndListClass->m_pManager = this;

	DispEventAdvise(m_pTooltip);
	Release();

	RECT r = {0, 0, 0, 0};
	Create(NULL, r, NULL, WS_POPUP);

	m_Timer = SetTimer(TIMER_ID, 1000); // Call every second
	m_bGotTimer = g_Timer->SetTimer(m_hWnd);

	return S_OK;
}

HRESULT CVXPTooltipManager::FinalRelease()
{
	if(m_Timer)
		KillTimer(m_Timer);
	if(m_bGotTimer)
		g_Timer->KillTimer(m_hWnd);

	AddRef();
	DispEventUnadvise(m_pTooltip);
	m_pTooltipClass->Release();
	m_pTooltip->Release();
	
	m_pWndListClass->Release();
	m_pWndList->Release();

	m_pCmdListClass->Release();
	m_pCmdList->Release();

	DestroyWindow();

	UnhookMenu();

	return S_OK;
}

CToolList * CVXPTooltipManager::GetToolFromWindow(HWND hWnd, LPPOINT pt)
{
	// Make sure the element resides within this process
	// and this thread:
	if(GetCurrentThreadId() != GetWindowThreadProcessId(hWnd, NULL))
		return NULL;

	CString name;
	GetClassName(hWnd, name.PreAllocate(MAX_CLASS_NAME), MAX_CLASS_NAME);
	name.PostAllocate();
	name.ShrinkEffective();
	m_ClassName = name;

// If a toolbar has a button with tooltip attached to it such tooltip
// is to be shown even if the whole toolbar window has a tooltip attached
// to it:
	if(FindClassName(m_pToolbarClasses, name))
		return GetToolFromToolbar(hWnd, pt);

	CToolWndList * A = m_pWndListClass->m_pList;
	while(A)
	{
		if(A->m_hWnd == hWnd)
		{
			if(m_bDebugMode)
				break;
			if(A->m_bActive)
				return A;
			else
				A = NULL;	// Not found, i.e. use OnActivateCustomTooltip even if the control
							// was found but the tooltip wasn't active

			break;	// Otherwise, just continue the search.
					// Even if a general for the window tooltip is not active,
					// the window still might have command elements that are active.
		}
		A = (CToolWndList*)A->Next;
	}

	// If such window wasn't found in the list of registered windows
	if(FindClassName(m_pMenuClasses, name))
		return GetToolFromMenu(hWnd);

	CToolCmdList * B = NULL;
	if(!A)
	{
		long ID = ::GetWindowLong(hWnd, GWL_ID);
		// Trying to find such window by its Command ID
		B = (CToolCmdList*)m_pCmdListClass->m_pList;
		while(B)
		{
			if(B->m_ID == ID)
			{
				if(m_bDebugMode)
					break;
				if(B->m_bActive)
					return B;
				else
					B = NULL;	// Not found, i.e. use OnActivateCustomTooltip even if the control
								// was found but the tooltip wasn't active
				break;
			}
			B = (CToolCmdList*)B->Next;
		}
	}

	if(m_bDebugMode)
	{
		m_DebugTool.m_text.Empty();
		if(A)
			return GetDebugWindowInfo(hWnd, pt, A?true:false, (A && A->m_bActive)?true:false);
		else
			return GetDebugWindowInfo(hWnd, pt, B?true:false, (B && B->m_bActive)?true:false);
	}
	else
		if(B)
		{
			if(B->m_bActive)
				return B;
			else
				return NULL;
		}

// Otherwise, we can try a ghost tool

	return &m_GhostTool;
}

STDMETHODIMP CVXPTooltipManager::get_ShowDelay(short * pVal)
{
	VXPLIB_BEGIN

	*pVal = m_uShowDelay;
	
	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::put_ShowDelay(short newVal)
{
	VXPLIB_BEGIN

	m_uShowDelay = newVal;
	
	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::get_tool(IVXPTooltip **pVal)
{
	VXPLIB_BEGIN

	*pVal = m_pTooltip;
	m_pTooltip->AddRef();

	VXPLIB_END
}

HRESULT __stdcall CVXPTooltipManager::OnStateChanged(VXPTooltipState NewState)
{
	m_state = NewState;
	
	if(NewState == ttsShown)
		m_nHideDelayCounter = 0;
	
	if(NewState == ttsHidden)
	{
		m_nDelayed = 0;
		m_nHideDelayCounter = 0;
		m_bWaiting = false;
	}

	return S_OK;
}

HRESULT __stdcall CVXPTooltipManager::OnMouseClicked(LONG x, LONG y, LONG uMsg, LONG hWnd, LONG wHitTestCode, VARIANT_BOOL * bProcessed)
{
	switch(m_pTooltipClass->m_tooltip.m_MouseClickResponse)
	{
	case mcrHideAlways:
		{
			MarkDirty();
			*bProcessed = TRUE;
			break;
		}
	case mcrHideWhenShown:
		{
			if(m_state == ttsShown)
			{
				MarkDirty();
				*bProcessed = TRUE;
			}
			break;
		}
	case mcrMoveToCursor: // This behaviour is not applicable for VXPTooltipManager
		{
			*bProcessed = TRUE;
			break;
		}
	default:
		break;
	}
	return S_OK;
}

void CVXPTooltipManager::ShowCurrentTooltip(POINT * cursor)
{
	BSTR Text = NULL;

	if(m_pCurrentTool == &m_GhostTool)
	{
		HWND hWnd = WindowFromPoint(*cursor);
		if(GetCurrentThreadId() != GetWindowThreadProcessId(hWnd, NULL))
			return;
		CString name;
		GetClassName(hWnd, name.PreAllocate(MAX_CLASS_NAME), MAX_CLASS_NAME);
		name.PostAllocate();
		BSTR ClassName = name.SysAllocString();
		::ScreenToClient(hWnd, cursor);
		Fire_OnActivateCustomTooltip((long)hWnd, ClassName, (short)cursor->x, (short)cursor->y, &Text);
		SysFreeString(ClassName);
		if(Text)
			m_hGhostWnd = hWnd;
		else
			return;
	}
	else
		Text = m_pCurrentTool->m_text.SysAllocString();


// In here we force a condition under which RecalculateLayout() is called.
// However, in practice the tooltip contents may change at any moment due to
// dynamic self-modification. An attempt to keep tracking such changes by
// calling RecalculateLayout would overload the CPU. It is responsibility
// of the client application, when it suspects a change in the dynamic
// contents of the tooltip to call RecalculateLayout.
// It will be however responsibility of the tooltip object to respond
// to every contents recalculation and display updated context.
	if(m_pCurrentTool != m_pLastDisplayedTool || m_bDebugMode || m_pCurrentTool == &m_GhostTool)
	{
		IVXPQuickHTML * pHTML;
		m_pTooltip->get_html(&pHTML);
		pHTML->SetSourceText(Text);
		VARIANT_BOOL success;
		pHTML->RecalculateLayout(&success);
		pHTML->Release();
	}

	m_pLastDisplayedTool = m_pCurrentTool;
	
	VARIANT_BOOL success;
	m_pTooltip->Show(&success);

	SysFreeString(Text);
}

CToolList * CVXPTooltipManager::GetToolFromToolbar(HWND hWnd, LPPOINT pt)
{
	int index = ::SendMessage(hWnd, TB_HITTEST, 0, (long)pt);
	if(index < 0)
		return &m_GhostTool;

	TBBUTTON info;
	::SendMessage(hWnd, TB_GETBUTTON, index, (long)&info);

	CToolList * result = NULL;
	CToolCmdList * A = m_pCmdListClass->m_pList;
	while(A)
	{
		if(A->m_ID == info.idCommand)
		{
			if(A->m_bActive)
			{
				result = A;
				break;
			}
			else
				if(m_bDebugMode)
					break;
		}
		A = (CToolCmdList*)A->Next;
	}

	if(m_bDebugMode)
	{
		m_DebugTool.m_text.Empty();
		return GetDebugToolbarInfo(hWnd, index, info.idCommand, A?true:false, result?true:false);
	}

	if((info.fsState&TBSTATE_ENABLED || m_b_sfdToolbarButtons) && result)
		return result;

	return &m_GhostTool;
}


// Function name	: CVXPTooltipManager::MenuHandler
// Description	    : Handles all messages sent to the window that owns the current menu
// Return type		: LRESULT CALLBACK 
// Argument         : HWND hwnd
// Argument         : UINT uMsg
// Argument         : WPARAM wParam
// Argument         : LPARAM lParam
LRESULT CALLBACK CVXPTooltipManager::MenuHandler(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	//////////////////////////////////////////////////
	// Vitaly Tomilov; 08/05/2004;
	//
	// We need to unhook the window with menu
	// whenever a command is issued, because sometimes
	// it can cause a crash, especially in VS.NET:
	WNDPROC ProcAddress = m_pMenuHandlers->m_pTM->m_OldWndProc; // Remember the old procedure address
	if(uMsg == WM_COMMAND)						// If a command is issued
		m_pMenuHandlers->m_pTM->UnhookMenu();	// Release the hooked window
	//
	// Vitaly Tomilov; 08/05/2004;
	/////////////////////////////////////////////////

	if(uMsg == WM_MENUSELECT)
	{
		WORD HighWord = HIWORD(wParam);
		CTMList * pList = m_pMenuHandlers;
		while(pList)
		{
			CVXPTooltipManager * pObject = pList->m_pTM;
			if((HighWord&MF_MOUSESELECT) && (!(HighWord&MF_POPUP) || pObject->m_bUseMenuItemID))
			{
				if(pObject->m_bUseMenuItemID)
				{
					MENUITEMINFO info;
					info.cbSize = sizeof(MENUITEMINFO);
					info.fMask = MIIM_DATA;
					::GetMenuItemInfo((HMENU)lParam, LOWORD(wParam), (HIWORD(wParam)&MF_POPUP)?TRUE:FALSE, &info);
					pObject->m_nMenuCommand = info.dwItemData;
				}
				else
					pObject->m_nMenuCommand = LOWORD(wParam);
				pObject->m_hMenu = (HMENU)lParam;
			}
			else
				pObject->m_nMenuCommand = 0;
			pList = (CTMList*)pList->Next;
		}
	}
	// Calling the default procedure through the very first
	// element in collection, as it is the one actually overriding events:
	return ::CallWindowProc(ProcAddress, hwnd, uMsg, wParam, lParam);
}

CToolList * CVXPTooltipManager::GetToolFromMenu(HWND hWnd)
{
	if(!m_nMenuCommand)
		return &m_GhostTool;

	CToolCmdList * A = m_pCmdListClass->m_pList;
	CToolList * result = NULL;
	while(A)
	{
		if(A->m_ID == m_nMenuCommand)
		{
			if(A->m_bActive)
			{
				result = A;
				break;
			}
			else
				if(m_bDebugMode)
					break;
		}
		A = (CToolCmdList*)A->Next;
	}

	if(m_bDebugMode)
	{
		m_DebugTool.m_text.Empty();
		return GetDebugMenuInfo(hWnd, A?true:false, (result)?true:false);
	}

	if(result)
	{
		MENUITEMINFO info;
		info.cbSize = sizeof(MENUITEMINFO);
		info.fMask = MIIM_STATE;
		GetMenuItemInfo(m_hMenu, m_nMenuCommand, FALSE, &info);
		if(!(info.fState&MFS_GRAYED) || m_b_sfdMenuItems)
			return result;
	}

	return &m_GhostTool;
}

void CVXPTooltipManager::HookMenu(LPPOINT pt)
{
	if(m_bIgnoreMenus) // If all menus are to be ignored by this Tooltip Manager
		return;

	HWND wnd = WindowFromPoint(*pt);
	HWND parent = ::GetParent(wnd);

	if(!parent && ::IsWindow(m_hContextMenuOwner))
		parent = m_hContextMenuOwner;

	if(!parent)
	{
		UnhookMenu();
		return;
	}

	if(parent == m_hOldMenuHandler) // Then this menu is already hooked by this interface
		return;

	// Unhook whatever else
	UnhookMenu();

	// Make sure the menu resides within this process and this thread:
	if(GetCurrentThreadId() != GetWindowThreadProcessId(wnd, NULL))
		return;

	CString name;
	GetClassName(wnd, name.PreAllocate(MAX_CLASS_NAME), MAX_CLASS_NAME);
	name.PostAllocate();
	
	if(FindClassName(m_pMenuClasses, name)) // If it is a menu class
	{
		m_hOldMenuHandler = parent;
		if(m_pMenuHandlers)
			m_pMenuHandlers->Add(new CTMList(this));
		else
		{
			m_pMenuHandlers = new CTMList(this);
			m_OldWndProc = (WNDPROC)::GetWindowLong(m_hOldMenuHandler, GWL_WNDPROC);
			::SetWindowLong(m_hOldMenuHandler, GWL_WNDPROC, (long)MenuHandler);
		}
	}
}

void CVXPTooltipManager::UnhookMenu()
{
	// First of all we need to find where in the list of menu
	// handlers this object is:

	CTMList * pList = m_pMenuHandlers;
	while(pList)
	{
		CVXPTooltipManager * pObject = pList->m_pTM;
		if(pObject == this) // Found the object
		{
			if(pList->Prev) // If not the first object in the list
			{
				m_hOldMenuHandler = NULL;
				m_nMenuCommand = 0;
				pList->Remove(); // Remove from the list
			}
			else // The very first object in the list
			{
				if(pList->Next) // If there're other clients
				{
					// Copying all necessary information from this
					// object into the next one:
					CVXPTooltipManager * pManager = ((CTMList*)pList->Next)->m_pTM;
					pManager->m_hOldMenuHandler = m_hOldMenuHandler;
					pManager->m_OldWndProc = m_OldWndProc;
					m_pMenuHandlers = (CTMList*)pList->Next;
					pList->Remove();
				}
				else // This is the last client object
				{
					// Make sure the hooked window still exists
					if(::IsWindow(m_hOldMenuHandler))
						::SetWindowLong(m_hOldMenuHandler, GWL_WNDPROC, (long)m_OldWndProc);
					DELETE_COLLECTION(m_pMenuHandlers)
				}
				m_hOldMenuHandler = NULL;
				m_nMenuCommand = 0;
			}
			break;
		}
		pList = (CTMList*)pList->Next;
	}
}

// This method is to be called from dialog boxes mostly
STDMETHODIMP CVXPTooltipManager::UpdateContextMenuOwner(long hWnd, VARIANT_BOOL * success)
{
	VXPLIB_BEGIN

	*success = FALSE;

	// We must make sure the passed window handle is valid and it belongs
	// to the current process and the current thread.
	// If this requirement is not setisfied, then we return a error,
	// and generate an exception.
	if(!::IsWindow((HWND)hWnd) || (GetCurrentThreadId() != GetWindowThreadProcessId((HWND)hWnd, NULL)))
		AddException(EX_INVALIDHANDLE);
	else
	{
		m_hContextMenuOwner = (HWND)hWnd;
		*success = TRUE;
	}
	
	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::get_windows(IVXPWndToolList ** pVal)
{
	VXPLIB_BEGIN

	m_pWndList->AddRef();
	*pVal = m_pWndList;
	
	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::get_commands(IVXPCmdToolList ** pVal)
{
	VXPLIB_BEGIN

	m_pCmdList->AddRef();
	*pVal = m_pCmdList;

	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::AddMenuClass(BSTR ClassName)
{
	VXPLIB_BEGIN

	m_pMenuClasses->Add(new CClassList(ClassName));

	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::AddToolbarClass(BSTR ClassName)
{
	VXPLIB_BEGIN

	m_pToolbarClasses->Add(new CClassList(ClassName));

	VXPLIB_END
}

bool CVXPTooltipManager::FindClassName(CClassList * source, LPCTSTR ClassName)
{
	if(!source)
		return false;
	CClassList * A = (CClassList*)source->GetFirst();
	while(A)
	{
		if(A->m_name == ClassName)
			return true;
		A = (CClassList*)A->Next;
	}
	return false;
}

STDMETHODIMP CVXPTooltipManager::get_Active(VARIANT_BOOL *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_bActive;

	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::put_Active(VARIANT_BOOL newVal)
{
	VXPLIB_BEGIN

	m_bActive = newVal?true:false;
// Now, if m_bActive is true, and tooltip is shown, or a menu
// is handled, such activity must be ceased immediately:
	if(!m_bActive)
		ResetAll();
	
	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::get_DebugMode(VARIANT_BOOL *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_bDebugMode;

	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::put_DebugMode(VARIANT_BOOL newVal)
{
	VXPLIB_BEGIN

	m_bDebugMode = newVal?true:false;
	ResetAll();
	
	VXPLIB_END
}

CToolList * CVXPTooltipManager::GetDebugMenuInfo(HWND hWnd, bool HasTooltip, bool TooltipActive)
{
// Information to display:

// Menu window:
// 1. Window Class
// 2. Window Handle
// 3. Menu Handle
// 4. Total Items

// Menu Owner:
// 5. Window Class
// 6. Window Handle

// Menu Item:
// 7. Title
// 8. Index
// 9. Command ID
// 10. Enabled (Yes/No)
// 11. Has Tooltip (Yes/No)


	HWND parent = ::GetParent(hWnd);
	if(!parent)
		parent = m_hContextMenuOwner;

	CString ParentName(_T(" "));
	if(parent)
	{
		if(!GetClassName(parent, ParentName.PreAllocate(MAX_CLASS_NAME), MAX_CLASS_NAME))
			parent = 0;
		ParentName.PostAllocate();
	}

	MENUITEMINFO info;
	info.cbSize = sizeof(MENUITEMINFO);
	info.fMask = MIIM_DATA;
	int N = GetMenuItemCount(m_hMenu);
	int Index;
	for(Index = 0;Index < N;Index ++)
	{
		if(m_bUseMenuItemID)
		{
			GetMenuItemInfo(m_hMenu, Index, TRUE, &info);
			if(info.dwItemData == (UINT)m_nMenuCommand)
				break;
		}
		else
			if(GetMenuItemID(m_hMenu, Index) == (UINT)m_nMenuCommand)
				break;
	}

	CString tmp;
	tmp.LoadString(_Module.GetModuleInstance(), IDS_MENUINFO);

	CString title;
	::GetMenuString(m_hMenu, m_nMenuCommand, title.PreAllocate(255), 255, MF_BYCOMMAND);
	title.PostAllocate();
	title = title.TextToHTML(MAX_TEXT_LENGTH);

	CString tool;
	tool.LoadString(_Module.GetModuleInstance(), HasTooltip?IDS_YES:IDS_NO);

	info.fMask = MIIM_STATE;

	GetMenuItemInfo(m_hMenu, m_nMenuCommand, FALSE, &info);

	CString enabled;
	enabled.LoadString(_Module.GetModuleInstance(), (info.fState&MFS_GRAYED)?IDS_NO:IDS_YES);

	CString active;
	active.LoadString(_Module.GetModuleInstance(), TooltipActive?IDS_YES:IDS_NO);


	CString msg;
	msg.LoadHTML(_Module.GetResourceInstance(), IDR_MENUINFO);
	m_DebugTool.m_text.FormatMessage(msg, (LPCTSTR)m_ClassName, hWnd, m_hMenu, N, (LPCTSTR)ParentName, parent, (LPCTSTR)title, Index, m_nMenuCommand, (LPCTSTR)enabled, (LPCTSTR)tool, (LPCTSTR)active);
	return &m_DebugTool;
}

CToolList* CVXPTooltipManager::GetDebugToolbarInfo(HWND hWnd, int ButtonIndex, int ButtonID, bool HasTooltip, bool TooltipActive)
{
// Information to display:

// Toolbar window:
// 1. Toolbar Class
// 2. Toolbar Title
// 3. Toolbar Handle
// 4. Total Items

// Toolbar Owner:
// 5. Window Class
// 6. Window Handle

// Toolbar Button:
// 7. Index
// 8. Command ID
// 9. Enabled (Yes/No)
// 10. Has Tooltip (Yes/No)

	CString title;
	::GetWindowText(hWnd, title.PreAllocate(255), 255);
	title.PostAllocate();
	title = title.TextToHTML(MAX_TEXT_LENGTH);

	int N = ::SendMessage(hWnd, TB_BUTTONCOUNT, 0, 0L);

	HWND parent = ::GetParent(hWnd);
	CString ParentName(_T(" "));
	if(parent)
	{
		if(!GetClassName(parent, ParentName.PreAllocate(MAX_CLASS_NAME), MAX_CLASS_NAME))
			parent = 0;
		ParentName.PostAllocate();
	}

	TBBUTTONINFO info;
	info.cbSize = sizeof(TBBUTTONINFO);
	info.dwMask = TBIF_STATE;
	::SendMessage(hWnd, TB_GETBUTTONINFO, ButtonID, (long)&info);

	CString enabled;
	enabled.LoadString(_Module.GetModuleInstance(), (info.fsState&TBSTATE_ENABLED)?IDS_YES:IDS_NO);

	CString tool;
	tool.LoadString(_Module.GetModuleInstance(), HasTooltip?IDS_YES:IDS_NO);

	CString active;
	active.LoadString(_Module.GetModuleInstance(), TooltipActive?IDS_YES:IDS_NO);

	CString msg;
	msg.LoadHTML(_Module.GetResourceInstance(), IDR_TOOLBARINFO);
	m_DebugTool.m_text.FormatMessage(msg, (LPCTSTR)m_ClassName, (LPCTSTR)title, hWnd, N, (LPCTSTR)ParentName, parent, ButtonIndex, ButtonID, (LPCTSTR)enabled, (LPCTSTR)tool, (LPCTSTR)active);
	return &m_DebugTool;
}

CToolList * CVXPTooltipManager::GetDebugWindowInfo(HWND hWnd, LPPOINT cursor, bool HasTooltip, bool TooltipActive)
{
// Information to display:

// Window:
// 1. Class Name
// 2. Title
// 3. Handle
// 4. Width
// 5. Height
// 6. Client Width
// 7. Cient Height
// 8. Command ID
// 9. Has Tooltip (Yes/No)

// Window Owner:
// 10. Class Name
// 11. Handle

// Mouse Cursor:
// 12. X
// 13. Y

	CString title;
	::GetWindowText(hWnd, title.PreAllocate(255), 255);
	title.PostAllocate();
	title = title.TextToHTML(MAX_TEXT_LENGTH);

	RECT r;
	::GetWindowRect(hWnd, &r);

	RECT client;
	::GetClientRect(hWnd, &client);

	int ID = ::GetWindowLong(hWnd, GWL_ID);

	CString tool;
	tool.LoadString(_Module.GetModuleInstance(), HasTooltip?IDS_YES:IDS_NO);

	CString active;
	active.LoadString(_Module.GetModuleInstance(), TooltipActive?IDS_YES:IDS_NO);

	HWND parent = ::GetParent(hWnd);
	CString ParentName(_T(" "));
	if(parent)
	{
		if(!GetClassName(parent, ParentName.PreAllocate(MAX_CLASS_NAME), MAX_CLASS_NAME))
			parent = 0;
		ParentName.PostAllocate();
	}

	CString msg;
	msg.LoadHTML(_Module.GetResourceInstance(), IDR_WINDOWINFO);
	m_DebugTool.m_text.FormatMessage(msg, (LPCTSTR)m_ClassName, (LPCTSTR)title, hWnd, r.right-r.left, r.bottom-r.top, client.right, client.bottom, ID, (LPCTSTR)tool, (LPCTSTR)active, (LPCTSTR)ParentName, parent, cursor->x, cursor->y);
	return &m_DebugTool;
}

STDMETHODIMP CVXPTooltipManager::get_sfdMenuItems(VARIANT_BOOL *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_b_sfdMenuItems;

	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::put_sfdMenuItems(VARIANT_BOOL newVal)
{
	VXPLIB_BEGIN

	m_b_sfdMenuItems = newVal?true:false;
	
	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::get_sfdToolbarButtons(VARIANT_BOOL *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_b_sfdToolbarButtons;

	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::put_sfdToolbarButtons(VARIANT_BOOL newVal)
{
	VXPLIB_BEGIN

	m_b_sfdToolbarButtons = newVal?true:false;
	
	VXPLIB_END
}

void CVXPTooltipManager::ToolModifiedNotify(CToolList *pTool)
{
	// If the latest displayed tool is the one that has been modified,
	// then we should reset it to set updated HTML source after
	// in ShowCurrentTooltip(...)
	if(m_pLastDisplayedTool == pTool)
		m_pLastDisplayedTool = NULL;
}

STDMETHODIMP CVXPTooltipManager::get_HideDelay(short * pVal)
{
	VXPLIB_BEGIN

	*pVal = m_uHideDelay;

	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::put_HideDelay(short newVal)
{
	VXPLIB_BEGIN

	m_uHideDelay = newVal;
	if(!m_uHideDelay)
	{
		m_pForbiddenTool = NULL;
		m_nHideDelayCounter = 0;
	}

	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::MarkDirty()
{
	VXPLIB_BEGIN

	m_nHideDelayCounter = max(m_uHideDelay, 1);

	VXPLIB_END
}

void CVXPTooltipManager::ResetAll()
{
	m_nHideDelayCounter = 0;
	m_nDelayed = 0;
	m_bWaiting = false;		// Stop waiting, if we are waiting to show the tooltip
	UnhookMenu();			// Unhook menu, if was hooked
	HideTool(true);			// Hide immediately, without any fading effect
	m_LastCursor.x = 0;
	m_LastCursor.y = 0;
}

STDMETHODIMP CVXPTooltipManager::get_UseMenuItemID(VARIANT_BOOL *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_bUseMenuItemID;
	
	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::put_UseMenuItemID(VARIANT_BOOL newVal)
{
	VXPLIB_BEGIN

	m_bUseMenuItemID = newVal?true:false;
	
	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::get_Activator(long *pVal)
{
	*pVal = (long)m_hActivator;
	return S_OK;
}

STDMETHODIMP CVXPTooltipManager::put_Activator(long newVal)
{
	VXPLIB_BEGIN

	if(!newVal || ::IsWindow((HWND)newVal))
		m_hActivator = (HWND)newVal;
	else
		AddException(EX_INVALIDHANDLE);

	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::AddToolbarWndClass(long hWnd, VARIANT_BOOL *success)
{
	VXPLIB_BEGIN

	*success = FALSE;

	if(::IsWindow((HWND)hWnd))
	{
		CString buffer;
		int nChars = GetClassName((HWND)hWnd, buffer.PreAllocate(255), 255);
		buffer.PostAllocate(nChars);
		if(nChars)
		{
			m_pToolbarClasses->Add(new CClassList(buffer));
			*success = TRUE;
		}
	}
	else
		AddException(EX_INVALIDHANDLE);

	VXPLIB_END
}

void CVXPTooltipManager::HideTool(bool NoFadingEffect)
{
	if(m_state != ttsHidden)
	{
		VARIANT_BOOL success;
		m_pTooltip->Hide(NoFadingEffect?TRUE:FALSE, &success);
	}
}

STDMETHODIMP CVXPTooltipManager::get_IgnoreMenus(VARIANT_BOOL *pVal)
{
	VXPLIB_BEGIN

	*pVal = m_bIgnoreMenus;

	VXPLIB_END
}

STDMETHODIMP CVXPTooltipManager::put_IgnoreMenus(VARIANT_BOOL newVal)
{
	VXPLIB_BEGIN

	m_bIgnoreMenus = newVal?true:false;

	VXPLIB_END
}
