/*
 * VXPLib COMPONENT
 *
 * Module: VXPTooltipManager.h - Declaration of class CVXPTooltipManager
 *
 * Synopsis: Class CVXPTooltipManager implements methods and properties
 * of interface IVXPTooltipManager.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#ifndef __VXPTOOLTIPMANAGER_H_
#define __VXPTOOLTIPMANAGER_H_

#include "resource.h"       // main symbols
#include "VXPLibCP.h"
#include "ToolList.h"
#include "AtlWin.h"

#include "VXPWndToolList.h"
#include "VXPCmdToolList.h"
#include "VXPTooltip.h"
#include "VXPQuickHTML.h"
#include "String.h"	// Added by ClassView

class CClassList:public HCollection
{
public:
	explicit CClassList(BSTR ClassName):m_name(ClassName){}
	explicit CClassList(LPCTSTR ClassName):m_name(ClassName){}

	CString m_name;
};

class CVXPTooltipManager;

class CTMList:public HCollection
{
public:
	CTMList(CVXPTooltipManager * pTM):m_pTM(pTM){};
	CVXPTooltipManager * m_pTM;
};

extern _ATL_FUNC_INFO StateChangedInfo;

/////////////////////////////////////////////////////////////////////////////
// CVXPTooltipManager
class ATL_NO_VTABLE CVXPTooltipManager : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CVXPTooltipManager, &CLSID_VXPTooltipManager>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CVXPTooltipManager>,
	public IDispatchImpl<IVXPTooltipManager, &IID_IVXPTooltipManager, &LIBID_VXPLibrary>,
	public CProxy_IVXPTooltipManagerEvents< CVXPTooltipManager >,
	public CWindowImpl<CVXPTooltipManager>,
	public IDispEventImpl<0, CVXPTooltipManager, &DIID__IVXPTooltipEvents, &LIBID_VXPLibrary, 1, 0>,
	public CVXPException<&CLSID_VXPTooltipManager, &IID_IVXPTooltipManager>
{
public:
	CVXPTooltipManager();
	~CVXPTooltipManager();

DECLARE_REGISTRY_RESOURCEID(IDR_VXPTOOLTIPMANAGER)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CVXPTooltipManager)
	COM_INTERFACE_ENTRY(IVXPTooltipManager)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(CVXPTooltipManager)
CONNECTION_POINT_ENTRY(DIID__IVXPTooltipManagerEvents)
END_CONNECTION_POINT_MAP()

BEGIN_SINK_MAP(CVXPTooltipManager)
	SINK_ENTRY_INFO(0, DIID__IVXPTooltipEvents, 0x1, OnStateChanged, &StateChangedInfo)
	SINK_ENTRY_EX(0, DIID__IVXPTooltipEvents, 0x6, OnMouseClicked)
END_SINK_MAP()

BEGIN_MSG_MAP(CVXPTooltipManager)
	MESSAGE_HANDLER(WM_TIMER, OnTimer)
	MESSAGE_HANDLER(WM_VXPTIMER, OnVXPTimer)
END_MSG_MAP()

	HRESULT __stdcall OnStateChanged(VXPTooltipState NewState);
	HRESULT __stdcall OnMouseClicked(LONG x, LONG y, LONG uMsg, LONG hWnd, LONG wHitTestCode, VARIANT_BOOL * bProcessed);

	HRESULT FinalConstruct();
	HRESULT FinalRelease();

	LRESULT OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled);
	LRESULT OnVXPTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled);

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IVXPTooltipManager
public:
	STDMETHOD(get_IgnoreMenus)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_IgnoreMenus)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(AddToolbarWndClass)(/*[in]*/ long hWnd, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(get_Activator)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_Activator)(/*[in]*/ long newVal);
	STDMETHOD(get_UseMenuItemID)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_UseMenuItemID)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(MarkDirty)();
	STDMETHOD(get_HideDelay)(/*[out, retval]*/ short * pVal);
	STDMETHOD(put_HideDelay)(/*[in]*/ short newVal);
	void ToolModifiedNotify(CToolList * pTool);
	STDMETHOD(get_sfdToolbarButtons)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_sfdToolbarButtons)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_sfdMenuItems)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_sfdMenuItems)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_DebugMode)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_DebugMode)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_Active)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_Active)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(AddToolbarClass)(/*[in]*/ BSTR ClassName);
	STDMETHOD(AddMenuClass)(/*[in]*/ BSTR ClassName);
	STDMETHOD(get_commands)(/*[out, retval]*/ IVXPCmdToolList * *pVal);
	STDMETHOD(get_windows)(/*[out, retval]*/ IVXPWndToolList * *pVal);
	STDMETHOD(UpdateContextMenuOwner)(/*[in]*/ long hWnd, /*[out, retval]*/ VARIANT_BOOL * success);
	STDMETHOD(get_tool)(/*[out, retval]*/ IVXPTooltip* *pVal);
	STDMETHOD(get_ShowDelay)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_ShowDelay)(/*[in]*/ short newVal);
	
	// *** Menu hooking properties
	WNDPROC m_OldWndProc;
	HWND m_hOldMenuHandler;
	HWND m_hContextMenuOwner;
	long m_nMenuCommand;
	HMENU m_hMenu;	// Valid only when m_nMenuCommand != 0;
	// ***************************

	IVXPWndToolList * m_pWndList;
	CVXPWndToolList * m_pWndListClass;
	IVXPCmdToolList * m_pCmdList;
	CVXPCmdToolList * m_pCmdListClass;

	void HookMenu(LPPOINT pt);
	void UnhookMenu();

	static LRESULT CALLBACK MenuHandler(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	CToolList* GetToolFromToolbar(HWND hWnd, LPPOINT pt);
	CToolList* GetDebugToolbarInfo(HWND hWnd, int ButtonIndex, int ButtonID, bool HasTooltip, bool TooltipActive);
	void ShowCurrentTooltip(POINT * cursor);
	
	CToolList * GetToolFromWindow(HWND hWnd, LPPOINT pt);
	CToolList * GetToolFromMenu(HWND hWnd);
		
	CToolList * m_pCurrentTool;
	CToolList * m_pPreviousTool;
	CToolList * m_pForbiddenTool;	// The tool for which we've just hid the tooltip after a delay
	
	CToolList * m_pLastDisplayedTool; // This one is to optimize the HTML source modification

	CToolList m_GhostTool;
	
	CToolList m_DebugTool;

	short m_uShowDelay;		// Showing delay
	short m_uHideDelay;		// Hiding delay

	bool m_bIgnoreMenus;	// When True menus are not to be hooked

	bool m_bWaiting;	// Indicates the manager found the control and is
						// waiting till the delay expires

	bool m_bUseMenuItemID;

	POINT m_LastCursor;

	// Counters of delays:
	int m_nDelayed;		
	int m_nHideDelayCounter;

	IVXPTooltip * m_pTooltip;
	CVXPTooltip * m_pTooltipClass;
	
	VXPTooltipState m_state;	// Current tooltip state

	CClassList * m_pMenuClasses;
	CClassList * m_pToolbarClasses;

private:
	void HideTool(bool NoFadingEffect = false);

	void ResetAll();
	CToolList * GetDebugWindowInfo(HWND hWnd, LPPOINT cursor, bool HasTooltip, bool TooltipActive);
	CToolList* GetDebugMenuInfo(HWND hWnd, bool HasTooltip, bool TooltipActive);
	bool FindClassName(CClassList * source, LPCTSTR ClassName);
	bool m_bActive;	// Indicates the active state of the tooltip manager
	bool m_bDebugMode;
	
	bool m_b_sfdMenuItems;
	bool m_b_sfdToolbarButtons;
	
	HWND m_hLastWnd; // The window for which we initiated Wait=True

	HWND m_hGhostWnd;
	CString m_ClassName;

	long m_Exception;

	HWND m_hActivator;

	bool m_bGotTimer;	// Contains True, if the timer has succeeded to initialize
	UINT m_Timer;		// Timer identificator returned by function SetTimer

// This member will hold a list of all interface pointers that
// need to handle events from a menu:
	static CTMList * m_pMenuHandlers;
};

#endif //__VXPTOOLTIPMANAGER_H_
