/*
 * VXPLib COMPONENT
 *
 * Module: VXPWndToolList.cpp - Implementation of class CVXPWndToolList
 *
 * Synopsis: Class CVXPWndToolList implements methods and properties
 * of interface IVXPWndToolList.
 *
 * Copyright (c) 2002, Vitaly Tomilov
 * Copyright (c) 2003-2005, Tooltips.Net L.L.C.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Library support web site - http://www.tooltips.net
 */

#include "stdafx.h"
#include "VXPLib.h"
#include "VXPWndToolList.h"
#include "VXPTooltipManager.h"

/////////////////////////////////////////////////////////////////////////////
// CVXPWndToolList

STDMETHODIMP CVXPWndToolList::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IVXPWndToolList
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CVXPWndToolList::Add(long hWnd, BSTR ToolTip)
{
	VXPLIB_BEGIN

	// We must make sure the passed window handle is valid and it belongs
	// to the current process and the current thread.
	// If this requirement is not setisfied, then we return a error,
	// and generate an exception.
	if(!::IsWindow((HWND)hWnd) || (GetCurrentThreadId() != GetWindowThreadProcessId((HWND)hWnd, NULL)))
		AddException(EX_INVALIDHANDLE);
	else
		ADD_COLLECTION(m_pList, new CToolWndList((HWND)hWnd, ToolTip))
	
	VXPLIB_END
}

STDMETHODIMP CVXPWndToolList::Modify(long hWnd, BSTR ToolTip, VARIANT_BOOL *success)
{
	VXPLIB_BEGIN

	CToolWndList * A = m_pList;
	*success = FALSE;
	while(A)
	{
		if(A->m_hWnd == (HWND)hWnd)
		{
			*success = TRUE;
			A->m_text = ToolTip;
			m_pManager->ToolModifiedNotify(A);
			break;
		}
		A = (CToolWndList*)A->Next;
	}

	VXPLIB_END
}

STDMETHODIMP CVXPWndToolList::Remove(long hWnd, VARIANT_BOOL *success)
{
	VXPLIB_BEGIN

	CToolWndList * A = m_pList;
	*success = FALSE;
	while(A)
	{
		if(A->m_hWnd == (HWND)hWnd)
		{
			*success = TRUE;
			CToolWndList * B = (CToolWndList*)A->GetChain();
			if(A == B)
				DELETE_COLLECTION(m_pList)
			else
			{
				m_pList = B;
				A->Remove();
			}
			break;
		}
		A = (CToolWndList*)A->Next;
	}

	VXPLIB_END
}

STDMETHODIMP CVXPWndToolList::Clear()
{
	VXPLIB_BEGIN

	DELETE_COLLECTION(m_pList)
	
	VXPLIB_END
}

STDMETHODIMP CVXPWndToolList::HasTool(long hWnd, VARIANT_BOOL *result)
{
	VXPLIB_BEGIN

	CToolWndList * A = m_pList;
	*result = FALSE;
	while(A)
	{
		if(A->m_hWnd == (HWND)hWnd)
		{
			*result = TRUE;
			break;
		}
		A = (CToolWndList*)A->Next;
	}

	VXPLIB_END
}

STDMETHODIMP CVXPWndToolList::IsActive(long hWnd, VARIANT_BOOL *result)
{
	VXPLIB_BEGIN

	CToolWndList * A = m_pList;
	*result = FALSE;
	while(A)
	{
		if(A->m_hWnd == (HWND)hWnd)
		{
			*result = A->m_bActive?TRUE:FALSE;
			break;
		}
		A = (CToolWndList*)A->Next;
	}

	VXPLIB_END
}

STDMETHODIMP CVXPWndToolList::SetActive(long hWnd, VARIANT_BOOL Active, VARIANT_BOOL *success)
{
	VXPLIB_BEGIN

	CToolWndList * A = m_pList;
	*success = FALSE;
	while(A)
	{
		if(A->m_hWnd == (HWND)hWnd)
		{
			A->m_bActive = Active?true:false;
			*success = TRUE;
			break;
		}
		A = (CToolWndList*)A->Next;
	}

	VXPLIB_END
}

HRESULT CVXPWndToolList::FinalConstruct()
{
	m_pList = NULL;
	return S_OK;
}

HRESULT CVXPWndToolList::FinalRelease()
{
	DELETE_COLLECTION(m_pList)
	return S_OK;
}

STDMETHODIMP CVXPWndToolList::get_Size(short *pVal)
{
	VXPLIB_BEGIN

	if(m_pList)
		*pVal = m_pList->CountAll();
	else
		*pVal = 0;

	VXPLIB_END
}

